import  { Component } from "react";
import {
    Route,
    Redirect,
} from 'react-router-dom';
import AuthStore from '../authenticate/auth.store';
import AuthUtil from "../utils/auth.util";

class ProtectedRoute extends Component{

    constructor(props){
        super(props)
        this.state={
            permission_set: AuthStore.getState()
        }
    }

    async componentDidMount() {
        const permission = await AuthUtil.getUserRolePermission();
        this.setState({permission_set: permission});
    }
    
    hasPermission() {
        const permission = this.props.permissions
        if(this.state.permission_set['userRolePermission']){
            if(permission.length <= 2) {
                if(permission[0] !== undefined && permission[1] !== undefined) {
                    if(this.state.permission_set['userRolePermission'][permission[0]] && this.state.permission_set['userRolePermission'][permission[0]][permission[1]]) {
                        return true;
                    } else {
                        return false;
                    }
                }
            }
        }
    }

    updatePageTracker(path) {
        let pageTracker = sessionStorage.getItem("pageTracker");
        pageTracker = pageTracker?JSON.parse(pageTracker):{};
        if (pageTracker.currPath !== path) {
            pageTracker.prevPath = pageTracker.currPath || path;
        }
        pageTracker.currPath = path;
        sessionStorage.setItem("pageTracker", JSON.stringify(pageTracker));
    }

    render() {
        const { name, component, path, permissions, location } = this.props;
        if(permissions) {
           if(this.hasPermission()) {
                this.updatePageTracker(location.pathname);
                return <Route path={path} name={name} component={component} />
            }
            else {
                return <Redirect to={{
                    pathname: '/access-denied',
                    state: { from: this.props.location }
                }}/> 
            }
        }
        else {
            this.updatePageTracker(location.pathname);
            return <Route path={path} name={name} component={component} />
        }
    }
}

export default ProtectedRoute;