import  React, { Component } from 'react';

import { Dialog } from 'primereact/dialog';
import { Button } from 'primereact/button';
import Flatpickr from "react-flatpickr";

import moment from 'moment';
import _ from 'lodash';
import UIConstants from '../../utils/ui.constants';
import UtilService from '../../services/util.service';

export default class BetweenEditor extends Component {
  constructor(props) {
    super(props);
    this.tmpRowData = [];
    this.state = {
      showDialog: false,
      dialogTitle: '',
      validationMsg: ''
    };
    this.validationMsg = '';
    this.copyDateValue = this.copyDateValue.bind(this);
    this.eventHandling = this.eventHandling.bind(this);
    this.validateDate = this.validateDate.bind(this);
  }
  
  isPopup() {
    return true;
  }

  /**
   * Init the date value if exists
   */
  async componentDidMount(){
    let parentRows = this.props.agGridReact.props.rowData[this.props.node.rowIndex];
    let parentCellData = parentRows[this.props.colDef.field];
    this.tmpRowData = [];
    this.props.colDef.suppressKeyboardEvent = params => {
      // return true (to suppress) if Tab key pressed
      const keyCode = params.event.keyCode;
      let gridShouldDoNothing = false;
      if (params.editing && (keyCode === 9)) {
          gridShouldDoNothing = true;
      }
      return gridShouldDoNothing;
    }

    if(parentCellData){
      let cellDataArray = _.split(parentCellData, '|');
      await cellDataArray.forEach(dataVal =>{
        let line = {};
        let dataValue = _.split(dataVal, ',');
        line['from'] = (dataValue[0])? moment(dataValue[0]).toDate():'';
        line['until'] = ( dataValue[1])? moment(dataValue[1]).toDate():'';
        this.tmpRowData.push(line);
      });
    }
    
    if(this.tmpRowData.length>0){
      let row = this.tmpRowData[this.tmpRowData.length-1];
      if((row['from'] !== '' && row['from'] !== 'undefined') && (row['until'] !== '' && row['until'] !== 'undefined')){
        let line = {'from': '', 'until': ''};
        this.tmpRowData.push(line);
      }
    } else{
      let line = {'from': '', 'until': ''};
      this.tmpRowData.push(line);
    }
    
    let systemTime = moment.utc(await UtilService.getUTC());

    this.tmpRowData = this.validateDate(this.tmpRowData);
    await this.setState({
      rowData: this.tmpRowData,
      dialogTitle: (this.props.colDef.field === 'between') ? this.props.colDef.field : 'Not-Between',
      showDialog: true,
      systemTime: systemTime,
      validationMsg: this.validationMsg
    });

  }

  /**
   * Call the function on click Esc or Close the dialog
   */
  async copyDateValue(){
      let consolidateDates = '';
      this.state.rowData.map(row =>{
          if(row['from'] && row['until'] && row['from'] !== '' && row['until'] !== ''){
            consolidateDates += (moment(row['from']).format(UIConstants.CALENDAR_DATETIME_FORMAT))+","+(moment(row['until']).format(UIConstants.CALENDAR_DATETIME_FORMAT))+"|";
          }
      });
      await this.props.context.componentParent.updateTime(
          this.props.node.rowIndex,this.props.colDef.field, consolidateDates, true, true
      );
      this.setState({ showDialog: false});
  }

  /*
  Set value in relevant field
  */
  async updateDateChanges(rowIndex, field, value){
      let tmpRows = this.state.rowData;
      let row = tmpRows[rowIndex];
      row[field] = value;      
      tmpRows[rowIndex] = row;
        if(value && this.state.rowData.length === rowIndex+1){
          let line = {'from': '', 'until': '', isValid: ''};
          tmpRows.push(line);
      }    
      tmpRows = await this.validateDate(tmpRows);
      this.setState({
        rowData: tmpRows, validationMsg: this.validationMsg
      });
  }

  /**
   * Validate from and until date for each row
   * @param {Date array} dateArray 
   * @returns 
   */
  validateDate(dateArray) {
    this.validationMsg = '';
      dateArray.forEach(bdate => {
        if (bdate['from'] !== '' && bdate['until'] !== '' && moment(new Date(bdate['from'])).isAfter(moment(new Date(bdate['until'])))) {
          this.validationMsg= 'From date cannot be less than Until date';
          bdate['isValid'] = false;
        } else  {
          bdate['isValid'] = true;
        }
      });
      return dateArray;
  }
  
  /*
    Remove the the row from dialog
  */
  removeInput(rowIndex){
      let tmpRows = this.state.rowData;
      delete tmpRows[rowIndex];
      tmpRows = this.validateDate(tmpRows);
      this.setState({
         rowData: tmpRows, validationMsg: this.validationMsg
    } );
  }

  /**
   * 
   * @param {Key Event} e - Key code
   */
   eventHandling(e) {
    var key = e.which || e.keyCode;
    if(key === 9) {
      //do nothing
    }    
  }

render() {  
  return (
    <div onKeyDown={this.eventHandling}>
    {this.state.rowData && this.state.rowData.length > 0 &&
      <Dialog header={_.startCase(this.state.dialogTitle)} visible={this.state.showDialog} maximized={false}  
          onHide={() => {this.copyDateValue()}} inputId="confirm_dialog"
          footer={
            <div style={{display: 'flex'}}>
              <div style={{width: '85%', textAlign: 'left'}}><label style={{color: 'red'}}>{this.state.validationMsg}</label></div>
              <div><Button key="back" label="Close" onClick={() => {this.copyDateValue()}} disabled={this.state.validationMsg.length > 0} /></div>
            </div>
        } >
          <div className="ag-theme-balham" style={{ maxHeight: '335px', minHeight: '200px', width: '500px', paddingLeft: '5px' }}>
            <div className="p-field p-grid" >
              <React.Fragment>
                <label key={'labelFrom'} className="col-lg-5 col-md-5 col-sm-12">From</label>
                <label key={'labelUntil'} className="col-lg-5 col-md-5 col-sm-12">Until</label>
                <label key={'labelRemove'} className="col-lg-2 col-md-2 col-sm-12" >Remove</label>
              </React.Fragment>
            </div>
            {this.state.rowData.map((bdate, index) => (
                <React.Fragment key={index}>
                  <div className="p-field p-grid" >
                      <Flatpickr data-enable-time data-input className="col-lg-5 col-md-5 col.sm-12"
                          options={{  "inlineHideInput": true,
                                      "wrap": true,
                                      "enableSeconds": true,
                                      "time_24hr": true,
                                      "minuteIncrement": 1,
                                      "allowInput": true,
                                      "defaultDate": this.state.systemTime?this.state.systemTime.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT):"",
                                      "defaultHour": this.state.systemTime?this.state.systemTime.hours():12,
                                      "defaultMinute": this.state.systemTime?this.state.systemTime.minutes():0,
                                  }}
                          title=""
                          value={bdate.from}
                          onChange={value => {this.updateDateChanges(index, 'from', value[0])}} 
                          >
                          <input type="text" data-input className={`p-inputtext p-component calendar-input`} />
                          <button className="p-button p-component p-button-icon-only calendar-button" data-toggle
                              title="Reset to the default date range" >
                              <i className="fas fa-calendar"></i>
                          </button>
                          <button className="p-button p-component p-button-icon-only calendar-reset" 
                              onClick={() => {this.updateDateChanges(index, 'from', null)}} 
                              title="Reset to the default date range" >
                              <i className="fas fa-sync-alt"></i>
                          </button>
                      </Flatpickr>
                      <Flatpickr data-enable-time data-input className="col-lg-5 col-md-5 col.sm-12"
                          options={{  "inlineHideInput": true,
                                      "wrap": true,
                                      "enableSeconds": true,
                                      "time_24hr": true,
                                      "minuteIncrement": 1,
                                      "allowInput": true,
                                      "defaultDate": this.state.systemTime?this.state.systemTime.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT):"",
                                      "defaultHour": this.state.systemTime?this.state.systemTime.hours():12,
                                      "defaultMinute": this.state.systemTime?this.state.systemTime.minutes():0,
                                  }}
                          title=""
                          value={this.state.rowData[index].until}
                          onChange={value => {this.updateDateChanges(index, 'until', value[0])}} 
                          >
                          <input type="text" data-input className={`p-inputtext p-component calendar-input`} />
                          <button className="p-button p-component p-button-icon-only calendar-button" data-toggle
                                  title="Reset to the default date range" >
                                  <i className="fas fa-calendar"></i>
                          </button>
                          <button className="p-button p-component p-button-icon-only calendar-reset" 
                              onClick={() => {this.updateDateChanges(index, 'until', null)}} 
                              title="Reset to the default date range" >
                              <i className="fas fa-sync-alt"></i>
                          </button>
                      </Flatpickr>                      
                        <div className="col-lg-1 col-md-1 col.sm-12" title={this.state.validationMsg}>
                          {!this.state.rowData[index].isValid &&
                            <i className="fa fa-exclamation-triangle" style={{color: 'red', marginLeft: '-15px'}} ></i> 
                          }
                        </div>
                        <div className="col-lg-1 col-md-1 col.sm-12" title='Delete date'>
                          {this.state.rowData.length !== (index+1) &&
                            <button className="p-link" onClick={() => this.removeInput(index)} style={{marginLeft: '-15px'}} >
                              <i className="fa fa-trash pi-error"></i></button>
                          }
                        </div>
                    </div>
                  </React.Fragment>
                  ))}
            </div>
      </Dialog>
    }
   </div>
  );
  } 
}