import  {useEffect, useState} from "react";
import UtilService from "../../services/util.service";
import moment from "moment";
import UIConstants from "../../utils/ui.constants";

import { ProgressSpinner } from 'primereact/progressspinner';
import Clock from 'react-live-clock';

function getDateOrTimeElement(title, element) {
    return <span className="date-element" data-testid={title + "-element"}>
        <span  className="date-title">{title}</span>
        {element}
    </span>;
}

export default function DateTimeInfo() {
    const [currentUTC, setCurrentUTC] = useState()
    const [currentLST, setCurrentLST] = useState()

    async function fetchUTC() {
        let utc = await UtilService.getUTC()
        setCurrentUTC(moment(utc))
    }
        useEffect(() => { // initializer
        if (!currentUTC) {
            fetchUTC().catch(e => console.error("Couldn't get current utc from the server", e))
        }
    }, [])

    useEffect(() => { //get LST after current utc is retrieved used . in the tmss ui, this is displayed on top
        async function fetchLST(currentUTC) {
            let lstreponse = await UtilService.getLST(currentUTC.format(UIConstants.UTC_DATE_TIME_FORMAT))
           setCurrentLST(moment(lstreponse.data?.LST?.[UIConstants.TIMINGSERVER],"HH:mm:ss"))
              }

        if (!currentLST && currentUTC) {
            fetchLST(currentUTC).catch(e => console.error("Couldn't get current lst from the server", e))
        }
    }, [currentUTC])

    if (!currentUTC || !currentLST) {
        return <div className="p-grid timeline-datetime-navigator">
            <ProgressSpinner className="m-4" style={{color: "var(--primary-300)"}}/>
        </div>
    }

    if (currentUTC.isValid() && currentLST.isValid()) {
        return <div className="datetime-info-container">
            {getDateOrTimeElement("Date", <span className="clock-time"
                                                title={currentUTC.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)}>
                    {currentUTC.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)}
                </span>)}
            {getDateOrTimeElement("UTC", <Clock className="clock-time"
                                                date={currentUTC.format(UIConstants.CALENDAR_DATETIME_FORMAT)}
                                                format="HH:mm:ss"
                                                interval={1000}
                                                ticking={true}/>)}
            {getDateOrTimeElement("LOFAR Core LST", <Clock className="clock-time"
                                                date={currentLST.format(UIConstants.CALENDAR_DATETIME_FORMAT)}
                                                format="HH:mm:ss"
                                                interval={1000}
                                                ticking={true}/>)}
        </div>
    } else {
        return <div className="datetime-info-container" data-testid="errored-datetime-info">Something went wrong in retrieving the UTC and LST</div>
    }
}