import React, { Component } from 'react';
import moment from 'moment';
import _ from 'lodash';

/**
 * Component to get report data and display them for cycle(s).
 */
class CycleReportIntro extends Component {
    
    constructor(props) {
        super(props);
        this.state = {
        };
    }

    componentDidMount() {
        // if (this.props.project) {
        //     this.loadProjectReport(this.props.project);
        // }
    }

    componentDidUpdate() {
        // if (this.state.project !== this.props.project) {
        //     this.loadProjectReport(this.props.project);
        // }
    }

    getCycleRange() {
        let cycleRanges = [];
        const cycles = this.props.cycles;
        if (cycles && cycles.length > 0) {
            let prevCycle = null;
            for (const cycle of cycles) {
                if (prevCycle) {
                    if (moment(cycle.start).diff(moment(prevCycle.stop), 'days') > 1) {
                        cycleRanges.push({start: cycle.name, stop: cycle.name});
                    }   else {
                        let cycleRange = cycleRanges[cycleRanges.length-1];
                        cycleRange.stop = cycle.name;
                        cycleRanges.splice(cycleRanges.length-1, cycleRange);
                    }
                }   else {
                    cycleRanges.push({start: cycle.name, stop: cycle.name});
                }
                prevCycle = cycle;
            }
        }
        let cycleRangeValues = _.map(cycleRanges, cycleRange=> {
                                    return cycleRange.start===cycleRange.stop?cycleRange.start:
                                    `${cycleRange.start} - ${cycleRange.stop}`});
        // return cycleRange;
        return cycleRangeValues.join(', ');
    }

    render() {
        return(
            <React.Fragment>
                <>
                <div className="report-div1" id={`intro-report-div`}>
                    <div id={`intro-cycle-details`}>
                        <h2 style={{textAlign: "center", marginBottom:"25px"}}>Report statistics for Cycles </h2>
                        <div className="p-grid">
                            <div className="col-lg-3 col-md-4 col-sm-12">
                                <label>Cycle Documentation</label>
                            </div>
                            <div className="col-lg-9 col-md-8 col-sm-12">
                                <a href="https://support.astron.nl/jira" target="_blank" rel="noreferrer">Link to Jira Ticket</a>
                            </div>
                            <div className="col-lg-3 col-md-4 col-sm-12">
                                <label>Cycle statistics over the period</label>
                            </div>
                            <div className="col-lg-8 col-md-9 col-sm-12">
                                <span>{this.props.reportPeriod?`${moment(this.props.reportPeriod[0]).format("MMM DD YYYY")} - ${moment(this.props.reportPeriod[1]).format("MMM DD YYYY")}`:"-"}</span>
                            </div>
                            <div className="col-lg-3 col-md-4 col-sm-12">
                                <label>Range of Cycles</label>
                            </div>
                            <div className="col-lg-8 col-md-9 col-sm-12">
                                <span>{this.getCycleRange()}</span>
                            </div>
                        </div>
                        <div style={{paddingLeft: "10px"}}>
                            <p>The following info is detailed in this report:</p>
                            <ul>
                                <li><a href="#Overview" >Cycle Overview</a>, that shows the overview of the resources used till date</li>
                                <li><a href="#TelescopeTime" >The telescope time distribution</a>, which shows the different categories of observations for the selected observing cycles</li>
                                <li><a href="#AverageEfficiency" >The average efficiency</a> matched with the expected or targeted or bench-marked average efficiency value for the cycle</li>
                                <li><a href="#CompletionLevel" >The completion level</a>, which shows the fraction of successful observations performed per the total number of observations earmarked for the cycle in question</li>
                                <li><a href="#CategoryObservation" >Observation hours per category</a>, which shows per single cycle the observing time fraction of successful Prio-A telescope hours, successful Prio-B telescope hours, System unavailability, Failed, System idle</li>
                                <li><a href="#WeeklyEfficiency" >The weekly efficiency</a>, which shows per single cycle fractions of observing hours of successful observations per week</li>
                                <li><a href="#DataIngestedPerSite" >Ingested data overview per site and category</a>, which shows per single cycle fractions of data products ingested per site and type (i.e. raw, (pre-)processed, interferometric, beamformed, tbb)</li>
                                <li><a href="#ProjectSummary" >Projects summary overview</a>, which shows a tabular overview of the telescope resources (successful(/failed) telescope hours, processing hours, storage space at the LTA site(s)) used by the projects in that cycle</li>
                                <li><a href="#ILT_LocalUsage" >The ILT vs stand alone  mode usage</a>, which shows the amount of hours spent in ILT-mode observing (i.e. the telescope is using the full array), ILT-mode idle / test (i.e. the full array is available but not performing production observations), Local-mode (i.e. the telescope array consist of Dutch station), ILT-mode (in total), ILT_mode_Dutch_array_observing (i.e. the the full array is available but performing production observations requiring only Dutch stations)</li>
                                <li><a href="#Failures" >The rate of failures</a>, which shows the rate of failures as a function of time</li>
                            </ul>
                        </div>
                    </div>
                    
                </div>
                </>
            </React.Fragment>
        );
    }

}

export default CycleReportIntro;