import React,{ Component } from "react";
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import UnitConverter from "../../../utils/unit.converter";

/**
 * Component to show overview of cycle.
 */
class CycleOverview extends Component{

    constructor(props){
        super(props);
    }

    /**
     * Returns the overview of resources used in cycle(s) formatted to display in overview table.
     * @returns Array of used resources of cycle
     */
    getReportData() {
        let reportData = [];
        const timeConversionFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
        const dataConversionFactor = UnitConverter.resourceUnitMap["bytes"].conversionFactor;
        this.props.data.map(repData => {
            const cycleProjects = repData.overview;
                let projectData = {}
                projectData['cycle'] = repData.cycle;
                projectData['observingTime'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.durations.total_observed_succeeded })/timeConversionFactor).toFixed(2);
                projectData['noOfTriggers'] = _.sumBy(cycleProjects, (projectSummary) => { return projectSummary.nr_of_used_triggers });
                projectData['processingTime'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.durations.total_CPU_time_used })/timeConversionFactor).toFixed(2);
                projectData['ltaStorage'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.LTA_dataproducts.size || 0 })/dataConversionFactor).toFixed(2);
                projectData['prioSuccessTime'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.durations.total_observed_succeeded_A })/timeConversionFactor).toFixed(2);
                projectData['failedTime'] = (_.sumBy(cycleProjects, (projectSummary) => { return projectSummary.durations.total_observed_failed })/timeConversionFactor).toFixed(2);
                projectData['completionLevel'] = repData.completionLevel.succeeded_perc || 0;
                reportData.push(projectData);
        });
        if (this.props.setExportData && reportData.length>0) {
            this.props.setExportData('Overview', reportData);
        }
        return reportData;
    }

    render() {
        return(
            <React.Fragment>
                <DataTable value={this.getReportData()} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em', textAlign: 'center'}}>
                    <Column field="cycle" header="Cycle Code"></Column>
                    <Column field="observingTime" header="Total used Observing time (hrs)"></Column>
                    <Column field="noOfTriggers" header="Total used Triggers"></Column>
                    <Column field="processingTime" header="Total used Processing time (hrs)"></Column>
                    <Column field="ltaStorage" header="Total used LTA storage space (TB)"></Column>
                    <Column field="prioSuccessTime" header="Amount of prio A/B hours successfully observed"></Column>
                    <Column field="failedTime" header="Amount of failed hours"></Column>
                    <Column field="completionLevel" header="Level of cycle completion (%)"></Column>
                </DataTable>
            </React.Fragment>
        )
    }

}

export default CycleOverview;