import React,{ Component } from "react";
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import UnitConverter from "../../../utils/unit.converter";

/**
 * Component to show projects summary in cycle.
 */
class CycleProjectSummary extends Component{

    /**
     * Returns the projects summary of the cycle(s) formatted to display in summary table.
     * @returns Array of Cycle Projects list
     */
    getReportData() {
        let reportData = [];
        const timeConversionFactor = UnitConverter.resourceUnitMap["time"].conversionFactor;
        const dataConversionFactor = UnitConverter.resourceUnitMap["bytes"].conversionFactor;
        this.props.data.map(repData => {
            const cycleProjects = repData.projects_summary;
            for (const projData of cycleProjects) {
                let projectData = {}
                projectData['cycle'] = repData.cycle;
                projectData['project'] = projData.project;
                projectData['durationObserved'] = (projData.durations.total_observed/timeConversionFactor).toFixed(2);
                projectData['durationProcessed'] = (projData.durations.total_CPU_time_used/timeConversionFactor).toFixed(2);
                const saraData = projData.data_ingested_per_site?_.find(projData.data_ingested_per_site, {"cluster_name":"Sara"}):null;
                projectData['dataToSara'] =  ((saraData?saraData.size:0)/dataConversionFactor).toFixed(2);
                const juelichData = projData.data_ingested_per_site?_.find(projData.data_ingested_per_site, {"cluster_name":"Juelich"}):null;
                projectData['dataToJuelich'] =  ((juelichData?juelichData.size:0)/dataConversionFactor).toFixed(2);
                const poznanData = projData.data_ingested_per_site?_.find(projData.data_ingested_per_site, {"cluster_name":"Poznan"}):null;
                projectData['dataToPoznan'] =  ((poznanData?poznanData.size:0)/dataConversionFactor).toFixed(2);
                reportData.push(projectData);
            }
            return repData;
        });
        if (this.props.setExportData && reportData.length>0) {
            this.props.setExportData('ProjectSummary', reportData);
        }
        return reportData;
    }

    render() {
        return(
            <React.Fragment>
                <DataTable value={this.getReportData()} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em', textAlign: 'center'}}>
                    <Column field="cycle" header="Cycle Code"></Column>
                    <Column field="project" header="Project"></Column>
                    <Column field="durationObserved" header="Time Observed (hr)"></Column>
                    <Column field="durationProcessed" header="Time Processed (hr)"></Column>
                    <Column field="dataToSara" header="Ingested To Sara data size (TB)"></Column>
                    <Column field="dataToJuelich" header="Ingested To Juelich data size (TB)"></Column>
                    <Column field="dataToPoznan" header="Ingested To Poznan data size (TB)"></Column>
                </DataTable>
            </React.Fragment>
        )
    }

}

export default CycleProjectSummary;