import React,{ Component } from "react";
import 'chart.js/auto';
import { Bar } from 'react-chartjs-2';
import _ from 'lodash';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import UnitConverter from "../../../utils/unit.converter";

/**
 * Component to show data ingested per site in cycle(s).
 */
class CycleSiteWiseData extends Component{

    constructor(props){
        super(props);

        this.state= {
            reportData:[]
        };
    }

    getBarData() {
        let reportData = this.getReportData();
        if (this.props.setExportData && reportData.length>0) {
            this.props.setExportData('DataIngestedPerSite', reportData);
        }
        return {
            labels: _.map(reportData, 'cycle'),
            datasets:[
                
                {   label: 'Poznan', 
                    data:_.map(reportData, 'poznan'),
                    backgroundColor: '#4472C4',
                    barThickness: 75,
                },
                {   label: 'Juelich', 
                    data:_.map(reportData, 'juelich'),
                    backgroundColor: '#ED7D31',
                    barThickness: 75,
                            
                },
                {   label: 'Sara', 
                    data:_.map(reportData, 'sara'),
                    backgroundColor: '#A5A5A5',
                    barThickness: 75,       
                }
            ]
        };
    }

    getBarOptions() {
        return {
            indexAxis: 'x',
            elements: {
              bar: {
                borderWidth: 0,
              },
            },
            scales: {
                x: {
                    stacked: true,
                    categoryPercentage: 1.0,
                    barPercentage: 1.0
                },
                y: {
                    stacked: true,
                    // max: 5000,
                }
            },
            responsive: true,
            plugins: {
              legend: {
                position: 'bottom',
              },
              title: {
                display: true,
                text: 'Ingested Data per Site (TB)',
              }
            }
        };
    }

    getReportData() {
        let reportData = [];
        const dataConversionFactor = UnitConverter.resourceUnitMap["bytes"].conversionFactor;
        this.props.data.map(repData => {
            let siteData = {}
            const cycleData = repData.data_ingested_per_site_and_category.sites;
            siteData['cycle'] = repData.cycle;
            const saraData = _.find(cycleData, {"cluster_name":"Sara"});
            siteData['sara'] =  ((saraData?saraData.size:0)/dataConversionFactor).toFixed(2);
            const juelichData = _.find(cycleData, {"cluster_name":"Juelich"});
            siteData['juelich'] =  ((juelichData?juelichData.size:0)/dataConversionFactor).toFixed(2);
            const poznanData = _.find(cycleData, {"cluster_name":"Poznan"});
            siteData['poznan'] =  ((poznanData?poznanData.size:0)/dataConversionFactor).toFixed(2);
            reportData.push(siteData);
        });

        // if (isTableData) {
        //     reportData.push({cycle: 'Average',
        //         poznan: MathUtility.getAverage(_.map(reportData, 'poznan')), 
        //         juelich: MathUtility.getAverage(_.map(reportData, 'juelich')), 
        //         sara: MathUtility.getAverage(_.map(reportData, 'sara'))
        //     });
        // }
        return reportData;
    }

    render() {
        return(
            <React.Fragment>
                <div  id={`cycles-site-data`}
                    style={{paddingTop: "10px", paddingBottom: "10px"}}>
                    <Bar data={this.getBarData()} options={this.getBarOptions()} width="50%" height="20" />
                </div>
                
                <div className="ttd-details" id={`cycle-ttd-details`}>
                    <DataTable value={this.getReportData()} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em', textAlign: 'center'}}>
                        <Column field="cycle" header="Cycle Code"></Column>
                        <Column field="sara" header="To Sara (TB)"></Column>
                        <Column field="juelich" header="To Juelich (TB)"></Column>
                        <Column field="poznan" header="To Poznan (TB)"></Column>
                    </DataTable>
                </div>

            </React.Fragment>
        )
    }

}

export default CycleSiteWiseData;