import React, { Component } from 'react'
import 'primeflex/primeflex.css';
import { Chips } from 'primereact/chips';
import { Link } from 'react-router-dom';

import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';

import AppLoader from "./../../layout/components/AppLoader";
import PageHeader from '../../layout/components/PageHeader';
import ViewTable from './../../components/ViewTable';
import Jeditor from '../../components/JSONEditor/JEditor';
import ScheduleService from '../../services/schedule.service';
import moment from 'moment';
import _ from 'lodash';
import $RefParser from "@apidevtools/json-schema-ref-parser";
import SchedulingConstraint from './Scheduling.Constraints';
import { Dialog } from 'primereact/dialog';
import TaskStatusLogs from '../Task/state_logs';
import Stations from './Stations';
import { Redirect } from 'react-router-dom';
import { CustomDialog } from '../../layout/components/CustomDialog';
import { CustomPageSpinner } from '../../components/CustomPageSpinner';
import { appGrowl } from '../../layout/components/AppGrowl';
import Schedulingtaskrelation from './Scheduling.task.relation';
import UnitConverter from '../../utils/unit.converter';
import TaskService from '../../services/task.service';
import UIConstants from '../../utils/ui.constants';
import UtilService from '../../services/util.service';
import ParserUtility from '../../utils/parser.utility';
import AuthUtil from '../../utils/auth.util';
import AccessDenied from '../../layout/components/AccessDenied';
import AuthStore from '../../authenticate/auth.store';
import { TriStateCheckbox } from 'primereact/tristatecheckbox';
import Websocket from 'react-websocket';
import { InputText } from 'primereact/inputtext';
import { InputTextarea } from 'primereact/inputtextarea';
import { Dropdown } from 'primereact/dropdown';
import ProjectService from '../../services/project.service';
import IngestProgress from './ingest.progress';


class ViewSchedulingUnit extends Component {
    lsKeySortColumn = 'SortDataViewSchedulingUnit';
    defaultSortColumn = [];
    ignoreSorting = ['status logs'];
    SU_NOT_STARTED_STATUSES = ['defined', 'schedulable', 'scheduled', 'unschedulable'];
    SU_ACTIVE_STATUSES = ['started', 'observing', 'observed', 'processing', 'processed', 'ingesting', 'ingested'];
    SU_END_STATUSES = ['finished', 'error', 'cancelled'];
    TASK_END_STATUSES = ['finished', 'error', 'cancelled'];
    SU_BLUEPRINT_EXPAND= 'draft.scheduling_constraints_template,draft,draft.scheduling_set,task_blueprints.specifications_template,task_blueprints,task_blueprints.subtasks,draft.observation_strategy_template,task_blueprints.draft, scheduling_unit_blueprints'
    SU_BLUEPRINT_FIELDS= ['id','url','created_at','status', 'unschedulable_reason', 'tags','output_pinned','duration','name','on_sky_start_time','on_sky_stop_time','scheduling_constraints_doc','description',
                            'updated_at','draft.url','draft.id','draft.name','draft.scheduling_set.id','draft.scheduling_set.url','draft.scheduling_set.name','draft.scheduling_set.project_id','task_blueprints.status', 'task_blueprints.unschedulable_reason',
                            'draft.rank','draft.priority_queue_value','draft.priority_queue','task_blueprints.subtasks.id','task_blueprints.subtasks.primary', 'task_blueprints.subtasks.total_nr_of_dataproducts', 'task_blueprints.subtasks.total_size_of_dataproducts', 'task_blueprints.subtasks.total_size_of_dataproducts_on_filesysem','task_blueprints.subtasks.specifications_template_id',
                            'task_blueprints.task_type','task_blueprints.id','task_blueprints.subtasks_ids','task_blueprints.name','task_blueprints.description',
                            'task_blueprints.short_description','task_blueprints.on_sky_start_time','task_blueprints.on_sky_stop_time','task_blueprints.process_start_time',
                            'task_blueprints.process_stop_time','task_blueprints.duration','task_blueprints.relative_start_time','task_blueprints.relative_stop_time','task_blueprints.tags', 'task_blueprints.url',
                            'task_blueprints.do_cancel','task_blueprints.obsolete_since','task_blueprints.created_at','task_blueprints.updated_at','task_blueprints.specifications_template.id', 'task_blueprints.draft_id',
                            'task_blueprints.specifications_template.type_value','task_blueprints.specifications_template.schema',
                            'draft.scheduling_constraints_template.schema','draft.scheduling_constraints_template.ref_resolved_schema','draft.scheduling_constraints_template.url',
                            'task_blueprints.produced_by_ids','task_blueprints.specifications_doc', 'task_blueprints.draft',
                            'draft.observation_strategy_template_id','draft.observation_strategy_template.id','draft.observation_strategy_template.name',
                            'draft.observation_strategy_template.purpose_value','draft.observation_strategy_template.state_value','draft.observation_strategy_template.version',
                            'draft.observation_strategy_template.description', 'draft.observation_strategy_template.template']
    SU_DRAFT_EXPAND= 'scheduling_constraints_template,scheduling_set,task_drafts.specifications_template,task_drafts,observation_strategy_template,scheduling_unit_blueprints'
    SU_DRAFT_FIELDS=['id','url','created_at','status','tags','output_pinned','duration','name','on_sky_start_time','on_sky_stop_time','rank','priority_queue','priority_queue_value','description',
                        'scheduling_constraints_doc','scheduling_constraints_template.schema','scheduling_constraints_template.ref_resolved_schema','scheduling_constraints_template.url',
                        'observation_strategy_template_id','task_drafts.url','scheduling_constraints_template_id', 'task_drafts.connector_types', 
                        'updated_at','scheduling_set.id','scheduling_set.url','scheduling_set.name','scheduling_set.project_id','task_drafts.status','task_drafts.task_type','task_drafts.id','task_drafts.subtasks_ids',
                        'task_drafts.name','task_drafts.description','task_drafts.short_description','task_drafts.on_sky_start_time','task_drafts.on_sky_stop_time','task_drafts.process_start_time',
                        'task_drafts.process_stop_time','task_drafts.duration','task_drafts.relative_start_time','task_drafts.relative_stop_time','task_drafts.tags','task_drafts.do_cancel',
                        'task_drafts.obsolete_since','task_drafts.created_at','task_drafts.updated_at','task_drafts.specifications_template.id','task_drafts.specifications_template.type_value','task_drafts.specifications_template.schema',
                        'task_drafts.task_blueprints_ids','task_drafts.specifications_doc','task_drafts.produced_by_ids', 'task_drafts.output_pinned','scheduling_unit_blueprints_ids', 
                        'observation_strategy_template.id','observation_strategy_template.name','observation_strategy_template.description','observation_strategy_template.template', 'observation_strategy_template.scheduling_unit_template',
                        'observation_strategy_template.purpose_value','observation_strategy_template.state_value','observation_strategy_template.version', 'scheduling_unit_blueprints.url',
                        'scheduling_unit_blueprints.id','scheduling_unit_blueprints.name', 'scheduling_unit_blueprints.url']
                    
                        
    constructor(props) {
        super(props);   
        this.setToggleBySorting();
        this.state = {
            scheduleunit: null,
            schedulingUnitTasks: [],
            isLoading: true,
            showStatusLogs: false,
            showTaskRelationDialog: false,
            paths: [{
                "View": "/task",
            }],
            ingestGroup: {},
            missingStationFieldsErrors: [],
            columnOrders: [
                "Status Logs",
                "Status",
                "Type",
                "ID",
                "Control ID",
                "Name",
                "Task Type",
                "Description",
                "Short Description",
                "Observation Start Time",
                "Observation End Time",
                "Process Start Time",
                "Process End Time",
                "Duration (HH:mm:ss)",
                "Relative Start Time (HH:mm:ss)",
                "Relative End Time (HH:mm:ss)",
                "#Dataproducts",
                "size",
                "dataSizeOnDisk",
                "subtaskContent",
                "tags",
                "blueprint_draft",
                "url",
                "Cancelled",
                "Obsolete",
                "Created at",
                "Updated at"
            ],
            defaultcolumns: [{
                status_logs: {
                    name: "Status Logs",
                    filter: 'none'
                },
                status: {
                    name: "Status",
                    filter: "select",
                    showTag: true,
                    tagClass: 'su-$dynamic$' //$dynamic$ will be replace with cell value
                },
                unschedulable_reason: {
                    name:"Unschedulable Reason",    
                },
                tasktype: {
                    name: "Type",
                    filter: "none"
                },
                id: "ID",
                subTaskID: 'Control ID',
                name: "Name",
                description: "Description",
                short_description: "Short Description",
                on_sky_start_time: {
                    name: "Observation Start Time",
                    filter:"flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                on_sky_stop_time: {
                    name: "Observation End Time",
                    filter:"flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                process_start_time: {
                    name: "Process Start Time",
                    filter:"flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                process_stop_time: {
                    name: "Process End Time",
                    filter:"flatpickrDateRange",
                    format:UIConstants.CALENDAR_DATETIME_FORMAT
                },
                duration: {
                    name: "Duration (HH:mm:ss)",
                    format: UIConstants.CALENDAR_TIME_FORMAT
                },
                relative_start_time: "Relative Start Time (HH:mm:ss)",
                relative_stop_time: "Relative End Time (HH:mm:ss)",
                noOfOutputProducts: "#Dataproducts",
                obsolete: {
                    name:"Obsolete",
                    filter: "switch"
                },
                do_cancel: {
                    name: "Cancelled",
                    filter: "switch"
                },
            }],
            optionalcolumns: [{
                task_type: {
                    name: "Task Type",
                    filter: "select"
                },
                size: "Data size",
                dataSizeOnDisk: "Data size on Disk",
                subtaskContent: "Subtask Content",
                tags: "Tags",
                blueprint_draft: this.props.match.params.type==="draft"?"Linked Blueprint": "Linked Draft",
                url: {
                    name: "API URL",
                    newTab: true,
                },
                created_at: {
                    name: "Created at",
                    filter: "date",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                updated_at: {
                    name: "Updated at",
                    filter: "date",
                    format: UIConstants.CALENDAR_DATETIME_FORMAT
                },
                actionpath: "actionpath"
            }],
            columnclassname: [{
                "Status Logs": "filter-input-0",
                "Type": "filter-input-75",
                "ID": "filter-input-50",
                "Control ID": "filter-input-75",
                "Cancelled": "filter-input-50",
                "Obsolete": "filter-input-50",
                "Duration (HH:mm:ss)": "filter-input-75",
                "Template ID": "filter-input-50",
                "Observation Start Time":"filter-input-150",
                "Observation End Time":"filter-input-150",
                "Process Start Time":"filter-input-150",
                "Process End Time":"filter-input-150",
                // "BluePrint / Task Draft link": "filter-input-100",
                "Relative Start Time (HH:mm:ss)": "filter-input-75",
                "Relative End Time (HH:mm:ss)": "filter-input-75",
                "Status": "filter-input-100",
                "#Dataproducts": "filter-input-75",
                "Data size": "filter-input-50",
                "Data size on Disk": "filter-input-50",
                "Subtask Content": "filter-input-75",
                "Linked Blueprint": "filter-input-50",
                "Linked Draft": "filter-input-50"
            }],
            obsoleteTasks:[],
            stationGroup: [],
            stationGroups: {},
            stationsCollapsed: false,
            dialog: { header: 'Confirm', detail: 'Do you want to create a Scheduling Unit Blueprint? The Scheduling Unit may be scheduled immediately.' },
            dialogVisible: false,
            actions: [],
            dataformat: ['MeasurementSet'],
            taskStatus: [],
            permissionById: {},
            userPermission: {permissions: AuthStore.getState().userRolePermission},
            showAddIngestDialog : false
        }
        this.taskFilters = [];                      // To get Short_Description details
        this.taskTemplates = [];                    // All task templates to be filtered based on tasks in selected strategy template
        this.statusList = [];
        this.blueprintTaskTypeList = [];
        this.access_denied_message = "Don't have permission";
        this.actions = [];
        this.stations = [];
        this.constraintTemplates = [];
        this.selectedRows = [];
        this.ingestTaskRelations = []; // Holds the task relations of tasks with ingest task
        this.dialogWidth = "40vw";
        this.dialogType = "";
        this.dialogHeight = 'auto';
        this.dialogHeader = "";
        this.dialogMsg = "";
        this.dialogContent = "";
        this.statusUpdate = null;
        this.copyScheduleUnitID = null;
        this.pageUpdated = true;
        this.iconStyle = {marginLeft: '8px'}
        this.confirmObsoleteTasks =  this.confirmObsoleteTasks.bind(this);
        this.markObsoleteTasks =  this.markObsoleteTasks.bind(this);
        this.getObsTaskConfirmDlgContent =  this.getObsTaskConfirmDlgContent.bind(this);
        this.getObsTaskStatusContent =  this.getObsTaskStatusContent.bind(this);
        this.confirmDeleteTasks = this.confirmDeleteTasks.bind(this);
        this.confirmCancelTasks = this.confirmCancelTasks.bind(this);
        this.setEditorFunction = this.setEditorFunction.bind(this);
        this.onRowSelection = this.onRowSelection.bind(this);
        this.deleteTasks = this.deleteTasks.bind(this);
        this.deleteSchedulingUnit = this.deleteSchedulingUnit.bind(this);
        this.getTaskDeleteDialogContent = this.getTaskDeleteDialogContent.bind(this);
        this.getTaskCancelConfirmContent = this.getTaskCancelConfirmContent.bind(this);
        this.getTaskCancelStatusContent = this.getTaskCancelStatusContent.bind(this);
        this.getSUDialogContent = this.getSUDialogContent.bind(this);
        this.checkAndCreateBlueprint = this.checkAndCreateBlueprint.bind(this);
        this.confirmCopyingSU = this.confirmCopyingSU.bind(this);
        this.getSUCopyStatusContent = this.getSUCopyStatusContent.bind(this);
        this.copySchedulingUnit = this.copySchedulingUnit.bind(this);
        this.navigateToCopiedSUPage = this.navigateToCopiedSUPage.bind(this);
        this.createBlueprintTree = this.createBlueprintTree.bind(this);
        this.closeDialog = this.closeDialog.bind(this);
        this.showTaskRelationDialog = this.showTaskRelationDialog.bind(this);
        this.showDeleteSUConfirmation = this.showDeleteSUConfirmation.bind(this);
        this.showCancelSUConfirmation = this.showCancelSUConfirmation.bind(this);
        this.setAutoDeletion = this.setAutoDeletion.bind(this);
        this.cancelSchedulingUnit = this.cancelSchedulingUnit.bind(this);
        this.cancelTasks = this.cancelTasks.bind(this);
        this.getStatusContent = this.getStatusContent.bind(this);
        this.getStatusForCopiedShedulingUnit = this.getStatusForCopiedShedulingUnit.bind(this);
        this.createSUCleanupTask = this.createSUCleanupTask.bind(this);
        this.addNewIngestTask = this.addNewIngestTask.bind(this);
        this.prepareIngestTask = this.prepareIngestTask.bind(this);
        this.confirmAddIngestTask = this.confirmAddIngestTask.bind(this);
        this.closeIngestDialog = this.closeIngestDialog.bind(this);
        this.getCleanUpDialogContent = this.getCleanUpDialogContent.bind(this);
        this.cleanUpSUTask = this.cleanUpSUTask.bind(this);
        this.confirmAutoDeletion = this.confirmAutoDeletion.bind(this);
        this.cancelView = this.cancelView.bind(this);
        this.getFilterOptions = this.getFilterOptions.bind(this);
        this.confirmCopyFailedTasks = this.confirmCopyFailedTasks.bind(this);
        this.getCopyFailedTasksDlgContent = this.getCopyFailedTasksDlgContent.bind(this);
        this.copyFailedTasks = this.copyFailedTasks.bind(this);
        this.confirmUpdateSUBlueprint = this.confirmUpdateSUBlueprint.bind(this);
        this.updateSUB = this.updateSUB.bind(this);
        this.confirmCopySpecAndFailedTasks = this.confirmCopySpecAndFailedTasks.bind(this);
        this.copySpecAndFailedTasks = this.copySpecAndFailedTasks.bind(this);
        this.onConnect = this.onConnect.bind(this);
        this.handleData = this.handleData.bind(this);
        this.refreshSchedulingUnit = this.refreshSchedulingUnit.bind(this);
        this.getSchedulingUnitDetails = this.getSchedulingUnitDetails.bind(this);
        this.showAdvancedCopyDialog = this.showAdvancedCopyDialog.bind(this);
        this.getAdvancedSUCopyContent = this.getAdvancedSUCopyContent.bind(this);
        this.copyAdvancedSchedulingUnit = this.copyAdvancedSchedulingUnit.bind(this); 
        this.getAdvancedSUCopyStatusContent = this.getAdvancedSUCopyStatusContent.bind(this);
        this.showTaskRestartConfirmation = this.showTaskRestartConfirmation.bind(this);
        this.getTaskRestartConfirmContent = this.getTaskRestartConfirmContent.bind(this);
        this.restartTask = this.restartTask.bind(this);
        this.getTaskRestartStatusContent = this.getTaskRestartStatusContent.bind(this);
        this.confirmChangeSUSet = this.confirmChangeSUSet.bind(this);
        this.getSUSetContent = this.getSUSetContent.bind(this);
        this.changeSUSet = this.changeSUSet.bind(this);
    }

    componentDidUpdate() {
        if (this.state.scheduleunit && this.props.match.params &&
            (this.state.scheduleunitId !== this.props.match.params.id ||
                this.state.scheduleunitType !== this.props.match.params.type)) {
            this.getSchedulingUnitDetails(this.props.match.params.type, this.props.match.params.id);
        }
    }

    showTaskRelationDialog() {
        this.setState({ showTaskRelationDialog: !this.state.showTaskRelationDialog });
    }

    /**
     * Get Status list for UI drop downl
     */
    async getStatusList() {
        const taskFilters = await TaskService.getTaskFilterDefinition('blueprint');
        if (taskFilters && taskFilters.data.filters['status']) {
            taskFilters.data.filters['status'].choices.forEach(choice => {
                this.statusList.push(choice.value);
            })
        }
    }

    /**
     * Get Task type list for filter
     * @param {Array} suFilters 
     */
    async  getBlueprintTaskTypeList() {
        const taskFilters = await TaskService.getTaskFilterDefinition('blueprint');
        if (taskFilters && taskFilters.data.filters['task_type']) {
            taskFilters.data.filters['task_type'].choices.forEach(choice => {
                this.blueprintTaskTypeList.push(choice.value);
            });
        }
    }

    /**
     * Get Option-list values for Select Dropdown filter in 'Viewtable'
     * @param {String} id : Column id
     * @returns 
     */
     getFilterOptions(id) {
        let options = null;
        if(id && id === 'Status') {
            options = this.statusList;
        }   else if (id && id === 'Task Type') {
            options = this.blueprintTaskTypeList;
        }
        return options;
    }

    componentDidMount() {
        this.init();
    }

    async init() {
        this.getStatusList();
        this.getBlueprintTaskTypeList();
        let permission = (await AuthUtil.getUserRolePermission());
        permission = { permissions: permission.userRolePermission};
        this.taskFilters = await TaskService.getTaskFilterDefinition('draft');
        this.taskTemplates = await TaskService.getTaskTemplates();
        this.pageUpdated = true;
        this.setToggleBySorting();
        let schedule_id = this.props.match.params.id;
        let schedule_type = this.props.match.params.type;
        this.setState({userPermission: permission, schedule_id: schedule_id});
        if (schedule_type && schedule_id) {
            this.stations = await ScheduleService.getStationGroup();
            this.setState({ stationOptions: this.stations });
            this.subtaskTemplates = await TaskService.getSubtaskTemplates();
            this.getSchedulingUnitDetails(schedule_type, schedule_id);
        }
    }

    cancelView(){
        this.props.history.length>1?this.props.history.goBack():this.props.history.push(`/schedulingunit`);
    }

    toggleBySorting = (sortData) => {
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: sortData });
    }

    setToggleBySorting() {
        let sortData = UtilService.localStore({ type: 'get', key: this.lsKeySortColumn });
        if (sortData) {
            if (Object.prototype.toString.call(sortData) === '[object Array]') {
                this.defaultSortColumn = sortData;
            }
            else {
                this.defaultSortColumn = [{ ...sortData }];
            }
        }
        this.defaultSortColumn = this.defaultSortColumn || [];
        UtilService.localStore({ type: 'set', key: this.lsKeySortColumn, value: [...this.defaultSortColumn] });
     }

    subtaskComponent = (task) => {
        return (
            <button className="p-link" onClick={() => { this.setState({ showStatusLogs: true, task: task }) }}>
                <i className="fa fa-history"></i>
            </button>
        );
    };

    getSchedulingUnitDetails(schedule_type, schedule_id) {
        if (schedule_id === 0) {
            return;
        }
        let expand = schedule_type.toLowerCase() === 'draft' ? this.SU_DRAFT_EXPAND: this.SU_BLUEPRINT_EXPAND;
        let fields = schedule_type.toLowerCase() === 'draft' ? this.SU_DRAFT_FIELDS: this.SU_BLUEPRINT_FIELDS;
        ScheduleService.getExpandedSchedulingUnit(schedule_type, schedule_id,expand,fields)
            .then(async (schedulingUnit) => {
                if (schedulingUnit) {
                    const permissionById = await AuthUtil.getUserPermissionByModuleId(schedule_type === 'blueprint'? 'scheduling_unit_blueprint': 'scheduling_unit_draft', schedule_id)
                    this.projectSchedSets = await ScheduleService.getSchedulingSets(`project=${encodeURIComponent(schedule_type === 'blueprint'?schedulingUnit.draft.scheduling_set.project_id:schedulingUnit.scheduling_set.project_id)}`)
                    this.setState({permissionById: permissionById[schedule_id], scheduleunitType: schedule_type,});
                    this.schedulingConstraintDocRaw = _.cloneDeep(schedulingUnit.scheduling_constraints_doc);
                    schedulingUnit = this.formatConstraintDocForUI(schedulingUnit);
                    if (schedulingUnit.draft) {
                        schedulingUnit['observation_strategy_template_id'] = schedulingUnit.draft.observation_strategy_template_id;
                        schedulingUnit['observation_strategy_template'] = { name: schedulingUnit.draft.observation_strategy_template.name,
                                                                            description: schedulingUnit.draft.observation_strategy_template.description,
                                                                            purpose_value: schedulingUnit.draft.observation_strategy_template.purpose_value,
                                                                            state_value: schedulingUnit.draft.observation_strategy_template.state_value,
                                                                            version: schedulingUnit.draft.observation_strategy_template.version
                                                                        };
                        schedulingUnit['scheduling_set'] = schedulingUnit.draft.scheduling_set;
                        schedulingUnit['scheduling_constraints_template'] = schedulingUnit.draft.scheduling_constraints_template;
                        schedulingUnit['project'] = schedulingUnit.draft.scheduling_set.project_id;
                        schedulingUnit['scheduling_set_id'] = schedulingUnit.draft.scheduling_set.id
                    }
                    let tasks = schedulingUnit.task_drafts ? (await this.getFormattedTaskDrafts(schedulingUnit)) :await this.getFormattedTaskBlueprints(schedulingUnit);
                    if (schedulingUnit.task_drafts && tasks) {
                        const totalTasks = schedulingUnit.task_drafts.length;
                        const pinnedTasks = _.filter(schedulingUnit.task_drafts, {'output_pinned': true});
                        schedulingUnit['project'] = schedulingUnit.scheduling_set.project_id;
                        schedulingUnit['scheduling_set_id'] = schedulingUnit.scheduling_set.id
                        schedulingUnit['output_pinned'] = pinnedTasks.length === 0?false:(totalTasks === pinnedTasks.length || null);
                    }
                    this.setState({ scheduleunitId: schedule_id,
                                    scheduleunit: schedulingUnit,
                                    scheduleunitType: schedule_type,
                                    constraintTemplate: schedulingUnit.scheduling_constraints_template})
                   
                    let ingestGroup;
                    if(this.props.match.params.type === 'draft') {
                        ingestGroup = tasks.map(task => ({ name: task.name, 
                            connectors : _.filter(task.connector_types, connector => {
                                return connector.iotype_value === 'output'}),
                            consumers: task.consumers || [],
                            type_value: task.type_value, id: task.id,
                        }));
                       ingestGroup = _.groupBy(_.sortBy(ingestGroup, ['type_value','name']), 'type_value');   
                    } else if(this.props.match.params.type === 'blueprint') {
                        ingestGroup = tasks.map(task => ({ name: task.name, canRemove: true, 
                            connectors : _.filter(task.draft.connector_types, connector => {
                                return connector.iotype_value === 'output'}),
                            consumers: task.consumers || [], isObsolete: task.obsolete_since!==null,
                            type_value: task.task_type, id: task.id }));
                       ingestGroup = _.groupBy(_.sortBy(ingestGroup, ['type_value','name']), 'type_value');
                    }                      
                    await Promise.all(tasks.map(async task => {
                        task.status_logs = task.tasktype === "Blueprint" ? this.subtaskComponent(task) : "";
                        //Displaying SubTask ID of the 'control' Task
                        const subTaskIds = task.subTasks ? task.subTasks.filter(sTask => sTask.primary === true) : [];
                        task.size = 0;
                        task.dataSizeOnDisk = 0;
                        task.obsolete= task.obsolete_since!==null;
                        task.noOfOutputProducts = 0;
                        task.canSelect = task.tasktype.toLowerCase() === 'blueprint' ? true : (task.tasktype.toLowerCase() === 'draft' && (task.task_blueprints_ids && task.task_blueprints_ids.length === 0)) ? true : false;
                        if (task.tasktype.toLowerCase() === 'blueprint' && task.subTasks.length) {
                            task.noOfOutputProducts = _.sumBy(task.subTasks, 'total_nr_of_dataproducts');
                            task.size = _.sumBy(task.subTasks, 'total_size_of_dataproducts')
                            task.dataSizeOnDisk = _.sumBy(task.subTasks, 'total_size_of_dataproducts_on_filesysem')
                            task.size = UnitConverter.getUIResourceUnit('bytes', (task.size));
                            task.dataSizeOnDisk = UnitConverter.getUIResourceUnit('bytes', (task.dataSizeOnDisk));
                        }
                        task.subTaskID = subTaskIds.length ? subTaskIds[0].id : '';
                        return task;
                    }));

                    const targetObservation = _.find(tasks, (task) => { 
                                                        return task.template.type_value === 'observation' 
                                                                && task.tasktype.toLowerCase() === schedule_type 
                                                                && task.specifications_doc.station_configuration?.station_groups });
                    const isIngestPresent = _.find(tasks, (task) => { return task.template.type_value === 'ingest'});   
                    await this.getFilterColumns(this.props.match.params.type.toLowerCase());
                                        
                    this.setState({
                        scheduleunitId: schedule_id,
                        scheduleunit: schedulingUnit,
                        scheduleunitType: schedule_type,
                        schedulingUnitTasks: tasks,
                        isLoading: false,
                        stationGroup: targetObservation ? targetObservation.specifications_doc.station_configuration?.station_groups : [],
                        stationsCollapsed: false,
                        redirect: null,
                        dialogVisible: false,
                        isIngestPresent: isIngestPresent,
                        ingestGroup,
                        showTaskRelationDialog: this.state.openTaskRelationDialog,
                        openTaskRelationDialog: false
                    });
                    this.loadTaskParameters(schedulingUnit.draft?schedulingUnit.draft.observation_strategy_template: schedulingUnit.observation_strategy_template);
                    this.selectedRows = [];
                    // Add Action menu
                    this.getActionMenu(schedule_type, isIngestPresent);
                } else {
                    this.setState({
                        isLoading: false,
                        redirect: "/not-found"
                    });
                }
            });
    }
    
    /**
     * Format constraint field value for UI
     * @param {*} scheduling_constraints_doc 
     * @returns 
     */
    formatConstraintDocForUI(scheduleunit) {
        if (scheduleunit.scheduling_constraints_doc && !scheduleunit.scheduling_constraints_doc.sky.transit_offset.fromoffset) {
            scheduleunit.scheduling_constraints_doc.sky.transit_offset.from = (scheduleunit.scheduling_constraints_doc.sky.transit_offset.from<0?'-':'')+UnitConverter.getSecsToHHmmss(scheduleunit.scheduling_constraints_doc.sky.transit_offset.from);            
            scheduleunit.scheduling_constraints_doc.sky.transit_offset.to = (scheduleunit.scheduling_constraints_doc.sky.transit_offset.to<0?'-':'')+UnitConverter.getSecsToHHmmss(scheduleunit.scheduling_constraints_doc.sky.transit_offset.to);
        }
        return scheduleunit;
    }
    
    /**
     * To get task parameters from observation strategy to create custom json schema and load with existing value of the task as input 
     * parameters to pass to the JSON editor.
     */
    async loadTaskParameters(observStrategy) {
        if (observStrategy) {
            ParserUtility.addStationParameters(observStrategy);
            const tasks = observStrategy.template.tasks;
            const parameters = observStrategy.template.parameters;
            let paramsOutput = {};
            let schema = { type: 'object', additionalProperties: false, 
                            properties: {}, definitions:{}
                            };
            let bandPassFilter = null;
            let stationGroups = {};
            const $strategyRefs = await $RefParser.resolve(observStrategy.template);
            for (const param of parameters) {
                // TODO: make parameter handling more generic, instead of task specific.
                if (!param.refs[0].startsWith("#/tasks/")) { continue; }
                let taskPaths = param.refs[0].split("/");
                const taskName = taskPaths[2];
                /**
                 * For Short_Description, the task path length will be 4, so added below condition to get short_description details
                 *  #/tasks/Combined Observation/short_description
                 */
                taskPaths = taskPaths.slice((taskPaths.length===4?3:4), taskPaths.length);
                const task = tasks[taskName];
                const suTask = this.state.schedulingUnitTasks.find(taskD => taskD.name === taskName);
                if (suTask) { task.specifications_doc = suTask.specifications_doc;
                    task.short_description = suTask.short_description;
                    const taskKeys = Object.keys(task);
                    for (const taskKey of taskKeys) {
                        if (taskKey !== 'specifications_template') {
                            task[taskKey] = suTask[taskKey];
                        }
                    }
                }
                if (task) {
                    if (param.refs[0].includes("station_groups")) {
                        stationGroups[param.name] = task.specifications_doc.station_configuration.station_groups;
                    }   else {
                    
                        const taskTemplate = _.find(this.taskTemplates, {'name': task.specifications_template.name,
                                                                        'version': task.specifications_template.version || 1});

                        // Get the default Bandpass filter and pass to the editor for frequency calculation from subband list
                        if (taskTemplate.type_value === 'observation' && task.specifications_doc.station_configuration?.filter) {
                            bandPassFilter = task.specifications_doc.station_configuration?.filter;
                        }   else if (taskTemplate.type_value === 'observation' && taskTemplate.ref_resolved_schema.properties.filter) {
                            bandPassFilter = taskTemplate.ref_resolved_schema.properties.filter.default;
                        }
                        schema.definitions = {...schema.definitions, ...taskTemplate.ref_resolved_schema.definitions};
                        taskPaths.reverse();
                        const paramProp = await ParserUtility.getParamProperty($strategyRefs, taskPaths, taskTemplate.ref_resolved_schema, this.taskFilters);
                        schema.properties[param.name] = _.cloneDeep(paramProp);
                        if (schema.properties[param.name]) {
                            schema.properties[param.name].title = param.name;
                            try { schema.properties[param.name].default = $strategyRefs.get(param.refs[0]); } catch(err) { /* empty */ }
                            paramsOutput[param.name] = schema.properties[param.name].default; 
                        }
                    }
                }
            }
            this.setState({paramsSchema: schema, paramsOutput: paramsOutput, 
                            stationGroups: stationGroups, bandPassFilter: bandPassFilter });
        }
    }

    async getFilterColumns(type) {
        const columnOrderToRemove = type === 'blueprint' ?[] :['Status', 'Unschedulable Reason', 'Status Logs', 'Observation Start Time', 'Observation End Time', 'Process Start Time', 'Process End Time', 'Control ID','#Dataproducts', 'Data Size [TB]', 'Data Size on Disk [TB]', 'Subtask Content', 'Obsolete'];
        const columnDefinitionToRemove = type === 'blueprint' ?['scheduling_unit_draft']:['scheduling_unit_blueprint', 'status', 'unschedulable_reason', 'status_logs', 'on_sky_start_time', 'on_sky_stop_time', 'process_start_time', 'process_stop_time', 'subTaskID','noOfOutputProducts','size','dataSizeOnDisk','subtaskContent', 'obsolete'];
        let tmpDefaulColumns = _.cloneDeep(this.state.defaultcolumns[0]);
        let tmpOptionalColumns = _.cloneDeep(this.state.optionalcolumns[0]);
        tmpOptionalColumns.blueprint_draft = this.props.match.params.type==="draft"?"Linked Blueprint": "Linked Draft"
        let tmpColumnOrders = _.cloneDeep(this.state.columnOrders);
        tmpColumnOrders = _.differenceWith(tmpColumnOrders, columnOrderToRemove, _.isEqual);
        tmpDefaulColumns = _.omit(tmpDefaulColumns,columnDefinitionToRemove);
        tmpOptionalColumns = _.omit(tmpOptionalColumns,columnDefinitionToRemove)
        await this.setState({tmpDefaulcolumns: [tmpDefaulColumns], tmpOptionalcolumns:[tmpOptionalColumns], tmpColumnOrders: tmpColumnOrders, columnMap: this.columnMap})
    }
    /**
     * Get action menus for page header
     */
    getActionMenu(schedule_type, isIngestPresent, disableUpdateAction) {
        const userPermissions = this.state.userPermission?this.state.userPermission.permissions:{};
        this.actions = []; 
        this.actions.unshift({
            icon: 'fa-copy', 
            title: disableUpdateAction === undefined?this.state.permissionById.copySU?'Copy Draft/Blueprint':`${this.access_denied_message} to copy`:'This Scheduling Unit is deleted by another user.', 
            type: 'button',
            style: this.iconStyle,
            disabled: disableUpdateAction === undefined?!this.state.permissionById.copySU: disableUpdateAction,
            actOn: 'click', props: { callback: this.confirmCopyingSU },
        });     
        this.actions.push({
            icon: 'fa-window-restore', 
            title: disableUpdateAction === undefined?this.state.permissionById.copySU?'Advanced Copy':`${this.access_denied_message} to copy`:'This Scheduling Unit is deleted by another user.', 
            type: 'button',
            style: this.iconStyle,
            disabled: disableUpdateAction === undefined?!this.state.permissionById.copySU: disableUpdateAction,
            actOn: 'click', props: { callback: this.showAdvancedCopyDialog },
        });
        this.actions.push({
            icon: 'fas fa-arrows-alt', 
            title: 'Move to different Scheduling Set', 
            type: 'button',
            style: this.iconStyle,
            disabled: disableUpdateAction === undefined?!this.state.permissionById.copySU: disableUpdateAction,
            actOn: 'click', props: { callback: this.confirmChangeSUSet },
        });
        let canDelete = (this.state.scheduleunit &&
            (!this.state.scheduleunit.scheduling_unit_blueprints_ids || this.state.scheduleunit.scheduling_unit_blueprints_ids.length === 0));                 
        this.actions.push({
            icon: 'fa fa-trash',
            style: this.iconStyle,
            title: disableUpdateAction === undefined?this.state.permissionById.delete?!canDelete ? 'Cannot delete Draft when Blueprint exists' : 'Delete Scheduling Unit':`${this.access_denied_message} to delete`:'This Scheduling Unit is deleted by another user.', 
            disabled: disableUpdateAction === undefined?!this.state.permissionById.delete || !canDelete:disableUpdateAction,
            type: 'button', actOn: 'click', props: { callback: this.showDeleteSUConfirmation }
        });
        if (this.props.match.params.type ==='draft') { 
            this.actions.push({
                icon: 'fas fa-folder-plus',
                style: this.iconStyle,
                title:'Add Injest Task', 
                type: 'button', actOn: 'click', props: { callback: this.confirmAddIngestTask }
            });
        }

        if(isIngestPresent) {
            this.actions.push({
                icon: 'fa-file-import', 
                title: userPermissions.scheduleunit && userPermissions.scheduleunit.dataproduct?'Data Products To Ingest':`${this.access_denied_message} to edit dataproducts to ingest`, 
                type: 'button',
                style: this.iconStyle,
                disabled:  userPermissions.scheduleunit?!userPermissions.scheduleunit.dataproduct:!(this.state.scheduleunit.status === 'defined' || this.state.scheduleunit.status === 'scheduled' || this.state.scheduleunit.status === 'schedulable'),                    
                actOn: 'click', props: { callback: this.showTaskRelationDialog }
            });
        }  

        this.actions.push({ icon: 'fa-window-close', title: 'Click to Close Scheduling Unit View', type: 'button', style: this.iconStyle, actOn: 'click', props:{ callback: this.cancelView }});
        if (this.props.match.params.type ==='draft') {           
           let blueprintExist = this.state.scheduleunit && this.state.scheduleunit.scheduling_unit_blueprints_ids && this.state.scheduleunit.scheduling_unit_blueprints_ids.length>0;
            this.actions.unshift({
                icon: 'fa-edit',
                style: this.iconStyle,
                title: disableUpdateAction === undefined?this.state.permissionById.edit?'Click to edit':`${this.access_denied_message} to edit`: 'This Scheduling Unit is deleted by another user.', 
                disabled: disableUpdateAction === undefined ?!this.state.permissionById.edit: disableUpdateAction,
                props: { pathname: `/schedulingunit/edit/${this.props.match.params.id}` }
            });
            this.actions.unshift({
                icon: 'fa-stamp', 
                style: this.iconStyle,
                title: disableUpdateAction === undefined?this.state.permissionById.createSub?blueprintExist?'Blueprint already exists': 'Create Blueprint':`${this.access_denied_message} to create Blueprint`: 'This Scheduling Unit is deleted by another user.',
                type: 'button',
                disabled: disableUpdateAction === undefined?!this.state.permissionById.createSub?blueprintExist:!this.state.permissionById.createSub:disableUpdateAction,
                actOn: 'click', props: { callback: this.checkAndCreateBlueprint },
            });
        } else {
            this.actions.unshift({ 
                icon: 'fa-thumbtack', 
                style: this.iconStyle,
                title: this.state.permissionById.autoDeletion?this.state.scheduleunit.output_pinned? 'Unpin data after ingest' : 'Pin data after ingest':`${this.access_denied_message} to do Pin/Unpin data after ingest`, 
                disabled: !this.state.permissionById.autoDeletion,
                type: 'button', actOn: 'click', props: { callback: this.confirmAutoDeletion } 
            });
            
            this.actions.unshift({
                icon: 'fa-ban', type: 'button', actOn: 'click',
                style: this.iconStyle,
                title: this.state.permissionById.cancelSU?this.SU_END_STATUSES.indexOf(this.state.scheduleunit.status.toLowerCase())>=0?'Cannot Cancel Scheduling Unit':'Cancel Scheduling Unit':`${this.access_denied_message} to cancel Scheduling Unit`,  
                disabled: !this.state.permissionById.cancelSU || this.SU_END_STATUSES.indexOf(this.state.scheduleunit.status.toLowerCase())>=0, 
                props: { callback: this.showCancelSUConfirmation }
            });            
            this.actions.unshift({
                icon: 'fa fa-recycle', 
                style: this.iconStyle,
                title: this.state.permissionById.cleanupTask?'Create Clean-up Task':`${this.access_denied_message} to create Clean-up Task`, 
                type: 'button',
                disabled: !this.state.permissionById.cleanupTask,
                actOn: 'click', props: { callback: this.cleanUpSUTask }
            });
            this.actions.unshift({
                icon: 'fas fa-save',
                style: this.iconStyle,
                title: this.state.permissionById.updateSub?
                'Update blueprints from draft'
                :`${this.access_denied_message} to update blueprint from draft`, 
                type: 'button',
                disabled: !this.state.permissionById.updateSub,
                actOn: 'click', props: { callback: this.confirmUpdateSUBlueprint }
            });
            this.actions.unshift({
                icon: 'fas fa-code-branch',
                style: this.iconStyle,
                title: this.state.permissionById.copyFailedTasks
                    ?(this.state.scheduleunit.status === 'error' || this.state.scheduleunit.status === 'cancelled')
                    ?'Create draft copy for failed tasks' //If the SUB is valid to do this functionality
                    :'Can create draft copy for failed tasks only if the Scheduling Unit status is either Error/Cancelled' //If the SUB is not valid to do this functionality
                    :`${this.access_denied_message} to create draft copy for failed tasks`, //If the User doesn't have permission to do this functionality
                type: 'button',
                disabled: !this.state.permissionById.copyFailedTasks ||
                            !(this.state.scheduleunit.status === 'error' || this.state.scheduleunit.status === 'cancelled'),
                actOn: 'click', props: { callback: this.confirmCopySpecAndFailedTasks }
            });
            this.actions.unshift({
                icon: 'fa fa-redo',
                style: this.iconStyle,
                title: this.state.permissionById.createRerun?
                    (this.state.scheduleunit.status === 'error' || this.state.scheduleunit.status === 'cancelled')
                    ?'Re-run all failed tasks'
                    :'Can re-run all failed tasks if the Scheduling Unit status is either Error/Cancelled'
                    :`${this.access_denied_message} to re-run for all failed tasks`, 
                type: 'button',
                disabled: !this.state.permissionById.createRerun || !(this.state.scheduleunit.status === 'error' || this.state.scheduleunit.status === 'cancelled'),
                actOn: 'click', props: { callback: this.confirmCopyFailedTasks }
            });
            this.actions.unshift({ 
                icon: 'fa-sitemap', 
                style: this.iconStyle,
                title: this.state.permissionById.viewWorkflow?'View Workflow':`${this.access_denied_message} to view Workflow`, 
                disabled: !this.state.permissionById.viewWorkflow,
                props: { pathname: `/schedulingunit/${this.props.match.params.id}/workflow`, state: {origin: this.props.location.pathname} } });
            this.actions.unshift({ icon: 'fa-lock', title: 'Cannot edit blueprint' });            
        }
        this.setState({ actions: this.actions });
    }

    /**
     * Formatting the task_drafts and task_blueprints in draft view to pass to the ViewTable component
     * @param {Object} schedulingUnit - scheduling_unit_draft object from extended API call loaded with tasks(draft & blueprint) along with their template and subtasks
     */
    async getFormattedTaskDrafts(schedulingUnit) {
        const taskPermission = schedulingUnit.task_drafts.length > 0 && await AuthUtil.getUserPermissionByModuleId('task_draft', schedulingUnit.task_drafts[0].id);
        this.setState({taskDraftPermission : taskPermission &&  taskPermission[schedulingUnit.task_drafts[0].id]});
        let scheduletasklist = [];
        // Common keys for Task and Blueprint
        let commonkeys = ['id', 'created_at', 'description', 'short_description' , 'name', 'tags', 'updated_at', 'url', 'do_cancel', 'relative_start_time', 'relative_stop_time', 'on_sky_start_time', 'on_sky_stop_time', 'process_start_time', 'process_stop_time', 'duration', 'status', 'obsolete', 'task_type'];
        for (const task of schedulingUnit.task_drafts) {
            let scheduletask = {};
            scheduletask['tasktype'] = 'Draft';
            scheduletask['actionpath'] = '/task/view/draft/' + task['id'];
            scheduletask['blueprint_draft'] = this.getLinksList(task['task_blueprints_ids'], 'blueprint');
            scheduletask['task_blueprints_ids'] = task.task_blueprints_ids;
            scheduletask['status'] = task['status'];
            scheduletask['connector_types'] = task.connector_types;
            //fetch task draft details
            for (const key of commonkeys) {
                scheduletask[key] = task[key];
            }
            if (scheduletask['task_type']) {
                scheduletask['rowColor'] = 'task-'+scheduletask['task_type'];
            }
            scheduletask['specifications_doc'] = task['specifications_doc'];
            scheduletask['short_description'] = task['short_description'];
            scheduletask.duration = moment.utc((task['specifications_doc'].duration || scheduletask.duration || 0) * 1000).format(UIConstants.CALENDAR_TIME_FORMAT);
            scheduletask.relative_start_time = moment.utc(scheduletask.relative_start_time * 1000).format(UIConstants.CALENDAR_TIME_FORMAT);
            scheduletask.relative_stop_time = moment.utc(scheduletask.relative_stop_time * 1000).format(UIConstants.CALENDAR_TIME_FORMAT);
            scheduletask.template = task.specifications_template;
            scheduletask.type_value = task.specifications_template.type_value;
            scheduletask.produced_by = task.produced_by;
            scheduletask.produced_by_ids = task.produced_by_ids;
            scheduletask.output_pinned = task.output_pinned;
            scheduletask['links'] = ['API URL'];
            scheduletask['linksURL'] = {
                'API URL': task.url
            }
            //Add Task Draft details to array
            scheduletasklist.push(scheduletask);
        }
        //Ingest Task Relation
        const ingestTasks = scheduletasklist.filter(task => task.type_value === 'ingest' && task.tasktype.toLowerCase() === 'draft');
        this.ingestTaskRelations = [];
        for (const ingestTask of ingestTasks) {
            for (const producer_id of ingestTask.produced_by_ids) {
                const taskRelation = await ScheduleService.getTaskRelation(producer_id);
                let producerTask = scheduletasklist.find(task => task.id === taskRelation.producer_id && task.tasktype.toLowerCase() === 'draft');
                if(producerTask!==undefined){
                    producerTask.consumers = producerTask.consumers || [];
                    let producerTaskConsumer = _.find(producerTask.consumers, { id: ingestTask.id}) || { id: ingestTask.id};
                    producerTaskConsumer.canRemove = taskRelation && taskRelation.blueprints.length < 1 ? true: producerTaskConsumer.canRemove? producerTaskConsumer.canRemove : false;
                    await producerTask.connector_types.map(connector =>{
                        if(connector.id === taskRelation.output_role_id && connector.iotype_value === 'output') {
                            connector.consumers = connector.consumers || [];
                            let connectorConsumer = { id: ingestTask.id };
                            connectorConsumer.canIngest = true;
                            connectorConsumer.canRemove = taskRelation? taskRelation.blueprints.length < 1: true;
                            connector.consumers.push(connectorConsumer);
                            producerTaskConsumer.canIngest = producerTaskConsumer.canIngest===undefined?connectorConsumer.canIngest: producerTaskConsumer.canIngest&&connectorConsumer.canIngest;
                        }
                        return connector;
                    });
                    producerTask.consumers.push(producerTaskConsumer)
                    // Store all existing task relations of ingest task
                    this.ingestTaskRelations.push(taskRelation);
                }
            }
        }
        return scheduletasklist;
    }

    /**
     * Formatting the task_blueprints in blueprint view to pass to the ViewTable component
     * @param {Object} schedulingUnit - scheduling_unit_blueprint object from extended API call loaded with tasks(blueprint) along with their template and subtasks
     */
     async getFormattedTaskBlueprints(schedulingUnit) {
        const taskPermission = await AuthUtil.getUserPermissionByModuleId('task_blueprint', schedulingUnit.task_blueprints[0].id);
        this.setState({taskPermission : taskPermission})
        let taskBlueprintsList = [];
        for (const taskBlueprint of schedulingUnit.task_blueprints) {
            taskBlueprint['tasktype'] = 'Blueprint';
            taskBlueprint['actionpath'] = '/task/view/blueprint/' + taskBlueprint['id'];
            taskBlueprint['blueprint_draft'] =  this.getLinksList([taskBlueprint['draft_id']],'draft');
            taskBlueprint['relative_start_time'] = 0;
            taskBlueprint['relative_stop_time'] = 0;
            taskBlueprint.duration = moment.utc((taskBlueprint.duration || 0) * 1000).format(UIConstants.CALENDAR_TIME_FORMAT);
            taskBlueprint.template = taskBlueprint.specifications_template;
            for (const subtask of taskBlueprint.subtasks) {
                subtask.subTaskTemplate = _.find(this.subtaskTemplates, ['id', subtask.specifications_template_id]);
            }
            taskBlueprint.subTasks = taskBlueprint.subtasks;
            taskBlueprint['links'] = ['API URL'];
            taskBlueprint['linksURL'] = {
                'API URL': taskBlueprint.url
            }
            if (taskBlueprint['task_type']) {
                taskBlueprint['rowColor'] = 'task-'+taskBlueprint['task_type'];
            }
            taskBlueprintsList.push(taskBlueprint);
        }
        //Ingest Task Relation
        const ingestTasks = taskBlueprintsList.filter(task => task.task_type === 'ingest' && task.tasktype.toLowerCase() === 'blueprint');
        for (const ingestTask of ingestTasks) {
            for (const producer_id of ingestTask.produced_by_ids) {
                const blueprintRelation = await ScheduleService.getBlueprintRelation(producer_id);
                let producerTask = taskBlueprintsList.find(task => task.id === blueprintRelation.producer_id && task.tasktype.toLowerCase() === 'blueprint');
                if(producerTask!==undefined){
                    // Store all existing task relations of ingest task
                    producerTask.consumers = producerTask.consumers || [];
                    let producerTaskConsumer = { id: ingestTask.id, canRemove: true};
                    await producerTask.draft.connector_types.map(connector =>{
                        if(connector.id === blueprintRelation.output_role_id && connector.iotype_value === 'output') {
                            connector.consumers = connector.consumers || [];
                            let connectorConsumer = { id: ingestTask.id };
                            connectorConsumer.canIngest = true;
                            connectorConsumer.canRemove = true;
                            connector.consumers.push(connectorConsumer);
                            producerTaskConsumer.canIngest = producerTaskConsumer.canIngest===undefined?connectorConsumer.canIngest: producerTaskConsumer.canIngest&&connectorConsumer.canIngest;
                        }
                        return connector;
                    })
                    producerTask.consumers.push(producerTaskConsumer)
                    // Store all existing task relations of ingest task
                    this.ingestTaskRelations.push(blueprintRelation);
                }
            }
        }
        return taskBlueprintsList;
    }

    getLinksList = (linkedItems, type) => {
        return (
            <>
                {linkedItems.length>0 && linkedItems.map((item ) => (
                    <Link style={{paddingRight: '3px'}} to={`/task/view/${type}/${item}`} key={item}>{item}</Link>
                ))}
            </>
        );                    
    }

    getScheduleUnitTasks(type, scheduleunit) {
        if (type === 'draft')
            return ScheduleService.getTasksBySchedulingUnit(scheduleunit.id, true, true, true);
        else
            return ScheduleService.getTaskBPWithSubtaskTemplateOfSU(scheduleunit);
    }

    getScheduleUnit(type, id) {
        if (type === 'draft')
            return ScheduleService.getSchedulingUnitDraftById(id)
        else
            return ScheduleService.getSchedulingUnitBlueprintById(id)
    }

    /**
     * Checks if the draft scheduling unit has existing blueprints and alerts. If confirms to create, creates blueprint.
     */
    checkAndCreateBlueprint() {
        if (this.state.scheduleunit) {
            let dialog = this.state.dialog;
            dialog.header = "Confirm";
            dialog.onSubmit = this.createBlueprintTree;
            dialog.content = null;
            dialog.width = "40vw";
            if (this.state.scheduleunit.scheduling_unit_blueprints_ids.length > 0) {
                dialog.detail = "Blueprint(s) already exist for this Scheduling Unit. Do you want to create another one?";
            } else {
                dialog.detail = "Do you want to create a Scheduling Unit Blueprint? The Scheduling Unit may be scheduled immediately.";
            }
            dialog.actions = [{ id: 'yes', title: 'Create', callback: this.createBlueprintTree },
            { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
            this.setState({ dialogVisible: true, dialog: dialog });
        }
    }

    async changeProject(projectName) {
        const projectSchedSets = await ScheduleService.getSchedulingSets(`project=${encodeURIComponent(projectName)}`);
        let schedulingUnit = this.state.copiedSchedulingUnit;
        schedulingUnit.project = projectName;
        schedulingUnit.scheduling_set_id = null;
        const selectedProject = _.filter(this.projects, {'name': projectName});
        schedulingUnit['output_pinned'] = selectedProject[0].auto_pin;
        this.projectSchedSets = projectSchedSets;
        _.map(this.projectSchedSets, suSet => suSet.nameId = `${suSet.name} (${suSet.id})`);
        this.setState({selectedProject: selectedProject, copiedSchedulingUnit: schedulingUnit, projectSchedSets: projectSchedSets});
    }

    /**
     * To set the values for the copied schdeuling unit
     * @param {string} key 
     * @param {any} value 
     */
    setSchedUnitParams(key, value) {
        let schedulingUnit = _.cloneDeep(this.state.copiedSchedulingUnit);
        schedulingUnit[key] = value;
        this.setState({copiedSchedulingUnit: schedulingUnit});
    }

    /**
     * To get the form componet with required fields for copying to a different SU set
     * @returns Component
     */
    getAdvancedSUCopyContent() {
        return <>
            <div style={{marginTop: '1em'}}>              
                <div className="p-field p-grid">
                    <label htmlFor="schedUnitName" className="col-lg-2 col-md-2 col-sm-12">Name <span style={{color:'red'}}>*</span></label>
                    <div className="col-lg-4 col-md-4 col-sm-12" data-testid="namediv">
                        <InputText id="name" data-testid="name" style={{width:"100%"}}
                                    tooltip="Enter name of the Scheduling Unit" maxLength="128"
                                    ref={input => {this.nameInput = input;}}
                                    value={this.state.copiedSchedulingUnit.name}
                                    onChange={(e) => this.setSchedUnitParams('name', e.target.value)}
                                    onBlur={(e) => this.setSchedUnitParams('name', e.target.value)}
                                    />
                    </div>
                    <label htmlFor="description" className="col-lg-2 col-md-2 col-sm-12">Description <span style={{color:'red'}}>*</span></label>
                    <div className="col-lg-4 col-md-4 col-sm-3">
                        <InputTextarea rows={3} cols={30} style={{width:"100%"}}
                                    tooltip="Longer description of the scheduling unit" maxLength="128"
                                    data-testid="description" value={this.state.copiedSchedulingUnit.description} 
                                    onChange={(e) => this.setSchedUnitParams('description', e.target.value)}
                                    onBlur={(e) => this.setSchedUnitParams('description', e.target.value)}
                                    />
                    </div>
                    
                </div>
                <div className="p-field p-grid">
                    <label htmlFor="project" className="col-lg-2 col-md-2 col-sm-12">Project </label>
                    <div className="col-lg-4 col-md-4 col-sm-12" data-testid="project" >
                        <Dropdown inputId="project" optionLabel="name" optionValue="name" 
                                tooltip="Project" style={{width:"100%"}}
                                tooltipOptions={this.tooltipOptions}
                                value={this.state.copiedSchedulingUnit.project}
                                options={this.projects} 
                                placeholder="Select Project"
                                onChange={(e) => {this.changeProject(e.value)}} 
                                />
                    </div>
                    <label htmlFor="schedSet" className="col-lg-2 col-md-2 col-sm-12">Scheduling Set <span style={{color:'red'}}>*</span></label>
                    <div className="col-lg-4 col-md-4 col-sm-12">
                        <Dropdown data-testid="schedSet" id="schedSet" optionLabel="nameId" optionValue="id" 
                                tooltip="Scheduling set of the project" style={{width:"100%"}}
                                value={this.state.copiedSchedulingUnit.scheduling_set_id} 
                                options={this.projectSchedSets}
                                placeholder="Select Scheduling Set" 
                                onChange={(e) => {this.setSchedUnitParams('scheduling_set_id',e.value)}} 
                                />
                    </div>
                </div>
                <div className="p-field p-grid">
                    <label htmlFor="priority_queue_value" className="col-lg-2 col-md-2 col-sm-12">Priority Queue</label>
                    <div className="col-lg-4 col-md-4 col-sm-10">
                        <Dropdown data-testid="priority_queue_value" id="priority_queue_value" optionLabel="value" optionValue="url" 
                            tooltip="Priority queue of this scheduling unit. Queues provide a strict ordering between scheduling units."
                            value={this.state.copiedSchedulingUnit.priority_queue || this.state.copiedSchedulingUnit.draft.priority_queue} 
                            options={this.priorityQueueTypes} 
                            onChange={(e) => {this.setSchedUnitParams('priority_queue',e.value)}} 
                            placeholder="Select Priority Queue" style={{width:"100%"}}
                            />     
                    </div>
                </div>
            </div>
        </>
    }

    getAdvancedSUCopyStatusContent(){
        let suContent = [{suId:this.state.copiedSchedulingUnit.id, suName: this.state.copiedSchedulingUnit.name, status:this.statusUpdate}]
        return  <> 
                    <div style={{marginTop: '1em'}}>
                        <DataTable value={suContent} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                            <Column field="suId" header="Id"></Column>
                            <Column field="suName" header="Name"></Column>
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </div>
                    {
                        this.statusUpdate === 'Success' ?
                        <div style = {{ marginTop: '1em'}} >
                        Do you want to open the copied Scheduling Unit draft ?
                        </div> : null
                    }
                </>
     }
     
     getSUCopyStatusContent() {
         let selectedTasks = [{ suId: this.state.scheduleunit.id, 
            suName: this.state.scheduleunit.name, 
            status: this.state.scheduleunit.status, 
            suType: (this.state.scheduleunit.draft) ? 'Blueprint' : 'Draft' }];

        return <>
            <div style={{marginTop: '1em'}}>              
                <DataTable value={selectedTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="suId" header="Scheduling Unit Id"></Column>
                    <Column field="suName" header="Scheduling Unit Name"></Column>                    
                </DataTable>
            </div>
        </>
    }

    //Confirmation to Copy Scheduling Unit
    confirmCopyingSU() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Confirm to Copy Scheduling Unit";
        dialog.detail = "Do you want to copy this Scheduling Unit " + this.state.scheduleunitType + "?";
        dialog.content = this.getSUCopyStatusContent;
        dialog.actions = [{
                id: 'yes',
                title: 'Copy',
                callback: this.copySchedulingUnit
            },
            {
                id: 'no',
                title: 'Cancel',
                className:'act-btn-cancel',
                callback: this.closeDialog
            }
        ];
        dialog.onSubmit = this.copySchedulingUnit;
        dialog.width = '35vw';
        dialog.showIcon = false;
        this.setState({
            dialog: dialog,
            dialogVisible: true
        });
    }
    
    //Function to Copy SU Draft and Blueprint 
    async copySchedulingUnit() {
        let dialog = this.state.dialog;
        let response = null;
        this.statusUpdate = null;
        try {                    
            let param = {
            }
            if (this.state.scheduleunitType === 'draft') {
                response = await ScheduleService.copySchedulingUnitDraft(this.state.scheduleunit.id, param);
            } else {
                response = await ScheduleService.copySchedulingUnitBluePrint(this.state.scheduleunit.id, param);                
            }
            this.statusUpdate = response.status === 201 ? "Success" : "Failed";
            this.copyScheduleUnitID = response.data?.id;
        } catch (error) {
            this.statusUpdate = "Failed";
        }
    
        dialog.detail = '';
        dialog.type = response.status === 201 ? "success" : "warning";
        dialog.header = "Copy Scheduling Unit Status";
        dialog.content = this.getStatusForCopiedShedulingUnit;
        if (this.statusUpdate === 'Success') {
            dialog.actions = [{
                    id: 'yes',
                    title: 'Yes',
                    callback: this.navigateToCopiedSUPage
                },
                {
                    id: 'no',
                    title: 'No',
                    className:'act-btn-cancel',
                    callback: this.closeDialog
                }
            ];
        } else {
            dialog.actions = [{
                id: 'no',
                title: 'No',
                className:'act-btn-cancel',
                callback: this.closeDialog
            }];
        }
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({
            dialog: dialog,
            dialogVisible: true
        });
    
    }  

    /**
     * Function to show the advanced copy dialog with fields to specify
     */
     async showAdvancedCopyDialog() {
        const promises = [  ProjectService.getProjectList(false, 'name,url'),
                            UtilService.getPriorityQueueType(),
                            ScheduleService.getSchedulingSets(`project=${encodeURIComponent(this.state.scheduleunit.scheduling_set.project_id)}`)
                        ];
        Promise.all(promises).then(async(responses) => {
            this.projects = responses[0];
            this.priorityQueueTypes = responses[1];
            this.projectSchedSets = responses[2];
            _.map(this.projectSchedSets, suSet => suSet.nameId = `${suSet.name} (${suSet.id})`);

            // await this.setState({copiedSchedulingUnit: this.state.scheduleunit, projectSchedSets: this.projectSchedSets})

            let dialog = this.state.dialog;
            dialog.type = "confirmation";
            dialog.header = "Advanced Copy Scheduling Unit";
            dialog.detail = "Select the Project and/or Scheduling set to which you want to copy this Scheduling Unit " + this.state.scheduleunitType + "?";
            dialog.content = this.getAdvancedSUCopyContent;
            dialog.actions = [{
                    id: 'yes',
                    title: 'Copy',
                    callback: this.copyAdvancedSchedulingUnit
                },
                {
                    id: 'no',
                    title: 'Cancel',
                    className:'act-btn-cancel',
                    callback: this.closeDialog
                }
            ];
            dialog.onSubmit = this.copyAdvancedSchedulingUnit;
            dialog.width = '55vw';
            dialog.showIcon = false;
            this.setState({
                copiedSchedulingUnit: this.state.scheduleunit, projectSchedSets: this.projectSchedSet,
                dialog: dialog,
                dialogVisible: true
            });
        }); 
    }

    /**
     * Function to Copy SU Draft to different Scheduling Set
     * @returns 
     */
     async copyAdvancedSchedulingUnit() {
        let dialog = this.state.dialog;
        let response = null;
        this.statusUpdate = null;
        let copiedSchedulingUnit = {};
        let copyUpdateResponse = null;
        if (this.state.copiedSchedulingUnit.name === "" || this.state.copiedSchedulingUnit.description === ""
            || !this.state.copiedSchedulingUnit.scheduling_set_id) {
                appGrowl.show({ severity: 'info', summary: 'Missing Information', detail: 'Please fill all required fields' });
                return;
        }
        try {                    
            let param = {
                name: this.state.copiedSchedulingUnit.name,
                scheduling_set_id: this.state.copiedSchedulingUnit.scheduling_set_id
            }
            if (this.state.scheduleunitType === 'draft') {
                response = await ScheduleService.copySchedulingUnitDraft(this.state.scheduleunit.id, param);
            }   else {
                response = await ScheduleService.copySchedulingUnitBluePrint(this.state.scheduleunit.id, param);    
            }
            this.statusUpdate = response.status === 201 ? "Success" : "Failed";
            this.copyScheduleUnitID = response.data?.id;
            copiedSchedulingUnit = response.data;
            copiedSchedulingUnit['task_drafts'] = _.map(response.data.task_drafts, 'url');
            copiedSchedulingUnit['scheduling_set'] = (_.find(this.projectSchedSets, {'id': this.state.copiedSchedulingUnit.scheduling_set_id})).url;
            copiedSchedulingUnit['name'] = this.state.copiedSchedulingUnit.name ?this.state.copiedSchedulingUnit.name: response.data.name;
            copiedSchedulingUnit['description'] = this.state.copiedSchedulingUnit.description?this.state.copiedSchedulingUnit.description: response.data.description;
            copiedSchedulingUnit['priority_queue'] = this.state.copiedSchedulingUnit.priority_queue;
            copyUpdateResponse = await ScheduleService.updateSchedulingUnitDraft(copiedSchedulingUnit);
        } catch (error) {
            this.statusUpdate = "Failed";
        }
    
        dialog.detail = '';
        dialog.type = copyUpdateResponse.status === 201 ? "success" : "warning";
        dialog.header = "Advanced Copy Scheduling Unit - Status";
        dialog.content = this.getAdvancedSUCopyStatusContent;
        if (this.statusUpdate === 'Success') {
            dialog.actions = [{
                    id: 'yes',
                    title: 'Yes',
                    callback: this.navigateToCopiedSUPage
                },
                {
                    id: 'no',
                    title: 'No',
                    className: 'act-btn-cancel',
                    callback: this.closeDialog
                }
            ];
        } else {
            dialog.actions = [{
                id: 'no',
                title: 'No',
                callback: this.closeDialog
            }];
        }
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({
            dialog: dialog,
            dialogVisible: true
        });
    }

    /**
     * Function to move SUD/SUB to different Scheduling Set
     * @returns 
     */
     async changeSUSet() {
        if(this.state.selectedSUSet) {
            let schUnit = null; 
            if (this.state.scheduleunitType.toLowerCase() === "blueprint") {
                schUnit = await ScheduleService.getExpandedSchedulingUnit('draft', this.state.scheduleunitType.toLowerCase() === 'draft'?this.state.scheduleunit.id: this.state.scheduleunit.draft.id ,this.SU_DRAFT_EXPAND,this.SU_DRAFT_FIELDS);
            }   else {
                schUnit =  _.clone(this.state.scheduleunit);
            }
            let response = null;
            this.statusUpdate = null;
            schUnit['task_drafts'] = _.map(schUnit.task_drafts, 'url');
            schUnit['scheduling_unit_blueprints']  = _.map(schUnit.scheduling_unit_blueprints, 'url');
            schUnit.scheduling_constraints_template = schUnit.scheduling_constraints_template.url;
            schUnit.specifications_template=schUnit.observation_strategy_template.scheduling_unit_template;
            schUnit.scheduling_constraints_doc = this.schedulingConstraintDocRaw;
            delete schUnit.observation_strategy_template;
            schUnit.scheduling_set = this.state.selectedSUSet.url;

            try { 
                response = await ScheduleService.updateSchedulingUnitDraft(schUnit);
                if(response.status === 200) {
                    appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Successfully moved to another Scheduling Set!' });
                    this.closeDialog();
                    this.init();
                }
                else {
                    appGrowl.show({ severity: 'error', summary: 'Failed', detail: 'Unable to change Scheduling Set!'});
                }
            } catch (error) {
                this.statusUpdate = "Failed";
            }
        }
    }

    /**
     * Show confirmation dialog before moving SUD/SUB to different SU Set
     */
    async confirmChangeSUSet() {
        this.projectSchedSets = _.remove(this.projectSchedSets, (suSet) => {
            return suSet.id !== this.state.scheduleunit.scheduling_set.id
        })
        this.affectedSU = this.state.scheduleunitType.toLowerCase() === 'draft'? this.state.scheduleunit.scheduling_unit_blueprints: [this.state.scheduleunit.draft];
        let suList = [];
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Move to different Scheduling Set";
        dialog.detail = "Select the Scheduling Set to which you want to move this Scheduling Unit";
        dialog.content = this.getSUSetContent;
        dialog.actions = [{
                id: 'yes',
                title: 'Move',
                callback: this.changeSUSet
            },
            {
                id: 'no',
                title: 'Cancel',
                className:'act-btn-cancel',
                callback: this.closeDialog
            }
        ];
        dialog.onSubmit = this.changeSUSet;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({
            copiedSchedulingUnit: this.state.scheduleunit, projectSchedSets: this.projectSchedSet,
            dialog: dialog, suList: suList,
            dialogVisible: true
        }); 
    }

    /**
     * Content with confirmation message and affected SUB/SUD in confirmation dialog
     * @returns Component
     */
    getSUSetContent(){
        return  <> 
                    <div style={{marginTop: '1em'}}>
                        <DataTable value={this.projectSchedSets} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}
                        selectionMode="single" selection={this.state.selectedSUSet} onSelectionChange={e => this.setState({selectedSUSet: e.value})} sortField= "name">
                            <Column field="id" header="Id"></Column>
                            <Column field="name" header="Name"></Column>
                            <Column field="description" header="Description"></Column>
                        </DataTable>
                    </div>
                    {this.affectedSU.length > 0 &&
                        <div>
                            <div style={{marginTop: '1em'}}>
                            <span>Scheduling unit {this.state.scheduleunitType.toLowerCase() === "draft"?"blueprint":"draft"} shown below will be affected. Do you want to continue?</span>
                            <DataTable value={this.affectedSU} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="id" header="Id"></Column>
                                <Column field="name" header="Name"></Column>
                            </DataTable>
                        </div>
                        </div>
                    }
                </>
     }

    //navigate to new copied SU Page
    navigateToCopiedSUPage() {
        if (this.copyScheduleUnitID) {
            this.setState({redirect: '/schedulingunit/view/draft/' + this.copyScheduleUnitID});
        } else {
            this.setState({
                dialogVisible: false,
                cancelledTasks: []
            });
        }
    }

    /**
     * Funtion called to create blueprint on confirmation.
     */
    createBlueprintTree() {
        this.setState({ dialogVisible: false, showSpinner: true });
        ScheduleService.createSchedulingUnitBlueprintTree(this.state.scheduleunit.id)
            .then(blueprint => {
                if (blueprint) {
                    appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Blueprint created successfully!' });
                    this.setState({ showSpinner: false, redirect: `/schedulingunit/view/blueprint/${blueprint.id}`, isLoading: true });
                } else {
                    appGrowl.show({ severity: 'error', summary: 'Failed', detail: 'Unable to create blueprint!' });
                    this.setState({ showSpinner: false });
                }
            });
    }

    /**
     * JEditor's function that to be called when parent wants to trigger change in the JSON Editor
     * @param {Function} editorFunction 
     */
     setEditorFunction(editorFunction) {
        this.setState({editorFunction: editorFunction});
    }

    /**
     * Callback function to close the dialog prompted.
     */
    closeDialog() {
        this.setState({ dialogVisible: false, cancelledTasks: [] , obsoleteTasks: []});
    }

    onRowSelection(selectedRows) {
        this.selectedRows = selectedRows;
    }

    /**
     * Confirmation dialog for delete task(s)
     */
    confirmDeleteTasks() {
        this.pageUpdated = false;
        if (this.selectedRows.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', detail: 'Select Task to delete.' });
        } else {
            let dialog = this.state.dialog;
            dialog.type = "confirmation";
            dialog.header = "Confirm to Delete Task(s)";
            dialog.detail = "Do you want to delete the selected Task(s)?";
            dialog.content = this.getTaskDeleteDialogContent;
            dialog.actions = [{ id: 'yes', title: 'Delete Tasks', callback: this.deleteTasks, style:{"background": "#e91224","border": "1px solid #e91224"} },
            { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
            dialog.onSubmit = this.deleteTasks;
            dialog.width = '55vw';
            dialog.showIcon = false;
            this.setState({ dialog: dialog, dialogVisible: true });
        }
    }

    showDeleteSUConfirmation() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Confirm to Delete Scheduling Unit";
        dialog.detail = "Do you want to delete this Scheduling Unit?";
        dialog.content = this.getSUDialogContent;
        dialog.actions = [{ id: 'yes', title: 'Delete', callback: this.deleteSchedulingUnit, style:{"background": "#e91224","border": "1px solid #e91224"} },
        { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
        dialog.onSubmit = this.deleteSchedulingUnit;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({ dialog: dialog, dialogVisible: true });
    }

     /**
     * Show confirmation dialog before cancelling the scheduling unit.
     */
    showCancelSUConfirmation() {
        let dialog = this.state.dialog;
        dialog.detail = '';
        dialog.type = "confirmation";
        dialog.header = "Confirm to Cancel Scheduling Unit";
        dialog.actions = [{ id: 'yes', title: 'Cancel SU', callback: this.cancelSchedulingUnit, style:{"background": "#e91224","border": "1px solid #e91224"} },
                            { id: 'no', title: 'No', className:'act-btn-cancel', callback: this.closeDialog }];
        if (this.SU_NOT_STARTED_STATUSES.indexOf(this.state.scheduleunit.status) >= 0) {
            dialog.detail = "Cancelling this scheduling unit means it will no longer be executed. This action cannot be undone. Do you want to proceed?";
        }   else if (this.SU_ACTIVE_STATUSES.indexOf(this.state.scheduleunit.status) >= 0) {
            dialog.detail = "Cancelling this scheduling unit means it will be aborted. This action cannot be undone. Do you want to proceed?";
        }
        dialog.submit = this.cancelSchedulingUnit;
        dialog.width = '40vw';
        dialog.showIcon = true;
        this.setState({ dialog: dialog, dialogVisible: true });
    }
    
    /**
     * Prepare Task(s) details to show on confirmation dialog before deleting
     */
    getTaskDeleteDialogContent() {
        let selectedTasks = [];
        for (const obj of this.selectedRows) {
            selectedTasks.push({
                id: obj.id, suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name,
                taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
            });
        }
        return <>
            <DataTable value={selectedTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="suId" header="Scheduling Unit Id"></Column>
                <Column field="suName" header="Scheduling Unit Name"></Column>
                <Column field="taskId" header="Task Id"></Column>
                <Column field="controlId" header="Control Id"></Column>
                <Column field="taskName" header="Task Name"></Column>
                <Column field="status" header="Status"></Column>
            </DataTable>
        </>
    }

    /**
     * Prepare Scheduling Unit details to show on confirmation dialog
     */
    getSUDialogContent() {
        let selectedTasks = [{ suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name, suType: (this.state.scheduleunit.draft) ? 'Blueprint' : 'Draft' }];
        return <>
            <DataTable value={selectedTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="suId" header="Scheduling Unit Id"></Column>
                <Column field="suName" header="Scheduling Unit Name"></Column>
                <Column field="suType" header="Type"></Column>
            </DataTable>
        </>
    }

    /**
     * Delete Task(s)
     */
    async deleteTasks() {
        let hasError = false;
        let tasks = this.state.schedulingUnitTasks;
        for (const task of this.selectedRows) {
            if (!await TaskService.deleteTask(task.tasktype, task.id)) {
                hasError = true;
            }
            else {
                tasks = _.filter(tasks, SUtask => task.id !== SUtask.id)
            }
        }
        if (hasError) {
            appGrowl.show({ severity: 'error', summary: 'error', detail: 'Error while deleting Task(s)' });
            this.setState({ dialogVisible: false });
        } else {
            this.selectedRows = [];
            this.setState({ dialogVisible: false });
            this.init();
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Task(s) deleted successfully', schedulingUnitTasks: tasks });
        }
    }

    /**
    * Delete Scheduling Unit
    */
    async deleteSchedulingUnit() {
        let hasError = false;
        if (!await ScheduleService.deleteSchedulingUnit(this.state.scheduleunitType, this.state.scheduleunit.id)) {
            hasError = true;
        }
        if (hasError) {
            appGrowl.show({ severity: 'error', summary: 'error', detail: 'Error while deleting scheduling Unit' });
            this.setState({ dialogVisible: false });
        } else {
            this.selectedRows = [];
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Scheduling Unit is deleted successfully' });
            this.setState({ dialogVisible: false, redirect: '/schedulingunit' });
        }
    }

    fetchTaskRelationObj(producerTask, consumerTask, taskDraftRelObj, isDraft, producerRelations) {
        let taskRelAddDraftObj = [];
        const producerConnectors = _.filter(producerTask.connectors, connector => {
            const connectorConsumer = _.find(connector.consumers, {"id": consumerTask.id});
            return connectorConsumer && connectorConsumer.toAdd;
        });
        for (const producerConnector of producerConnectors) {
            let taskRelObj = _.cloneDeep(taskDraftRelObj)
            let consumerConnector = _.find(consumerTask.connector_types, connector => {
                return (connector.datatype === producerConnector.datatype && connector.iotype_value === 'input'
                        && connector.dataformat === producerConnector.dataformat
                        && connector.role === producerConnector.role);
                });
            consumerConnector = consumerConnector? consumerConnector: _.find(consumerTask.connector_types, connector => {
                return (connector.datatype === producerConnector.datatype && connector.iotype_value === 'input'
                        && connector.dataformat === producerConnector.dataformat
                        && connector.role_value === 'any');
                });
            if(producerRelations) {
                let producerDraftRelation = _.find(producerRelations, ['output_role_id', producerConnector.id])
                if(!producerDraftRelation) {
                    return [];
                }
                taskRelObj.draft = producerDraftRelation.url;
            }
            if (consumerConnector) {
                taskRelObj.name = producerTask.name
                taskRelObj.dataformat_value = consumerConnector.dataformat_value;
                taskRelObj.consumer = isDraft ? consumerTask.draft.url: consumerTask.url;
                taskRelObj.producer = isDraft ? producerTask.draft.url: producerTask.url;
                taskRelObj.input_role = `${taskRelObj.input_role}/${consumerConnector.id}`;
                taskRelObj.output_role = `${taskRelObj.output_role}/${producerConnector.id}`;
                taskRelAddDraftObj.push(taskRelObj);
            }
        }
        return taskRelAddDraftObj
    }

    async submitTaskRelationToIngest(data) {
        if(data) {
            let consumer = _.find(data.ingestGroup.ingest, {'id': data.ingestTask});           
            let taskRelationsToAdd = _.filter(data.taskRelationDraft, taskRelDraft => {return taskRelDraft.toAdd === true});
            let taskRelationsToDelete = _.filter(data.taskRelationDraft, taskRelDraft => {return taskRelDraft.toDelete === true && (_.find(taskRelDraft.consumers, {id: data.ingestTask})).canRemove === true});
            let taskRelAddDraftObj=[];
            const taskRelObj = await ScheduleService.getTaskRelationDraftAPIReqObj("all");
            let createdTaskRes=[],deletedTaskRes=[],updateTaskRelObj=[];
            let schedulingUnitTaskList = []
            if(this.props.match.params.type === 'draft') {
                schedulingUnitTaskList = this.state.scheduleunit.task_drafts;
            } else if(this.props.match.params.type === 'blueprint'){
                schedulingUnitTaskList = this.state.scheduleunit.task_blueprints;
            }
            const consumerTask = _.find(schedulingUnitTaskList, ['id', consumer.id]);
            if(this.props.match.params.type === 'blueprint') {
                consumerTask.connector_types = consumerTask.draft.connector_types;
            }
            if(taskRelationsToAdd.length) {
                const producerTasks = _.filter(schedulingUnitTaskList, draft => {
                    const matchingTask = _.find(taskRelationsToAdd, ['id', draft.id]);
                    if(matchingTask) {
                        draft.connectors = matchingTask.connectors;
                    }
                    return (matchingTask !== null && matchingTask !== undefined);
                });
                for (const producerTask of producerTasks) {
                    let taskDraftRelObj = {...taskRelObj};
                    if(this.props.match.params.type === 'blueprint') {
                        producerTask.connector_types = producerTask.draft.connector_types;
                        let producerRelations = await ScheduleService.getAllTaskRelation('draft', `producer=${producerTask.draft_id}`);
                        producerRelations = _.filter(producerRelations, ['consumer_id', consumerTask.draft_id])
                        if (producerRelations.length > 0) {
                            let taskRel = this.fetchTaskRelationObj(producerTask, consumerTask, taskDraftRelObj, false, producerRelations);
                            if(taskRel.length<1) {
                                taskDraftRelObj.blueprints = [];
                                let taskAddRel =  this.fetchTaskRelationObj(producerTask, consumerTask, taskDraftRelObj, true);
                                let createDraftRelation = await ScheduleService.createTaskRelation([...taskAddRel],[producerTask.id], 'draft');//'dataFormat':this.state.dataformat
                                if(createDraftRelation) {
                                    taskDraftRelObj = {...taskRelObj};
                                    let taskRel = this.fetchTaskRelationObj(producerTask, consumerTask, taskDraftRelObj, false, _.map(createDraftRelation, 'msg'));
                                    taskRelAddDraftObj = [...taskRelAddDraftObj, ...taskRel];
                                }
                            } else {
                                taskRelAddDraftObj = [...taskRelAddDraftObj, ...taskRel];
                            }
                            
                        }
                        else {
                            taskDraftRelObj.blueprints = [];
                            let taskAddRel =  this.fetchTaskRelationObj(producerTask, consumerTask, taskDraftRelObj, true);
                            let createDraftRelation = await ScheduleService.createTaskRelation([...taskAddRel],[producerTask.id], 'draft');//'dataFormat':this.state.dataformat
                            if(createDraftRelation) {
                                taskDraftRelObj = {...taskRelObj};
                                let taskRel = this.fetchTaskRelationObj(producerTask, consumerTask, taskDraftRelObj, false, _.map(createDraftRelation, 'msg'));
                                taskRelAddDraftObj = [...taskRelAddDraftObj, ...taskRel];
                            }
                        }
                    }
                    if (this.props.match.params.type === 'draft') {
                        taskDraftRelObj.blueprints = [];
                        let taskRel = this.fetchTaskRelationObj(producerTask, consumerTask, taskDraftRelObj);
                        taskRelAddDraftObj = [...taskRelAddDraftObj, ...taskRel];
                    }
                }        
                if(taskRelAddDraftObj) {
                    createdTaskRes = await ScheduleService.createTaskRelation(taskRelAddDraftObj,taskRelationsToAdd, this.props.match.params.type);//'dataFormat':this.state.dataformat
                    // Add newly created task relation to the existing relations list
                    for (const newRelation of createdTaskRes) {
                        if (newRelation.action === 'Added') {
                            this.ingestTaskRelations.push(newRelation.msg);
                        }
                    }
                    updateTaskRelObj=[...updateTaskRelObj,...createdTaskRes];
                }
            }
            if(taskRelationsToDelete.length) {
                let taskRelationData = this.ingestTaskRelations;
                let deletableTaskRelations = taskRelationData;
                if(this.props.match.params.type === 'draft') {
                    deletableTaskRelations = _.uniqBy(taskRelationData.filter(taskRelation => {
                        if(taskRelation.blueprints) {
                            return taskRelation.blueprints.length < 1 && taskRelation.consumer_id===data.ingestTask;
                        }
                        return false;
                    }), (taskRelation => {
                        return taskRelation.id;
                    }))
                }
                else {
                    deletableTaskRelations = _.uniqBy(taskRelationData.filter(taskRelation => {
                        return !taskRelation.blueprints && taskRelation.consumer_id===data.ingestTask;
                    }), (taskRelation => {
                        return taskRelation.id;
                    }))
                }
                let getDeleteTaskRelation= deletableTaskRelations.filter(obj1 => {
                    obj1.producer = _.find(taskRelationsToDelete, rel => rel.id === obj1.producer_id)
                    if (obj1.producer) {
                        let connectors = _.filter(obj1.producer.connectors, connector => {
                            const canDelete = _.find(connector.consumers, {id: consumer.id})?.toDelete;
                            return canDelete;
                        });
                        const connector = _.find(connectors, {id: obj1.output_role_id});
                        return connector!==undefined && connector!==null;
                    }   else {
                        return false;
                    }
                });
                deletedTaskRes = await ScheduleService.deleteTaskRelation(getDeleteTaskRelation,taskRelationsToDelete, this.props.match.params.type);
                // Remove the deleted relations from the existing relations in the local list
                for (const deletedRelation of deletedTaskRes) {
                    if (deletedRelation.action === 'Removed') {
                        let draftRelation = _.find(getDeleteTaskRelation, ['id', deletedRelation.msg]);
                        deletedRelation.data_format = _.find(consumerTask.connector_types, ['id', draftRelation.input_role_id]).dataformat_value
                        _.remove(this.ingestTaskRelations, oldRelation => {return oldRelation.id === deletedRelation.msg});
                    }
                }
                updateTaskRelObj=[...updateTaskRelObj,...deletedTaskRes];
            }
            if(updateTaskRelObj.length) {
            // To refresh the Data Product to Ingest the pop up
            //   this.getSchedulingUnitDetails (this.props.match.params.type, this.props.match.params.id);
                    this.setState({
                        ingestGroup: data.ingestGroup,
                        confirmDialogVisible: false,
                        showSpinner: true,
                        taskStatus: []
                    });
                    let taskStatus = [];
                    updateTaskRelObj.forEach((task)=>{
                        let tempTask = {};
                        tempTask['name'] = `${task.name} (${task.data_format})`;
                        tempTask['action'] = task.action;
                        taskStatus.push(tempTask);
                    });
                    this.setState({taskStatus:taskStatus});
                    this.showIcon = false;
                    this.dialogType = "success";
                    this.dialogHeader = "Status";
                    this.dialogWidth = "40vw";
                    this.dialogHeight = 'auto';
                    this.onCancel = this.cancelDialog;
                    this.onClose = this.cancelDialog;
                    this.callBackFunction = this.cancelDialog;
                    this.dialogContent = this.getTaskRelationDialogContent;
                    this.setState({isDirty : false, showSpinner: false, confirmDialogVisible: true});
            }
            else  {
                this.setState({isDirty: false, showSpinner: false});
                appGrowl.show({severity: 'error', summary: 'Warning', detail: 'No Tasks create/update '});
            }
        }
    }

    cancelDialog = () => {
        this.setState({
            confirmDialogVisible: false,
            cancelTaskReplationDialog: false,
            isDirty: false
        });
    }

    getTaskRelationDialogContent = () => {
        let taskStatus = this.state.taskStatus;
       return  <>
                    {taskStatus.length > 0 &&
                       <div style={{marginTop: '1em'}}>
                           <b>Data Products to Ingest </b>
                           <DataTable value={taskStatus} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                               <Column field="name" header="Task Name (Dataformat)"></Column>
                               <Column field="action" header="Action"></Column>
                           </DataTable>
                       </div>
                   }
               </>
   }

    /**
     * Function to cancel the scheduling unit and update its status and status of tasks if succeeeded.
     */
    async cancelSchedulingUnit() {
        let schedulingUnit = this.state.scheduleunit;
        let cancelledSU = await ScheduleService.cancelSchedulingUnit(schedulingUnit.id);
        if (!cancelledSU) {
            appGrowl.show({ severity: 'error', summary: 'error', detail: 'Error while cancelling Scheduling Unit' });
            this.setState({ dialogVisible: false });
        } else {
            schedulingUnit.status = cancelledSU.status;
            this.getActionMenu();
            let actions = this.state.actions;
            let cancelAction = _.find(actions, ['icon', 'fa-ban']);
            cancelAction.disabled = true;
            const cancelActionIndex = _.findIndex(actions, {'icon': 'fa-ban'});
            actions.splice(cancelActionIndex, 1, cancelAction);
            const cancelledSUTasks = cancelledSU.task_blueprints;
            let suTasks = this.state.schedulingUnitTasks;
            for (let suTask of suTasks) {
                const cancelledSUTask = _.find(cancelledSUTasks, {'id': suTask.id});
                suTask.status = cancelledSUTask.status;
            }
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Scheduling Unit is cancelled successfully' });
            this.setState({ dialogVisible: false, scheduleunit: schedulingUnit,
                            schedulingUnitTasks:suTasks, actions: actions});
        }
    }

    /**
     * Prepare Task(s) details to show on confirmation dialog before cancelling
     */
    getTaskCancelConfirmContent() {
        let selectedTasks = [];
        let ignoredTasks = [];
        for (const obj of this.selectedRows) {
            if (this.TASK_END_STATUSES.indexOf(obj.status) < 0) {
                selectedTasks.push({
                    id: obj.id, suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name,
                    taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
                });
            }   else {
                ignoredTasks.push({
                    id: obj.id, suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name,
                        taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
                });
            }
        }
        return <>
            <div style={{marginTop: '1em'}}>
                <b>Task(s) that can be cancelled</b>
                <DataTable value={selectedTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="suId" header="Scheduling Unit Id"></Column>
                    <Column field="suName" header="Scheduling Unit Name"></Column>
                    <Column field="taskId" header="Task Id"></Column>
                    <Column field="controlId" header="Control Id"></Column>
                    <Column field="taskName" header="Task Name"></Column>
                    <Column field="status" header="Status"></Column>
                </DataTable>
            </div>
            {ignoredTasks.length > 0 &&
            <div style={{marginTop: '1em'}}>
                <b>Task(s) that will be ignored</b>
                <DataTable value={ignoredTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="suId" header="Scheduling Unit Id"></Column>
                    <Column field="suName" header="Scheduling Unit Name"></Column>
                    <Column field="taskId" header="Task Id"></Column>
                    <Column field="controlId" header="Control Id"></Column>
                    <Column field="taskName" header="Task Name"></Column>
                    <Column field="status" header="Status"></Column>
                </DataTable>
            </div>
            }
        </>
    }

    /**
     * Content for Confirmation dialog to mark task as obsolete
     */
    getObsTaskConfirmDlgContent() {
        const blueprintsWithObsolete = this.state.blueprintsWithObsolete;
        const blueprintWithoutObsolete = _.difference(this.selectedRows, blueprintsWithObsolete);
        let selectedTasksWithObsolete = [];
        let selectedTasksWithoutObsolete = [];
        for (const obj of blueprintsWithObsolete) {           
            selectedTasksWithObsolete.push({
                id: obj.id, suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name,
                taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
            });       
        }
        for (const obj of blueprintWithoutObsolete) {           
            selectedTasksWithoutObsolete.push({
                id: obj.id, suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name,
                taskId: obj.id, controlId: obj.subTaskID, taskName: obj.name, status: obj.status
            });       
        }
        return <>
            <div style={{marginTop: '1em'}}>
                {selectedTasksWithoutObsolete.length>0 &&
                    <>
                        <b>Task(s) that can be marked as obsolete</b>
                        <DataTable value={selectedTasksWithoutObsolete} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="suId" header="Scheduling Unit Id"></Column>
                            <Column field="suName" header="Scheduling Unit Name"></Column>
                            <Column field="taskId" header="Task Id"></Column>
                            <Column field="controlId" header="Control Id"></Column>
                            <Column field="taskName" header="Task Name"></Column>
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>}
                {selectedTasksWithObsolete.length>0 &&
                    <>
                        <b>Task(s) that are marked as obsolete will be ignored</b>
                        <DataTable value={selectedTasksWithObsolete} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="suId" header="Scheduling Unit Id"></Column>
                            <Column field="suName" header="Scheduling Unit Name"></Column>
                            <Column field="taskId" header="Task Id"></Column>
                            <Column field="controlId" header="Control Id"></Column>
                            <Column field="taskName" header="Task Name"></Column>
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>}
            </div>
        </>
    }

    /**
     * Prepare Task(s) details to show status of Task cancellationn
     */
    getTaskCancelStatusContent() {
        let cancelledTasks = this.state.cancelledTasks;
        return <>
            <DataTable value={cancelledTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="suId" header="Scheduling Unit Id"></Column>
                <Column field="suName" header="Scheduling Unit Name"></Column>
                <Column field="taskId" header="Task Id"></Column>
                <Column field="controlId" header="Control Id"></Column>
                <Column field="taskName" header="Task Name"></Column>
                <Column field="status" header="Status"></Column>
            </DataTable>
        </>
    }

    /**
     * Content to show status after marking task as obsolete
     */
    getObsTaskStatusContent() {
        let obsoleteTasks = this.state.obsoleteTasks;
        return <>
            <DataTable value={obsoleteTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                <Column field="suId" header="Scheduling Unit Id"></Column>
                <Column field="suName" header="Scheduling Unit Name"></Column>
                <Column field="taskId" header="Task Id"></Column>
                <Column field="controlId" header="Control Id"></Column>
                <Column field="taskName" header="Task Name"></Column>
                <Column field="status" header="Status"></Column>
                <Column field="actionStatus" header="Action Status"></Column>
            </DataTable>
        </>
    }

    /**
     * To check if selected tasks are marked as obsolete or not
     * @param {Array} selectedBlueprints 
     * @returns Array - List of tasks which have obsolete_since as null
     */
    checkObsolete(selectedBlueprints) {
        const blueprintsWithObsolete = selectedBlueprints.filter(task => {
            return task.obsolete}); 
        return blueprintsWithObsolete;
    }

    /**
     * Function to get confirmation before cancelling all selected task blueprints if the task status is 
     * not one of the end statuses. If no selected task is cancellable, show info to select a cancellable task.
     * 
     */
    confirmCancelTasks() {
        this.pageUpdated = false;
        let selectedBlueprints = this.selectedRows.filter(task => {
                                        return task.tasktype === 'Blueprint' && 
                                        this.TASK_END_STATUSES.indexOf(task.status)<0});
        if (this.selectedRows.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', 
                            detail: 'Select at least one cancellable Task Blueprint to cancel.' });
        } else {
            if(selectedBlueprints.length === 0) {
                appGrowl.show({ severity: 'warn', summary: 'Already Cancelled', 
                            detail: 'Selected task(s) are already cancelled' });
            }
            else {
                let dialog = this.state.dialog;
                dialog.type = "confirmation";
                dialog.header = "Confirm to Cancel Task(s)";
                dialog.actions = [{ id: 'yes', title: 'Cancel Tasks', callback: this.cancelTasks, style:{"background": "#e91224","border": "1px solid #e91224"} },
                                    { id: 'no', title: 'No', className:'act-btn-cancel', callback: this.closeDialog }];
                dialog.detail = "Cancelling the task means it will no longer be executed / will be aborted. This action cannot be undone. Do you want to proceed?";
                dialog.content = this.getTaskCancelConfirmContent;
                dialog.submit = this.cancelTasks;
                dialog.width = '55vw';
                dialog.showIcon = false;
                this.setState({ dialog: dialog, dialogVisible: true, cancelledBlueprint: selectedBlueprints });
            }
        }
    }

    /**
     * Function to get conirmation before marking task(s) as obsolete
     */
    confirmObsoleteTasks() {
        this.pageUpdated = false;
        let selectedBlueprints = this.selectedRows.filter(task => {
            return task.tasktype === 'Blueprint';
        });
        const blueprintsWithObsolete = this.checkObsolete(selectedBlueprints);
        const blueprintWithoutObsolete = _.difference(selectedBlueprints, blueprintsWithObsolete);
        if (selectedBlueprints.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', 
            detail: 'Please select minimum one task to mark as obsolete' });
        } else {
            if (blueprintWithoutObsolete.length === 0) {
                appGrowl.show({ severity: 'warn', summary: 'Already marked as obsolete', 
                detail: 'Selected task(s) are already marked as obsolete' });
            }
            else {
                let dialog = this.state.dialog;
                dialog.type = "confirmation";
                dialog.header = "Confirm to mark task(s) as obsolete";
                dialog.actions = [{ id: 'yes', title: 'Mark as obsolete', className:'act-btn-dispose', callback: this.markObsoleteTasks },
                                    { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
                dialog.detail = "Warning: Obsolete task(s) will not be copied and dataproducts will not be used for pipelines and ingest. Are you sure you want to make the task(s) obsolete?";
                dialog.content = this.getObsTaskConfirmDlgContent;
                dialog.submit = this.markObsoleteTasks;
                dialog.width = '55vw';
                dialog.showIcon = false;
                this.setState({ dialog: dialog, dialogVisible: true, blueprintsWithObsolete: blueprintsWithObsolete});
            }
        }
    }

    /**
     * Function to mark task(s) as obsolete and show the status after completion
     */
    async markObsoleteTasks() {
        let schedulingUnitTasks = this.state.schedulingUnitTasks;
        const blueprintWithoutObsolete = _.difference(this.selectedRows, this.state.blueprintsWithObsolete);
        let obsoleteTask = []
        for (const selectedTask of blueprintWithoutObsolete) {           
            const obsTask = await TaskService.markTaskObsolete(selectedTask.id);
            let task = _.find(schedulingUnitTasks, {'id': selectedTask.id, tasktype: 'Blueprint'});
            if (obsTask) {
                task.status = obsTask.status;
                task.obsolete_since = obsTask.obsolete_since;
                task.obsolete = obsTask.obsolete_since!==null;
            }
            obsoleteTask.push({
                id: task.id, suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name,
                taskId: task.id, controlId: task.subTaskID, taskName: task.name, 
                status: task.status.toLowerCase()==='obsolete'?'Obsolete': task.status,
                actionStatus: obsTask?'Success': 'Failed'
            });
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Mark task(s) as obsolete - Status";
        dialog.actions = [{ id: 'no', title: 'Ok', callback: this.closeDialog }];
        dialog.detail = ""
        dialog.content = this.getObsTaskStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ schedulingUnitTasks: schedulingUnitTasks, obsoleteTasks: obsoleteTask, dialog: dialog, dialogVisible: true });
    }

    /**
     * Function to cancel all selected task blueprints if the task status is not one of the end statuses
     * and update their status on successful cancellation.
     */
    async cancelTasks() {
        let schedulingUnitTasks = this.state.schedulingUnitTasks;
        let selectedBlueprints = this.selectedRows.filter(task => {return task.tasktype === 'Blueprint'});
        let cancelledTasks = []
        for (const selectedTask of selectedBlueprints) {
            if (this.TASK_END_STATUSES.indexOf(selectedTask.status) < 0) {
                const cancelledTask = await TaskService.cancelTask(selectedTask.id);
                let task = _.find(schedulingUnitTasks, {'id': selectedTask.id, tasktype: 'Blueprint'});
                if (cancelledTask) {
                    task.status = cancelledTask.status;
                    this.init();
                }
                cancelledTasks.push({
                    id: task.id, suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name,
                    taskId: task.id, controlId: task.subTaskID, taskName: task.name, 
                    status: task.status.toLowerCase()==='cancelled'?'Cancelled': 'Error Occured'
                });
            }
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Cancel Task(s) Status";
        dialog.actions = [{ id: 'no', title: 'Ok', callback: this.closeDialog }];
        dialog.detail = ""
        dialog.content = this.getTaskCancelStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ schedulingUnitTasks: schedulingUnitTasks, cancelledTasks: cancelledTasks, dialog: dialog, dialogVisible: true });
    }

    /**
     * Function to show confirmation before create Clean-Up Task
     */
    async cleanUpSUTask() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header= "Confirm to Create Clean-up Task";
        dialog.detail = "Do you want to create Clean-up task for this Scheduling Unit Blueprint?";
        dialog.content = this.getCleanUpDialogContent;
        dialog.actions = [{id: 'yes', title: 'Create Clean-up Task', callback: this.createSUCleanupTask},
        {id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog}];
        dialog.onSubmit = this.createSUCleanupTask;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }

    /**
     * Function to show confirmation before add Inguest Task
     */
     async confirmAddIngestTask() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header= "Confirm to Add Injest Task";
        dialog.detail = "Do you want to add a new Injest task for this Scheduling Unit Draft?";
        dialog.content = this.getCleanUpDialogContent;
        dialog.actions = [{id: 'yes', title: 'Add', callback: this.addNewIngestTask},
        {id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog}];
        dialog.onSubmit = this.addNewIngestTask;
        dialog.width = '45vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }   

    /**
     * Prepare dialog content before create clean-up task
     * @returns : return dialog content
     */
    getCleanUpDialogContent() {
        let selectedSchedulingUnits = [{suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name}];
        return  <>
                    {selectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <DataTable value={selectedSchedulingUnits} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                                <Column field="suId" header="Id"></Column>
                                <Column field="suName" header="Name"></Column>
                            </DataTable>
                        </div>
                    }
                </>
    }

    /**
     * Function to create Clean-up task
     */
    async createSUCleanupTask() {
        let dialog = this.state.dialog;
        this.statusUpdate = null;
        let response = null;
        try{
            response = await ScheduleService.createSUCleanupTask(this.state.scheduleunit); 
            this.statusUpdate =  response.message === 'Success'? "Success":"Failed";
        }   catch(error) {
            this.statusUpdate = "Failed";
            console.log('Error While creating Clean-up task', error);
        }
        dialog.detail = '';
        dialog.type = response.message === 'Success' ? "success":"warning";
        dialog.header= "Create Clean-up Task Status";
        dialog.content = this.getStatusContent;
        dialog.actions = [{id: 'no', title: 'Close', callback: this.closeDialog}];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }

    /**
     * Prepare new Ingest Task details for current Scheduling Unit
     * @param {*} existingIngestTasks 
     */
    async prepareIngestTask(existingIngestTasks) {
        let ingestTaskTemplate = _.find(this.taskTemplates, {'name': 'ingest'});
        let newIngestTask = { name: "Ingest"} ;
        if (existingIngestTasks && existingIngestTasks.length>0) {
            ingestTaskTemplate = _.find(this.taskTemplates, {'id': existingIngestTasks[0].specifications_template.id});
            newIngestTask = _.cloneDeep(existingIngestTasks[0]);
            let taskNamePrefix = existingIngestTasks[0].name;
            taskNamePrefix = taskNamePrefix.replace( /[^\d]*/g, '');
            taskNamePrefix = taskNamePrefix === '' ? '1' : (parseInt(taskNamePrefix)+1);
            newIngestTask['name'] = `Ingest ${taskNamePrefix}`;
        }   else {
            newIngestTask['description'] = "Ingest all preprocessed dataproducts";
            newIngestTask['specifications_doc'] = {
                "$schema": ingestTaskTemplate.ref_resolved_schema['$id']
            };
        }
        newIngestTask['specifications_template'] = ingestTaskTemplate.url;
        newIngestTask.produced_by = [];
        newIngestTask.consumed_by = [];
        newIngestTask.task_blueprints = [];
        newIngestTask.first_scheduling_relation = [];
        newIngestTask.second_scheduling_relation = [];
        newIngestTask.scheduling_unit_draft = this.state.scheduleunit.url;
        return newIngestTask;
    }

    /**
     * Add new Ingest task
     * @returns 
     */
    async addNewIngestTask() {
        let response = null;
        let ingestTasks = _.filter(this.state.scheduleunit.task_drafts, {task_type : 'ingest'});
        ingestTasks = _.orderBy(ingestTasks, "id", "desc");        
        let newIngestTask = await this.prepareIngestTask(ingestTasks);        
        try{
            response = await TaskService.createTask(newIngestTask);
            if (response.statusText === 'Created' ) {
                newIngestTask = response.data;
                if (ingestTasks[0]) {
                    let newIngestTaskRelations = _.cloneDeep(_.uniqBy(_.filter(this.ingestTaskRelations, {'consumer_id': ingestTasks[0].id}), "id"));
                    for (let newIngestTaskRelation of newIngestTaskRelations) {
                        newIngestTaskRelation.consumer_id = newIngestTask.id;
                        newIngestTaskRelation.consumer = newIngestTask.url;
                        newIngestTaskRelation.producer = _.find(this.state.scheduleunit.task_drafts, {id : newIngestTaskRelation.producer_id}).url;
                        newIngestTaskRelation.blueprints = [];
                        await ScheduleService.addTaskRelation(newIngestTaskRelation,newIngestTask.name, 'draft');
                    }
                }
                appGrowl.show({ severity: 'success', summary: 'Success', detail: 'New Ingest Task is added successfully!' });
                this.setState({dialogVisible:false, openTaskRelationDialog: true});
            }   else {
                appGrowl.show({ severity: 'error', summary: 'Add Ingest Task - Status', detail: 'Add new Ingest Task has Failed' });
            }
        }   catch(error) {
            this.statusUpdate = "Failed";
            console.log('Error While add Ingest Task', error);
        }
        await this.init();
    }

    /**
     * Close the Add Ingect Task Dialog
     */
    closeIngestDialog() {
        this.setState({showAddIngestDialog : false});
        this.init();
    }

    /**
     * Preparedialog content for after create Clean-up task
     * @returns : dialog content for clean-up task status
     */
    getStatusContent() {
        let suContent = [{suId:this.state.scheduleunit.id, suName: this.state.scheduleunit.name, status:this.statusUpdate}]
        return  <> 
                    <div style={{marginTop: '1em'}}>
                        <DataTable value={suContent} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                            <Column field="suId" header="Id"></Column>
                            <Column field="suName" header="Name"></Column>
                            <Column field="status" header="Action Status"></Column>
                        </DataTable>
                    </div>
                </>
    }

    getStatusForCopiedShedulingUnit(){
        let suContent = [{suId:this.state.scheduleunit.id, suName: this.state.scheduleunit.name, status:this.statusUpdate}]
        return  <> 
                    <div style={{marginTop: '1em'}}>
                        <DataTable value={suContent} resizableColumns columnResizeMode="expand" className="card" style={{paddingLeft: '0em'}}>
                            <Column field="suId" header="Id"></Column>
                            <Column field="suName" header="Name"></Column>
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </div>
                    {
                        this.statusUpdate === 'Success' ?
                        <div style = {{ marginTop: '1em'}} >
                        Do you want to open the copied Scheduling Unit draft ?
                        </div> : null
                    }
                   
                </>
     }

     /**
      * Confirm Autodeletion before update the value
      */
     confirmAutoDeletion() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header= "Confirm Pin/Unpin data after ingest";
        dialog.detail = this.state.scheduleunit.output_pinned ? 'Do you want to Unpin data after ingest?' : 'Do you want to Pin data after ingest?';
        dialog.content = '';
        dialog.actions = [{id: 'yes', title: this.state.scheduleunit.output_pinned?'Unpin data': 'Pin data', 
        className: this.state.scheduleunit.output_pinned?'act-btn-dispose':'', callback: this.setAutoDeletion},
        {id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog}];
        dialog.onSubmit = this.setAutoDeletion;
        dialog.width = '35vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
     }

     /**
      * Enable/Disable autodeletion in the scheduling unit
      */
    async setAutoDeletion() {
        let suCopy = _.cloneDeep(this.state.scheduleunit);
        let tasks = [];
        let updatedResSU = await ScheduleService.updateSchedulingUnitValue(this.props.match.params.type, 
            {id: this.state.scheduleunit.id, 'output_pinned': !this.state.scheduleunit.output_pinned}, tasks);
        if (updatedResSU) {
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'Pin/Unpin data after ingest updated successfully' });
            suCopy['output_pinned'] = updatedResSU.output_pinned;
            let index = _.indexOf(this.actions, _.find(this.actions, {'icon' :'fa-thumbtack'}));
            this.actions.splice(index, 1, { icon: 'fa-thumbtack', title: updatedResSU.output_pinned? 'Unpin data after ingest' : 'Pin data after ingest', 
            type: 'button', actOn: 'click', props: { callback: this.confirmAutoDeletion } }); 
            this.setState({scheduleunit: suCopy, actions: this.actions, dialogVisible: false});
        }   else {
            appGrowl.show({ severity: 'error', summary: 'Failed', detail: 'Unable to update Automatic Deletion' });
            this.setState({dialogVisible: false});
        }
    }

    getTaskRestartConfirmContent() {
        const taskWithOnhold = this.state.selectedOnHoldRows;
        const taskWithoutOnhold = _.difference(this.selectedRows, taskWithOnhold);
        let selectedTaskWithOnhold = [];
        let selectedTasksWithoutOnhold = [];
        for (const obj of taskWithOnhold) {           
            selectedTaskWithOnhold.push({
                    id: obj.id, type: obj.type, status: obj.status
                });           
        }
        for (const obj of taskWithoutOnhold) {           
            selectedTasksWithoutOnhold.push({
                    id: obj.id, type: obj.type, status: obj.status
                });           
        }
        this.selectedRows = [];
        return <>
            <div style={{marginTop: '1em'}}>
                {selectedTaskWithOnhold.length>0 &&
                    <>
                        <b>Task(s) that will be queued to restart</b>
                        <DataTable value={selectedTaskWithOnhold} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="id" header="Task Id"></Column>                
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>} 
                {selectedTasksWithoutOnhold.length>0 &&
                    <>
                        <b>Task(s) with status other than &apos;on hold&apos; will be ignored</b>
                        <DataTable value={selectedTasksWithoutOnhold} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                            <Column field="id" header="Task Id"></Column>            
                            <Column field="status" header="Status"></Column>
                        </DataTable>
                    </>}
                  
            </div>
            </>
    }

    
    showTaskRestartConfirmation() {
        this.pageUpdated = false;
        let selectedOnHoldRows = this.selectedRows.filter(task => {
            return task.status === 'on hold'});
        if(this.selectedRows.length === 0) {
            appGrowl.show({ severity: 'info', summary: 'Select Row', 
                            detail: 'Please select minimum one task to restart' });
        } else {
            if (selectedOnHoldRows.length === 0) {
                appGrowl.show({ severity: 'warn', summary: 'Select at least one task with status on hold', 
                            detail: 'Selected task(s) cannot be queued to restart' });
            }
            else {
                let dialog = this.state.dialog;
                dialog.type = "confirmation";
                dialog.header = "Confirm to restart Task(s)";
                dialog.actions = [{ id: 'yes', title: 'Restart', className:'act-btn-dispose', callback: this.restartTask },
                                    { id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog }];
                dialog.detail = "Below tasks will be queued to restart";
                dialog.content = this.getTaskRestartConfirmContent;
                dialog.submit = this.restartTask;
                dialog.width = '55vw';
                dialog.showIcon = false;
                this.setState({ dialog: dialog, dialogVisible: true,  selectedOnHoldRows: selectedOnHoldRows });   
            }
                          
        }
    }

    async restartTask() {
        let tasks = this.state.schedulingUnitTasks;
        let restartTasks = []
        for (const selectedTask of _.cloneDeep(this.state.selectedOnHoldRows)) {
                const restartTask = await TaskService.restartTaskOnHold(selectedTask.id);
                let task = _.find(tasks, {'id': selectedTask.id});
                restartTasks.push({
                    id: task.id,
                    actionStatus:restartTask && restartTask.status === 200 ?'Success':'Failed'
                });            
        }
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header = "Restart Task (s) - Status";
        dialog.actions = [{ id: 'no', title: 'Ok', callback: this.closeDialog }];
        dialog.detail = ""
        dialog.content = this.getTaskRestartStatusContent;
        dialog.submit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.selectedRows = [];
        this.setState({ restartTasks: restartTasks, dialog: dialog, dialogVisible: true });        
    }

    getTaskRestartStatusContent() {
        this.selectedRows = [];
        return <>
            <div style={{marginTop: '1em'}}>
                <DataTable value={this.state.restartTasks} resizableColumns columnResizeMode="expand" className="card" style={{ paddingLeft: '0em' }}>
                    <Column field="id" header="Id"></Column>  
                    <Column field="actionStatus" header="Action Status"></Column>
                </DataTable>
            </div>
            </>
    }

    /**
     * Function to show confirmation before create Re-run Failed Tasks
     */
     async confirmCopyFailedTasks() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header= "Confirm to re-run all failed tasks";
        dialog.detail = "Do you want to re-run all failed tasks of this Scheduling Unit Blueprint?";
        dialog.content = this.getCopyFailedTasksDlgContent;
        dialog.actions = [{id: 'yes', title: 'Re-run', callback: this.copyFailedTasks},
        {id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog}];
        dialog.onSubmit = this.copyFailedTasks;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }

    /**
     * Prepare dialog content before create Re-run failed tasks
     * Also this function re-used for Update SUB
     * @returns : return dialog content
     */
     getCopyFailedTasksDlgContent() {
        let selectedSchedulingUnits = [{suId: this.state.scheduleunit.id, suName: this.state.scheduleunit.name, suStatus: this.state.scheduleunit.status}];
        return  <>
                    {selectedSchedulingUnits.length > 0 &&
                        <div style={{marginTop: '1em'}}>
                            <DataTable value={selectedSchedulingUnits} resizableColumns columnResizeMode="expand" 
                            className="card" style={{paddingLeft: '0em', width: '90%'}}>
                                <Column field="suId" header="Id" style={{width: '30%'}}></Column>
                                <Column field="suName" header="Name"></Column>
                                <Column field="suStatus" header="Status"></Column>
                            </DataTable>
                        </div>
                    }
                </>
    }

    /**
     * Function to create Re-run failed tasks
     */
    async copyFailedTasks() {
        let dialog = this.state.dialog;
        this.statusUpdate = null;
        let response = null;
        try{
            this.setState({showSpinner: true});
            response = await ScheduleService.copySUFailedTasks(this.state.scheduleunit); 
            this.statusUpdate =  response.message === 'Success'? "Success":"Failed";
            await this.init();
        }   catch(error) {
            this.statusUpdate = "Failed";
            console.log('Error while re-run all failed tasks', error);
        }
        dialog.detail = '';
        dialog.type = response.message === 'Success' ? "success":"warning";
        dialog.header= "Re-run all failed tasks - Status";
        dialog.content = this.getStatusContent;
        dialog.actions = [{id: 'no', title: 'Close', callback: this.closeDialog}];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({showSpinner: false, dialog: dialog, dialogVisible: true});
    }


    /**
     * Function to show confirmation before update Scheduling Unit Blueprint
     */
     async confirmUpdateSUBlueprint() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header= "Confirm to update blueprints from draft";
        dialog.detail = "Do you want to update blueprints of the scheduling unit according to the changed and added task drafts?";
        dialog.content = this.getCopyFailedTasksDlgContent;
        dialog.actions = [{id: 'yes', title: 'Update Blueprint', callback: this.updateSUB},
        {id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog}];
        dialog.onSubmit = this.updateSUB;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }
   
    /**
     * Function to update SUB
     */
     async updateSUB() {
        let dialog = this.state.dialog;
        this.statusUpdate = null;
        let response = null;
        try{
            response = await ScheduleService.updateSUBTasks(this.state.scheduleunit); 
            this.statusUpdate =  response.message === 'Success'? "Success":"Failed";
            await this.init();
        }   catch(error) {
            this.statusUpdate = "Failed";
            console.log('Error while updating Scheduling Unit Blueprint from Task Drafts', error);
        }
        dialog.detail = '';
        dialog.type = response.message === 'Success' ? "success":"warning";
        dialog.header= "Update blueprints from draft - Status";
        dialog.content = this.getStatusContent;
        dialog.actions = [{id: 'no', title: 'Close', callback: this.closeDialog}];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }

    /**
     * Function to show confirmation before copy specification doc for failed task
     */
     async confirmCopySpecAndFailedTasks() {
        let dialog = this.state.dialog;
        dialog.type = "confirmation";
        dialog.header= "Confirm to create draft copy for failed tasks";
        dialog.detail = "Do you want to create draft copy for failed tasks of this Scheduling Unit Blueprint?";
        dialog.content = this.getCopyFailedTasksDlgContent;
        dialog.actions = [{id: 'yes', title: 'Create draft tasks', callback: this.copySpecAndFailedTasks},
        {id: 'no', title: 'Cancel', className:'act-btn-cancel', callback: this.closeDialog}];
        dialog.onSubmit = this.copySpecAndFailedTasks;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }

    /**
     * Function to copy Specification Doc for Failed task
     */
    async copySpecAndFailedTasks() {
        let dialog = this.state.dialog;
        this.statusUpdate = null;
        let response = null;
        try{
            response = await ScheduleService.copySUSpecAndFailedTasks(this.state.scheduleunit); 
            this.statusUpdate =  response.message === 'Success'? "Success":"Failed";
        }   catch(error) {
            this.statusUpdate = "Failed";
            console.log('Error while copy Specification Doc for Failed Task', error);
        }
        dialog.detail = '';
        dialog.type = response.message === 'Success' ? "success":"warning";
        dialog.header= "Create copy for failed tasks - Status";
        dialog.content = this.getStatusContent;
        dialog.actions = [{id: 'no', title: 'Close', callback: this.closeDialog}];
        dialog.onSubmit = this.closeDialog;
        dialog.width = '55vw';
        dialog.showIcon = false;
        this.setState({dialog: dialog, dialogVisible: true});
    }
    
    /**
     * Function to call wnen websocket is connected
     */
    onConnect() {
        try{
            console.log("WS Opened");
            const userDets = localStorage.getItem("user");
            if (userDets) {
                this.websocket.sendMessage(JSON.stringify({"token": JSON.parse(userDets).websocket_token}));
                console.log("Auth token submitted");
            }
        }catch (err) {
            console.log('err',err)
        }
    }

    /**
     * Function to call when websocket is disconnected
     */
    onDisconnect() {
        console.log("WS Closed")
    }

    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    handleData(data) {
        if (data) {
            console.log('received websocket data:', data)
            const jsonData = JSON.parse(data);
            this.refreshSchedulingUnit(jsonData);
        }
    }

    /**
     * Refresh the SU Draft details
     * @param {*} jsonData 
     */
    refreshSchedulingUnit(jsonData) {
        let schedule_type = 'blueprint';
        let suID = 0;
        if (jsonData.object_type === 'scheduling_unit_draft' && this.props.match.params.type === 'draft') {
            schedule_type = 'draft';
            if ((jsonData.object_details.id+'') === (this.props.match.params.id+'')) {
                if (jsonData.action === 'delete') {
                    appGrowl.show({ severity: 'error', summary: `error`, sticky: true, detail: `This Scheduling Unit Draft [${jsonData.object_details.id}] is deleted by another user and will not be available for further action.`});
                    this.getActionMenu(null, null, true);
                    return;
                }  else {
                    suID = jsonData.object_details.id;
                }
            }
        }   else if (jsonData.object_type === 'scheduling_unit_draft' && this.props.match.params.type === 'blueprint') {
            if (jsonData.object_details.id === this.state.scheduleunit.draft_id) {
                suID = this.props.match.params.id;
            }
        }   else if (jsonData.object_type === 'scheduling_unit_blueprint' && this.props.match.params.type === 'blueprint') {
            if ((jsonData.object_details.id+'') === this.props.match.params.id) {
               suID = this.props.match.params.id;
            }
        }
        this.getSchedulingUnitDetails( schedule_type, suID);
    }

    render() {
        if (this.state.redirect) {
            return <Redirect to={{ pathname: this.state.redirect }}></Redirect>
        }
        const userPermissions = this.state.userPermission.permissions;
        let jeditor = null;
        if (this.state.paramsSchema) {
            jeditor = React.createElement(Jeditor, {title: "Task Parameters",
                                                        schema: this.state.paramsSchema,
                                                        initValue: this.state.paramsOutput, 
                                                        // callback: this.setEditorOutput,
                                                        disabled: true,
                                                        parentFunction: this.setEditorFunction,
                                                        bandPassFilter: this.state.bandPassFilter
                                                    });
        }
        const obsStrategy = this.state.scheduleunit?.observation_strategy_template;
        if (obsStrategy) {
            ParserUtility.addStationParameters(obsStrategy);
        }
        return (
            <>
            {userPermissions.scheduleunit.list ?
            <>
                <PageHeader location={this.props.location} title={'Scheduling Unit - Details'}
                    actions={this.state.actions} />
                { this.state.isLoading ? <AppLoader /> : this.state.scheduleunit &&
                    <>
                        <div className="main-content">
                            <div className="p-grid">
                                <label className="col-lg-2 col-md-2 col-sm-12">Strategy Name</label>
                                <span className="p-col-lg-4 col-md-4 col-sm-12">{obsStrategy.name} ({obsStrategy.purpose_value} - {obsStrategy.state_value} - v{obsStrategy.version})</span>
                                <label className="col-lg-2 col-md-2 col-sm-12">Strategy Description</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{obsStrategy.description}</span>
                            </div>
                            <div className="p-grid">
                                <label className="col-lg-2 col-md-2 col-sm-12">Name</label>
                                <span className="p-col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.name}</span>
                                <label className="col-lg-2 col-md-2 col-sm-12">Description</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.description}</span>
                            </div>
                            <div className="p-grid">
                                <label className="col-lg-2 col-md-2 col-sm-12">Created At</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.created_at && moment(this.state.scheduleunit.created_at, moment.ISO_8601).format(UIConstants.CALENDAR_DATETIME_FORMAT)}</span>
                                <label className="col-lg-2 col-md-2 col-sm-12">Updated At</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.created_at && moment(this.state.scheduleunit.updated_at, moment.ISO_8601).format(UIConstants.CALENDAR_DATETIME_FORMAT)}</span>
                            </div>
                            {this.props.match.params.type === 'blueprint' &&
                            <div className="p-grid">
                                <label className="col-lg-2 col-md-2 col-sm-12">Observation Start Time</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.on_sky_start_time && moment(this.state.scheduleunit.on_sky_start_time).format(UIConstants.CALENDAR_DATETIME_FORMAT)}</span>
                                <label className="col-lg-2 col-md-2 col-sm-12">Observation End Time</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.on_sky_stop_time && moment(this.state.scheduleunit.on_sky_stop_time).format(UIConstants.CALENDAR_DATETIME_FORMAT)}</span>
                            </div>}
                            <div className="p-grid">
                                <label className="col-lg-2 col-md-2 col-sm-12" >Duration (HH:mm:ss)</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{moment.utc((this.state.scheduleunit.duration ? this.state.scheduleunit.duration : 0) * 1000).format(UIConstants.CALENDAR_TIME_FORMAT)}</span>
                                <label className="col-lg-2 col-md-2 col-sm-12">Strategy ID</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.observation_strategy_template_id}</span>
                            </div>
                            <div className="p-grid">
                                {this.state.scheduleunit.scheduling_set && this.state.scheduleunit.scheduling_set.project_id &&
                                    <>
                                        <label className="col-lg-2 col-md-2 col-sm-12">Project</label>
                                        <span className="col-lg-4 col-md-4 col-sm-12">
                                            <Link to={`/project/view/${this.state.scheduleunit.scheduling_set.project_id}`}>{this.state.scheduleunit.scheduling_set.project_id}</Link>
                                        </span>
                                    </>
                                }
                                <label className="col-lg-2 col-md-2 col-sm-12">Scheduling set</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.scheduling_set && this.state.scheduleunit.scheduling_set.name}</span>
                            </div>
                            <div className="p-grid">
                                <label className="col-lg-2 col-md-2 col-sm-12" >Rank</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.draft?this.state.scheduleunit.draft.rank: this.state.scheduleunit.rank}</span>
                                <label className="col-lg-2 col-md-2 col-sm-12">Priority Queue</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">{this.state.scheduleunit.draft? this.state.scheduleunit.draft.priority_queue_value : this.state.scheduleunit.priority_queue_value}</span>
                            </div>
                            <div className="p-grid">
                                <label className="col-lg-2 col-md-2 col-sm-12">{this.props.match.params.type === 'blueprint' ? 'Draft' : 'Blueprints'}</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">
                                    <ul className="task-list">
                                        {(this.state.scheduleunit.scheduling_unit_blueprints || []).map(blueprint => (
                                            <li key={blueprint.id}>
                                                <Link to={{ pathname: `/schedulingunit/view/blueprint/${blueprint.id}` }}>{blueprint.name}</Link>
                                            </li>))}
                                        {this.state.scheduleunit.draft &&
                                            <li>
                                                <Link to={{ pathname: `/schedulingunit/view/draft/${this.state.scheduleunit.draft.id}` }}>
                                                    {this.state.scheduleunit.draft.name}
                                                </Link>
                                            </li>}
                                    </ul>
                                </span>
                                {this.props.match.params.type === 'blueprint' &&
                                    <label className="col-lg-2 col-md-2 col-sm-12 ">Status</label>}
                                {this.props.match.params.type === 'blueprint' &&
                                    <div className="col-lg-4 col-md-2 col-sm-12" style={{marginBottom: '10px'}}>
                                        <span className={"tag-content p-chips-token-label su-"+this.state.scheduleunit.status}>{this.state.scheduleunit.status}</span>                                        
                                        {this.state.scheduleunit.status === 'unschedulable' &&
                                            <span style={{marginLeft: '5px'}} > ({this.state.scheduleunit.unschedulable_reason})</span>
                                        }
                                    </div>
                                }
                            </div> 
                            <div className="p-grid">
                                <label className="col-lg-2 col-md-2 col-sm-12">Tags</label>
                                <Chips className="p-col-4 chips-readonly" disabled value={this.state.scheduleunit.tags}></Chips>
                                <label className="col-lg-2 col-md-2 col-sm-12">Pin Data</label>
                                <TriStateCheckbox  
                                    disabled={true} 
                                    className="p-checkbox-disabled col-lg-4 col-md-4 col-sm-12"
                                    value={this.state.scheduleunit.output_pinned}>
                                </TriStateCheckbox> 
                            </div>
                            {this.props.match.params.type === 'blueprint' && this.state.isIngestPresent && this.state.scheduleunit.task_blueprints &&
                            <div className="p-grid">
                                <label className="col-lg-2 col-md-2 col-sm-12" >Ingest Completeness</label>
                                <span className="col-lg-4 col-md-4 col-sm-12">
                                    <IngestProgress tasks={this.state.scheduleunit.task_blueprints} />
                                </span>
                            </div>
                            }
                        </div>
                    </>
                }

                <div>
                    <h3>Tasks Details</h3>
                </div>
                {/*
                    * Call View table to show table data, the parameters are,
                     data - Pass API data
                    defaultcolumns - These columns will be populate by default in table with header mentioned
                    optionalcolumns - These columns will not appear in the table by default, but user can toggle the columns using toggle menu
                    showaction - {true/false} -> to show the action column
                    keyaccessor - This is id column for Action item
                    paths - specify the path for navigation - Table will set "id" value for each row in action button                    
                */}                
                <div className="delete-option">
                    <div >
                        <span className="p-float-label">
                            {this.state.schedulingUnitTasks && this.state.schedulingUnitTasks.length > 0 &&
                                <>
                                    {this.props.match.params.type === 'draft' && this.state.taskDraftPermission &&
                                    <>
                                        <button className="p-link" href="#" style={{ pointerEvents: this.props.disabled ? 'none' : 'auto', marginLeft: '8px' }} onClick={this.confirmDeleteTasks} 
                                            title={this.state.taskDraftPermission.delete?"Delete selected Task(s)": `${this.access_denied_message} to delete Task(s)`} >
                                            <i className={this.state.taskDraftPermission.delete?"fa fa-trash":"fa fa-trash fa-disabled"}  aria-hidden="true" ></i>
                                        </button>
                                    </>
                                    }
                                    {this.props.match.params.type === 'blueprint' && this.state.taskPermission && this.state.taskPermission[this.state.schedulingUnitTasks[0].id] &&
                                    <>
                                        <a href="#" style={{ pointerEvents: this.props.disabled ? 'none' : 'auto', marginLeft: '8px' }} onClick={this.showTaskRestartConfirmation} 
                                            title={`Restart Task(s)`} >
                                            <i className={"fa fa-sync"}  aria-hidden="true" ></i>
                                        </a>
                                        <a href="#" onClick={this.confirmCancelTasks} style={this.iconStyle}
                                            title={this.state.taskPermission[this.state.schedulingUnitTasks[0].id].cancel?"Cancel selected Task(s)": `${this.access_denied_message} to cancel Task(s)`}>
                                            <i className={this.state.taskPermission[this.state.schedulingUnitTasks[0].id].cancel?"fa fa-ban":"fa fa-ban fa-disabled"} aria-hidden="true" ></i></a>
                                        <a className="p-link" style={this.iconStyle} href="#" onClick={this.confirmObsoleteTasks} 
                                           title={this.state.taskPermission[this.state.schedulingUnitTasks[0].id].obsolete?"Mark as Obsolete": `${this.access_denied_message}`}>
                                            <i className={this.state.taskPermission[this.state.schedulingUnitTasks[0].id].obsolete?"far fa-times-circle":"far fa-times-circle fa-disabled"} aria-hidden="true" ></i>
                                        </a>
                                        {/* <a href="#" style={{ pointerEvents: this.props.disabled ? 'none' : 'auto', marginLeft: '8px' }} onClick={this.confirmDeleteTasks} 
                                            title={this.state.taskPermission[this.state.schedulingUnitTasks[0].id].delete?"Delete selected Task(s)": `${this.access_denied_message} to delete Task(s)`} >
                                            <i className={this.state.taskPermission[this.state.schedulingUnitTasks[0].id].delete?"fa fa-trash":"fa fa-trash fa-disabled"}  aria-hidden="true" ></i>
                                        </a> */}
                                    </>
                                    }
                                </>
                            }
                        </span>
                    </div>
                </div>
                {this.state.isLoading ? <AppLoader /> : (this.state.schedulingUnitTasks.length > 0) ?
                    <ViewTable
                        data={this.state.schedulingUnitTasks}
                        defaultcolumns={this.state.tmpDefaulcolumns ? this.state.tmpDefaulcolumns : this.state.defaultcolumns} 
                        optionalcolumns={this.state.tmpOptionalcolumns ? this.state.tmpOptionalcolumns : this.state.optionalcolumns}
                        columnclassname={this.state.columnclassname}
                        columnOrders={this.state.tmpColumnOrders}
                        showFilterOption={this.getFilterOptions} //Callback function to provide inputs for option-list in Select Dropdown filter
                        showaction="true"
                        keyaccessor="id"
                        paths={this.state.paths}
                        unittest={this.state.unittest}
                        tablename={`scheduleunit_${this.props.match.params.type}_task_list`}
                        allowRowSelection={true}
                        onRowSelection={this.onRowSelection}
                        ignoreSorting={this.ignoreSorting}
                        lsKeySortColumn={this.lsKeySortColumn}
                        toggleBySorting={(sortData) => this.toggleBySorting(sortData)}
                        defaultSortColumn={this.defaultSortColumn}
                        pageUpdated={this.pageUpdated}
                        defaultpagesize={50|| this.state.schedulingUnitTasks.length}
                        storeFilter={true}
                    />
                    : <div>No Tasks found</div>
                }

                {!this.state.isLoading &&
                    <>
                       {_.keys(this.state.stationGroups).length >0 &&
                        <div className='grouping station_editor'>
                            <fieldset>
                                <h3 className="card-title level-1 je-object__title" style={{display: "inline-block"}}>
                                    <button type="button" title={this.state.stationsCollapsed?"Expand":"Collapse"} 
                                        className="btn btn-secondary btn-sm json-editor-btn-collapse json-editor-btntype-toggle" 
                                        onClick={() => this.setState({stationsCollapsed: !this.state.stationsCollapsed})}>
                                        <i className={this.state.stationsCollapsed?"fas fa-caret-right":"fas fa-caret-down"}></i>
                                    </button>
                                    <label>Station specification</label>
                                </h3>
                                {!this.state.stationsCollapsed &&
                                <>
                                    { _.keys(this.state.stationGroups).map((stnGroup) => {
                                        return <Stations
                                        taskName={stnGroup}
                                        stationGroup={this.state.stationGroups[stnGroup]}
                                        height={'auto'} view={true}
                                        key={stnGroup}
                                    />
                                    })}
                                </>
                                }
                            </fieldset>
                        </div>
                        }
                        
                        {this.state.scheduleunit && this.state.scheduleunit.scheduling_constraints_doc &&
                            <div className="p-fluid">
                                <fieldset className="border-style">
                                    <div className="p-col-12" >
                                        <SchedulingConstraint disable constraintTemplate={this.state.constraintTemplate}
                                            initValue={this.state.scheduleunit.scheduling_constraints_doc} 
                                        />
                                    </div> 
                                </fieldset>
                            </div>
                        }
                        {this.state.paramsSchema &&
                            <div className="p-fluid">
                                <fieldset className="border-style">
                                    <div className="p-col-12" style={{marginLeft: '-0.5em'}}>
                                        {jeditor}
                                    </div> 
                                </fieldset>
                            </div>
                        }
                    </>
                }
                {this.state.showStatusLogs &&
                    <Dialog header={`Status change logs - ${this.state.task ? this.state.task.name : ""}`}
                        visible={this.state.showStatusLogs} maximizable maximized={false} position="left" style={{ width: '50vw' }}
                        onHide={() => { this.setState({ showStatusLogs: false }) }}
                        className="content_dlg">
                        <TaskStatusLogs taskId={this.state.task.id}></TaskStatusLogs>
                    </Dialog>
                }
                {/* Dialog component to show messages and get confirmation */}
                <CustomDialog type="confirmation" visible={this.state.dialogVisible}
                    header={this.state.dialog.header} message={this.state.dialog.detail} actions={this.state.dialog.actions}
                    content={this.state.dialog.content} width={this.state.dialog.width} showIcon={this.state.dialog.showIcon}
                    onClose={this.closeDialog} onCancel={this.closeDialog} onSubmit={this.state.dialog.onSubmit} />

                {/* Show spinner during backend API call */}
                <CustomPageSpinner visible={this.state.showSpinner} />

                {/* To show Data Products To Ingest */}
                {this.state.showTaskRelationDialog && (
                    <Schedulingtaskrelation
                        showTaskRelationDialog={this.state.showTaskRelationDialog}
                        ingestGroup={this.state.ingestGroup}
                        toggle={this.showTaskRelationDialog}
                        submitTaskRelationToIngest={(trDraft) => this.submitTaskRelationToIngest(trDraft)}
                    />
                )}
                <CustomDialog type={this.dialogType} visible={this.state.confirmDialogVisible} width={this.dialogWidth} height={this.dialogHeight}
                    header={this.dialogHeader} message={this.dialogMsg}
                    content={this.dialogContent}
                    showIcon={this.showIcon} onClose={this.cancelDialog} onCancel={this.cancelDialog} onSubmit={this.callBackFunction}>
                </CustomDialog>

                <CustomDialog type={this.dialogType} visible={this.state.confirmTaskReplationDialog} width={this.dialogWidth} height={this.dialogHeight}
                    header={this.dialogHeader} message={this.dialogMsg}
                    content={this.dialogContent}
                    showIcon={this.showIcon} onClose={this.cancelDialog} onCancel={this.cancelDialog} onSubmit={this.submitTaskReplationDialog}>
                </CustomDialog>
                <CustomPageSpinner visible={this.state.showSpinner} />
                {!this.state.isLoading &&
                    <Websocket ref={websocket => this.websocket = websocket} url={process.env.REACT_APP_WEBSOCKET_URL} 
                        onOpen={this.onConnect} onMessage={this.handleData} onClose={this.onDisconnect} />}
            </> : <AccessDenied/>}
            </>

        )
    }
}

export default ViewSchedulingUnit