import React, {Component} from 'react';
import {Redirect} from 'react-router-dom';
import {Dropdown} from 'primereact/dropdown';
import {Button} from 'primereact/button';
import {Toast} from 'primereact/toast';
import {Checkbox} from 'primereact/checkbox';
import {Accordion, AccordionTab} from 'primereact/accordion';
import {DataTable} from 'primereact/datatable';
import {Column} from 'primereact/column';

import TimeInputmask from '../../components/Spreadsheet/TimeInputmask'
import OffsetTimeInputmask from '../../components/Spreadsheet/OffsetTimeInputmask'
import DegreeInputmask from '../../components/Spreadsheet/DegreeInputmask'
import NumericEditor from '../../components/Spreadsheet/NumericEditor';
import BetweenEditor from '../../components/Spreadsheet/BetweenEditor';
import BetweenRenderer from '../../components/Spreadsheet/BetweenRenderer';
import BeamformersRenderer from '../../components/Spreadsheet/BeamformerRenderer';
import MultiSelector from '../../components/Spreadsheet/MultiSelector';
import CustomDateComp from '../../components/Spreadsheet/CustomDateComp';
import StationEditor from '../../components/Spreadsheet/StationEditor';
import Beamformer from '../../components/Spreadsheet/Beamformer';
import {CustomPageSpinner} from '../../components/CustomPageSpinner';

import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import {publish} from '../../App';
import {CustomDialog} from '../../layout/components/CustomDialog';
import SchedulingSet from './schedulingset.create';
import SchedulingConstraints from './Scheduling.Constraints';

import ProjectService from '../../services/project.service';
import ScheduleService from '../../services/schedule.service';
import TaskService from '../../services/task.service';
import UtilService from '../../services/util.service';

import Validator from '../../utils/validator';
import UnitConverter from '../../utils/unit.converter'
import UIConstants from '../../utils/ui.constants';
import ParserUtility from '../../utils/parser.utility';

import moment from 'moment';
import _ from 'lodash';
import $RefParser from "@apidevtools/json-schema-ref-parser";

import {AgGridReact} from 'ag-grid-react';
import {AllCommunityModules} from '@ag-grid-community/all-modules';
import 'ag-grid-community/dist/styles/ag-grid.css';
import 'ag-grid-community/dist/styles/ag-theme-alpine.css';
import AccessDenied from '../../layout/components/AccessDenied';
import AuthUtil from '../../utils/auth.util';
import TopProgressBar from '../../layout/components/TopProgressBar';
import ObservationStrategySelector, {
    queryMatchingObservationStrategyTemplates
} from "./components/ObservationStrategySelector";
import ProjectScheduleSetSelector from "./components/ProjectScheduleSetSelector";
import {getConstraintTemplate, getTaskTemplateForTask} from "./create.helper";
import {getDailyColumns} from "./excelview.schedulingset.helper";
/* eslint-disable no-loop-func */

const BG_COLOR = '#f878788f';

/**
 * Component to create / update Scheduling Unit Drafts using Spreadsheet
 */
export class SchedulingSetCreate extends Component {
    SU_DRAFT_EXPAND = 'task_drafts'
    SU_DRAFT_FIELDS = ['id', 'name', 'url', 'rank', 'priority_queue_value', 'description',
        'scheduling_constraints_doc', 'scheduling_constraints_template',
        'observation_strategy_template_id', 'task_drafts.url', 'scheduling_constraints_template_id',
        'scheduling_set', 'task_drafts.task_type', 'task_drafts.id',
        'task_drafts.name', 'task_drafts.description', 'task_drafts.short_description', 'task_drafts.specifications_doc',
        'task_drafts.specifications_template.id', 'task_drafts.specifications_template.type_value', 'task_drafts.specifications_template.schema',
        'scheduling_unit_blueprints',
        'observation_strategy_template',
        'specifications_template']

    constructor(props) {
        super(props);
        this.state = {
            observStrategyFilters: {
                states: ['active'],
                purpose: []
            },
            userrole: {},
            redirect: null,
            errors: [],
            validFields: {},
            observStrategy: {},
            selectedProject: {},
            copyHeader: false,                    // Copy Table Header to clipboard
            applyEmptyValue: false,
            showAddSet: false,
            projectDisabled: (props.match ? (props.match.params.project ? true : false) : false),
            isLoading: true,
            isAGLoading: false,                       // Flag for loading spinner
            dialog: {header: '', detail: ''},      // Dialog properties
            clipboard: [],
            totalCount: 0,
            validEditor: false,
            noOfSU: 10,
            defaultCellValues: {},
            showDefault: false,
            confirmDialogVisible: false,
            isDirty: false,
            observationIdSet: [],
            multiSelectorOptions: {},
            isObsoletStrategy: false,
            schedulingUnit: {
                name: '',
                description: '',
                project: (props.match ? props.match.params.project : null) || null,
            },
            columnMap: [],
            columnDefs: [],
            columnTypes: {
                numberValueColumn: {
                    editable: true,
                    valueParser: function numberParser(params) {
                        return Number(params.newValue);
                    },
                }
            },
            defaultColDef: {
                editable: true, flex: 1, sortable: true, minWidth: 100, resizable: true,
            },
            rowSelection: 'multiple',
            context: {componentParent: this},
            modules: AllCommunityModules,
            frameworkComponents: {
                numericEditor: NumericEditor,
                timeInputMask: TimeInputmask,
                offsetTimeInputmask: OffsetTimeInputmask,
                degreeInputMask: DegreeInputmask,
                betweenRenderer: BetweenRenderer,
                betweenEditor: BetweenEditor,
                multiselector: MultiSelector,
                agDateInput: CustomDateComp,
                station: StationEditor,
                beamformer: Beamformer,
                beamformersRenderer: BeamformersRenderer,
            },
            components: {
                rowIdRenderer: function (params) {
                    return 1 + params.rowIndex;
                },
                validCount: 0,
                inValidCount: 0,
            },
            noOfSUOptions: [
                {label: '10', value: '10'},
                {label: '50', value: '50'},
                {label: '100', value: '100'},
                {label: '250', value: '250'},
                {label: '500', value: '500'}
            ],
            customSelectedStations: [],
            selectedStations: [],
            defaultStationGroups: {},
            selectedSchedulingSetId: null,
            rowData: [],
            newSet: null,
            priorityQueuelist: null,
            validSpecification: false,
            specificationValidationMessage: '',
            validConstraints: false,
            constraintsValidationMessage: '',
            custId: 1
        };
        this.constraintsValidationMessage = {};
        this.specificationValidationMessage = {};
        this.durationList = [];
        this.stationGroupsName = [];
        this.columnDefinition = {};
        this.taskFilters = [];
        this.isMacOS = false;
        this.gridApi = '';
        this.gridColumnApi = '';
        this.topGridApi = '';
        this.topGridColumnApi = '';
        this.rowData = [];
        this.tmpRowData = [];
        this.daily = [];
        this.isNewSet = false;
        this.constraintSchema = null;
        this.resolvedConstraintSchema = [];
        this.currentTaskDrafts = [];
        this.showIcon = true;
        this.fieldProperty = {};
        this.myRoles = {};

        this.applyToAllRow = false;
        this.callBackFunction = "";
        this.onClose = this.close;
        this.onCancel = this.close;
        this.applyToEmptyRowOnly = false;
        this.isSpecificationValid = true;
        this.isConstraintValid = true;
        this.dialogWidth = "40vw";
        this.dialogType = "confirmation";
        this.dialogHeight = 'auto';
        this.dialogHeader = "";
        this.dialogMsg = "";
        this.dialogContent = "";
        this.projects = [];                         // All projects to load project dropdown
        this.taskTemplates = [];                    // All task templates to be filtered based on tasks in selected strategy template
        this.taskTemplateSchemas = [];
        this.constraintTemplates = [];
        this.agSUWithDefaultValue = {'id': 0, 'suname': '', 'sudesc': '', isDirty: false};
        this.emptyAGSU = {};
        this.priorityQueuelist = [];
        this.intervelTrigger = null;
        this.warningClipboardData = '';
        this.constraintVariables = [];
        this.strategyVariables = [];

        this.onProjectChange = this.onProjectChange.bind(this);
        this.setSchedulingSetParams = this.setSchedulingSetParams.bind(this);
        this.onStrategyChange = this.onStrategyChange.bind(this);
        this.setNoOfSUint = this.setNoOfSUint.bind(this);
        this.isNotEmpty = this.isNotEmpty.bind(this);
        this.onGridReady = this.onGridReady.bind(this);
        this.onTopGridReady = this.onTopGridReady.bind(this);
        this.saveSchedulingUnit = this.saveSchedulingUnit.bind(this);
        this.validateGridAndSave = this.validateGridAndSave.bind(this);
        this.showDialogContent = this.showDialogContent.bind(this);
        this.saveSU = this.saveSU.bind(this);
        this.reset = this.reset.bind(this);
        this.refreshSchedulingSet = this.refreshSchedulingSet.bind(this);
        this.close = this.close.bind(this);
        this.cancelCreate = this.cancelCreate.bind(this);
        this.checkIsDirty = this.checkIsDirty.bind(this);
        this.cellValueChageEvent = this.cellValueChageEvent.bind(this);
        this.showWarning = this.showWarning.bind(this);
        this.copyHeader = this.copyHeader.bind(this);
        this.copyOnlyHeader = this.copyOnlyHeader.bind(this);
        this.clipboardEvent = this.clipboardEvent.bind(this);
        this.copyFromClipboard = this.copyFromClipboard.bind(this);
        this.pasteTextField = this.pasteTextField.bind(this);
        this.applyToAll = this.applyToAll.bind(this);
        this.applyToSelected = this.applyToSelected.bind(this);
        this.applyToEmptyRows = this.applyToEmptyRows.bind(this);
        this.resetCommonData = this.resetCommonData.bind(this);
        this.reload = this.reload.bind(this);
        this.applyChanges = this.applyChanges.bind(this);
        this.getSchedulingDialogContent = this.getSchedulingDialogContent.bind(this);
        this.setSUSet = this.setSUSet.bind(this);
        this.copyClipText = this.copyClipText.bind(this);
        this.copyWarningContent = this.copyWarningContent.bind(this);
        this.showConstraintError = this.showConstraintError.bind(this);
        this.showSpecificationError = this.showSpecificationError.bind(this);
        this.showConstraintErrorDialog = this.showConstraintErrorDialog.bind(this);
        this.showSpecificationErrorDialog = this.showSpecificationErrorDialog.bind(this);
        this.formRules = {                          // Form validation rules
            project: {required: true, message: "Select project to get Scheduling Sets"},
            scheduling_set_id: {required: true, message: "Select the Scheduling Set"},
        };
        this.setConstraintsEditorOutput = this.setConstraintsEditorOutput.bind(this);
        this.setState = this.setState.bind(this);
        this.prepareScheduleUnitListForGrid = this.prepareScheduleUnitListForGrid.bind(this);
    }

    async onTopGridReady(params) {
         this.setState({
            topGridApi: params.api,
            topGridColumnApi: params.columnApi,
        })
        params.api.hideOverlay();
    }

    async onGridReady(params) {
         this.setState({
            gridApi: params.api,
            gridColumnApi: params.columnApi,
        })
        params.api.hideOverlay();
    }

    /**
     * Check is empty string
     * @param {*} value
     */
    isNotEmpty(value) {
        if (value === null || value === undefined || value.length === 0) {
            return false;
        } else {
            return true;
        }
    }


    /**
     * Trigger when the project drop down get changed and check isDirty
     * @param {*} projectName
     */
    onProjectChange(projectName) {
        if (this.state.isDirty) {
            this.showWarning(() => {
                this.changeProject(projectName);
            });
        } else {
            this.changeProject(projectName);
        }
    }

    /**
     * Function to call on change of project and reload scheduling set dropdown
     * @param {string} projectName
     */
    async changeProject(projectName) {
        const projectSchedluingSets = await ScheduleService.getSchedulingSets(`project=${encodeURIComponent(projectName)}`);
        let schedulingUnit = this.state.schedulingUnit;
        schedulingUnit.project = projectName;
        const selectedProject = _.filter(this.projects, {'name': projectName});
        this.setState({
            confirmDialogVisible: false,
            isDirty: false,
            selectedProject: selectedProject,
            schedulingUnit: schedulingUnit,
            schedulingSets: projectSchedluingSets,
            validForm: this.validateForm('project'),
            rowData: [],
            observStrategy: {},
            copyHeader: false,
            observationIdSet: []
        });
        publish('edit-dirty', false);
        schedulingUnit.scheduling_set_id = null;
    }

    /**
     * Function to set form values to the SU object
     * @param {string} key
     * @param {object} value
     */

    async setSchedulingSetParams(key, value) {
        this.setState({isAGLoading: true, copyHeader: false, confirmDialogVisible: false, isDirty: false});
        publish('edit-dirty', false);
        let schedulingUnit = this.state.schedulingUnit;
        schedulingUnit[key] = value;
        // Fetches only the observation_strategy_template_id of the SUDrafts of selected SUSet
        let schedulingUnitList = await ScheduleService.getSchedulingBySet(value, 'observation_strategy_template_id');
        let observationIdSet = new Set();
        if (schedulingUnitList && schedulingUnitList.length > 0) {
            schedulingUnitList.forEach(schedulingUnit => {
                observationIdSet.add(schedulingUnit.observation_strategy_template_id);
            });
            this.setState({schedulingUnitList: schedulingUnitList});
        }
        observationIdSet = Array.from(observationIdSet);
        this.setState({
            observStrategy: {},
            schedulingUnit: schedulingUnit,
            selectedSchedulingSetId: value,
            copyHeader: false,
            confirmDialogVisible: false,
            isDirty: false,
            rowData: [],
            observationIdSet: observationIdSet
        });
        if (observationIdSet.length === 1) {
            this.onStrategyChange(observationIdSet[0]);
        } else if (this.state.observStrategy && this.state.observStrategy.id) {
            this.onStrategyChange(this.state.observStrategy.id);
        } else {
            this.setState({isAGLoading: false});
        }
    }

    /**
     * Set No. of Scheduling Unit load/show in the excel view table
     * @param {*} value
     */
    async setNoOfSUint(value) {
        this.setState({isDirty: true, isAGLoading: true});
        publish('edit-dirty', true);
        if (value >= 0 && value < 501) {
            await this.setState({noOfSU: value});
        } else {
            await this.setState({noOfSU: 500});
        }

        let noOfSU = this.state.noOfSU;
        this.tmpRowData = [];
        if (this.state.rowData && this.state.rowData.length > 0 && this.state.emptyRow) {
            if (this.state.totalCount <= noOfSU) {
                for (var count = 0; count < noOfSU; count++) {
                    if (this.state.rowData.length > count) {
                        this.tmpRowData.push(_.cloneDeep(this.state.rowData[count]));
                    } else {
                        let tmpRow = _.cloneDeep(this.state.agSUWithDefaultValue);
                        this.setState({custId: this.state.custId + 1})
                        tmpRow['custId'] = this.state.custId;
                        this.tmpRowData.push(tmpRow);

                    }
                }
                this.setState({
                    rowData: this.tmpRowData,
                    noOfSU: noOfSU,
                    isAGLoading: false
                });
            } else {
                this.setState({
                    isAGLoading: false
                })
            }
        } else {
            if (Object.keys(this.state.observStrategy).length > 0) {
                for (let count = 0; count < noOfSU; count++) {
                    let tmpRow = _.cloneDeep(this.state.agSUWithDefaultValue);
                    this.setState({custId: this.state.custId + 1})
                    tmpRow['custId'] = this.state.custId;
                    this.tmpRowData.push(tmpRow);

                }
                this.setState({
                    rowData: this.tmpRowData,
                    noOfSU: noOfSU,
                    isAGLoading: false
                });
            }
        }
    }

    /**
     * Set newly created as current SU Set
     * @param {SU Set Name} suSet
     */
    setSUSet(suSet) {
        this.setState({newSet: suSet});
    }

    /**
     * Update isDirty when cell value updated in AG grid
     * @param {*} params
     */
    cellValueChageEvent(params) {
        let rowData = this.state.rowData;
        let paramRowData = rowData[params.rowIndex];
        if (params.value && !_.isEqual(params.value, params.oldValue)) {
            paramRowData["isDirty"] = true;
            this.setState({isDirty: true, rowData: rowData});
            publish('edit-dirty', true);
        }
        if (_.includes(this.constraintVariables, params.column['colId'])) {
            this.validateSchedulingConstraints(params.rowIndex, paramRowData);
        }
        if (_.includes(this.strategyVariables, params.column['colId'])) {
            this.validateSpecificationsDoc(params.rowIndex, paramRowData);
        }
    }

    /**
     * Validate the updated cell value when the focus leaves
     * @param {int} rowIndex
     * @param {String} field
     * @param {Object} value
     */
    onFocusLost(rowIndex, field, value) {
        let row = {};
        if (!field.startsWith('gdef_')) {
            row = this.state.rowData[rowIndex];
            row[field] = value;
            if (_.includes(this.constraintVariables, field)) {
                this.validateSchedulingConstraints(rowIndex, row);
            }
            if (_.includes(this.strategyVariables, field)) {
                this.validateSpecificationsDoc(rowIndex, row);
            }
        }
    }

    /**
     * If any changes detected warn before cancel the page
     */
    /*
     Duplicate fnction
     checkIsDirty() {
         if( this.state.isDirty ){
             this.showIcon = true;
             this.dialogType = "confirmation";
             this.dialogHeader = "Add Multiple Scheduling Unit(s)";
             this.dialogMsg = "Do you want to leave this page? Your changes may not be saved.";
             this.dialogContent = "";
             this.dialogHeight = '5em';
             this.callBackFunction = this.cancelCreate;
             this.onClose = this.close;
             this.onCancel = this.close;
             this.setState({confirmDialogVisible: true});
         } else {
             this.cancelCreate();
         }
     }*/

    /**
     * Set the new Set created in drop down
     */

    /*async setCurrentSUSet(id) {
        this.refreshSchedulingSet();
        if(id) {
            let currentSU = this.state.schedulingUnit;
            currentSU.scheduling_set_id = id;
            this.setState({schedulingUnit: currentSU});
        }

    }*/

    /** After adding new Scheduling Set, refresh the Scheduling Set list */
    async refreshSchedulingSet() {
        let tmpSU = _.cloneDeep(this.state.schedulingUnit);
        const schedulingSets = await ScheduleService.getSchedulingSets(`project=${encodeURIComponent(this.state.schedulingUnit.project)}`);
        if (this.state.newSet) {
            tmpSU.scheduling_set_id = this.state.newSet.id;
            await this.setState({
                schedulingUnit: tmpSU,
                newSet: null
            });
        }
        this.setState({
            showAddSet: false,
            confirmDialogVisible: false,
            schedulingSets: schedulingSets,
            observStrategy: {}
        });
        this.setSchedulingSetParams('scheduling_set_id', tmpSU.scheduling_set_id);
    }

    close() {
        this.setState({confirmDialogVisible: false});
    }

    validateForm(fieldName) {
        let validForm = false;
        let errors = this.state.errors;
        let validFields = this.state.validFields;
        if (fieldName) {
            delete errors[fieldName];
            delete validFields[fieldName];
            if (this.formRules[fieldName]) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.schedulingUnit[fieldName];
                if (rule.required) {
                    if (!fieldValue) {
                        errors[fieldName] = rule.message ? rule.message : `${fieldName} is required`;
                    } else {
                        validFields[fieldName] = true;
                    }
                }
            }
        } else {
            errors = {};
            validFields = {};
            for (const fieldName in this.formRules) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.schedulingUnit[fieldName];
                if (rule.required) {
                    if (!fieldValue) {
                        errors[fieldName] = rule.message ? rule.message : `${fieldName} is required`;
                    } else {
                        validFields[fieldName] = true;
                    }
                }
            }
        }
        this.setState({errors: errors, validFields: validFields});
        if (Object.keys(validFields).length === Object.keys(this.formRules).length) {
            validForm = true;
        }
        return validForm;
    }

    /**
     * This function is mainly added for Unit Tests. If this function is removed Unit Tests will fail.
     */
    validateEditor() {
        return this.validEditor ? true : false;
    }

    async componentDidMount() {

        let observStrategyFilter = UtilService.localStore({type: 'get', key: 'strategyFilterSUSetEditor'});
        if (Object.keys(observStrategyFilter).length === 0) {
            observStrategyFilter = {states: ['active'], purpose: []}
        }
        const permission =  await AuthUtil.getUserPermissionByModule('scheduleunit_draft'); // calling synchronize function getUserPermissionByModule
        this.setState({userrole: permission});
        const promises = [
            ProjectService.getProjectList(false, 'name,url,auto_pin'),
            TaskService.getTaskTemplates(),
            ScheduleService.getSchedulingConstraintTemplates(),
            UtilService.getPriorityQueueType(),
            TaskService.getTaskFilterDefinition('draft'),
            ScheduleService.getStrategyPurpose(),
            ScheduleService.getStrategyState(),
            ProjectService.getMyRoles(),
            this.getMatchingObservationStrategyTemplates(observStrategyFilter)
        ];
        
        let observStrategies;
        await Promise.all(promises).then(async (responses) => {
            this.projects = responses[0];
            this.taskTemplates = responses[1];
            this.constraintTemplates = responses[2];
            this.priorityQueuelist = responses[3];
            this.taskFilters = responses[4];
            this.templatePurposes = this.getStrategyFilterOptions(responses[5], 'purpose');
            this.templateState = this.getStrategyFilterOptions(responses[6], 'state');
            this.myRoles = responses[7];
            observStrategies = responses[8];
            const systemRoles = this.myRoles.system_roles.map(role => {
                return role.toLowerCase()
            });
            const projectRoles = Object.fromEntries(
                Object.entries(this.myRoles.project_roles).map(([project, roles]) => [project.toLowerCase(), roles])
            );
            // Users with system_roles other than superuser, operator & support can view the projects for which they have shared_support project_role
            if (_.intersection(systemRoles, ['superuser', 'operator', 'support']).length < 1) {
                this.projects = _.filter(this.projects, (project) => {
                    return _.includes(projectRoles[project.name.toLowerCase()], 'shared_support')
                })
            }

            let queueList = [" "];
            if (this.priorityQueuelist) {
                this.priorityQueuelist.forEach(queue => {
                    queueList.push(queue.value);
                });
            }
            if (this.state.schedulingUnit.project) {
                const selectedProject = _.filter(this.projects, {'name': this.state.schedulingUnit.project});
                const projectSchedluingSets = await ScheduleService.getSchedulingSets(`project=${encodeURIComponent(this.state.schedulingUnit.project)}`);
                this.setState({
                    isLoading: false,
                    schedulingSets: projectSchedluingSets,
                    priorityQueuelist: queueList,
                    selectedProject: selectedProject
                });
            } else {
                this.setState({isLoading: false, priorityQueuelist: queueList});
            }
        });
       

        this.setState({
            observStrategies: observStrategies,
            observStrategyFilters: observStrategyFilter
        });
    }

    async getMatchingObservationStrategyTemplates(filters) {
        return await queryMatchingObservationStrategyTemplates(filters)
    }

    /**
     *  Trigger when the Strategy drop down get changed and check isDirty
     * @param {*} strategyId
     */
    onStrategyChange(strategyId) {
        if (this.state.isDirty) {
            this.showWarning(() => {
                this.changeStrategy(strategyId);
            });
        } else {
            this.changeStrategy(strategyId);
        }
    }

    /**
     * Function called when observation strategy template is changed.
     *
     * @param {number} strategyId
     */
    async changeStrategy(strategyId) {
        await this.setState({
            noOfSU: 10,
            isAGLoading: true,
            copyHeader: false,
            rowData: [],
            confirmDialogVisible: false,
            isDirty: false
        });
        publish('edit-dirty', false);
        const observStrategy = _.find(this.state.observStrategies, {'id': strategyId});
        ParserUtility.addStationParameters(observStrategy);
        this.setState({
            observStrategy: observStrategy,
            noOfSU: 10,
            isAGLoading: true,
            copyHeader: false,
            rowData: [],
            agSUWithDefaultValue: {},
            confirmDialogVisible: false,
            isDirty: false,
            isObsoletStrategy: observStrategy.state_value === 'obsolete' ? true : false
        });
        
         this.getTaskSchema(observStrategy).then((schemainfo )=>  {
            if (this.state.schedulingUnit.project && this.state.schedulingUnit.scheduling_set_id) {
                this.prepareScheduleUnitListForGrid(observStrategy,schemainfo.schema,schemainfo.defaultStationGroups);
            }});
    }

    async getTaskSchema(observStrategy) {
        let station_groups = {}
        this.stationGroupsName = [];
        if (observStrategy) {
            const tasks = observStrategy.template.tasks;
            const parameters = observStrategy.template.parameters;
            let paramsOutput = {};
            let schema = {
                type: 'object', additionalProperties: false,
                properties: {}, definitions: {}
            };
            const $strategyRefs = await $RefParser.resolve(observStrategy.template);
            // TODo: This schema reference resolving code has to be moved to common file and needs to rework
            for (const param of parameters) {
                // TODO: make parameter handling more generic, instead of task specific.
                if (!param.refs[0].startsWith("#/tasks/")) {
                    continue;
                }

                let taskPaths = param.refs[0].split("/");
                const taskName = taskPaths[2];
                /**
                 * For Short_Description, the task path length will be 4, so added below condition to get short_description details
                 *  #/tasks/Combined Observation/short_description
                 */
                taskPaths = taskPaths.slice((taskPaths.length === 4 ? 3 : 4), taskPaths.length);
                const task = tasks[taskName];
                if (task) {
                    const taskTemplate = getTaskTemplateForTask(this.taskTemplates, task);
                    if (!taskTemplate) {
                        this.growl.show({
                            severity: 'error',
                            summary: 'Task template missing',
                            detail: 'Unable to find task template defined in the strategy template'
                        });
                    }
                    if (taskTemplate?.type_value === 'observation' && task.specifications_doc.station_configuration?.station_groups) {
                        if (taskPaths.includes("station_groups")) {
                            station_groups[param.name] = task.specifications_doc.station_configuration.station_groups;
                            this.stationGroupsName.push(param.name);
                        }
                    }
                    schema.definitions = {...schema.definitions, ...taskTemplate.ref_resolved_schema.definitions};
                    taskPaths.reverse();
                    const paramProp = await ParserUtility.getParamProperty($strategyRefs, taskPaths, taskTemplate.ref_resolved_schema, this.taskFilters);
                    schema.properties[param.name] = _.cloneDeep(paramProp);
                    if (schema.properties[param.name]) {
                        schema.properties[param.name].title = param.name;
                        try {
                            schema.properties[param.name].default = $strategyRefs.get(param.refs[0]);
                        } catch (err) { /* empty */ }
                        if (schema.properties[param.name].default) {
                            paramsOutput[param.name] = schema.properties[param.name].default;
                        }
                    }
                }
            }
            await this.setState({
                observStrategy: observStrategy,
                paramsSchema: schema,
                paramsOutput: paramsOutput,
                defaultStationGroups: station_groups
            });
            return {  schema:schema,defaultStationGroups:station_groups}  ;
        }
        return {schema:null,defaultStationGroups:null}
    }

    /**
     * Resolve JSON Schema
     */
    async resolveSchema(schema) {
        let properties = schema.properties;
        schema.definitions = schema.definitions ? schema.definitions : {};
        if (properties) {
            for (const propertyKey in properties) {
                let property = properties[propertyKey];
                if (property["$ref"] && !property["$ref"].startsWith("#")) {    // 1st level reference of the object
                    const refUrl = property["$ref"];
                    let newRef = refUrl.substring(refUrl.indexOf("#"));
                    if (refUrl.endsWith("/pointing")) {                         // For type pointing
                        schema.definitions["pointing"] = (await $RefParser.resolve(refUrl)).get(newRef);
                        property["$ref"] = newRef;
                    } else {                   // General object to resolve if any reference in child level
                        property = await this.resolveSchema((await $RefParser.resolve(refUrl)).get(newRef));
                    }
                } else if (property["type"] === "array") {             // reference in array items definition
                    let resolvedItems = await this.resolveSchema(property["items"]);
                    schema.definitions = {...schema.definitions, ...resolvedItems.definitions};
                    delete resolvedItems['definitions'];
                    property["items"] = resolvedItems;
                }
                properties[propertyKey] = property;
            }
        } else if (schema["oneOf"]) {             // Reference in OneOf array
            let resolvedOneOfList = [];
            for (const oneOfProperty of schema["oneOf"]) {
                const resolvedOneOf = await this.resolveSchema(oneOfProperty);
                resolvedOneOfList.push(resolvedOneOf);
            }
            schema["oneOf"] = resolvedOneOfList;
        } else if (schema["$ref"] && !schema["$ref"].startsWith("#")) {   //reference in oneOf list item
            const refUrl = schema["$ref"];
            let newRef = refUrl.substring(refUrl.indexOf("#"));
            if (refUrl.endsWith("/pointing")) {
                schema.definitions["pointing"] = (await $RefParser.resolve(refUrl)).get(newRef);
                schema["$ref"] = newRef;
            } else {
                schema = await this.resolveSchema((await $RefParser.resolve(refUrl)).get(newRef));
            }
        }
        return schema;
    }

    /**
     * Function to prepare row data for ag-grid.
     */
    async prepareScheduleUnitListForGrid(observStrategy,paramsSchema,defaultStationGroups) {
        this.custId = 1;
        let defaultCommonRowData = {};
        let observStrategyContraints = observStrategy.template.scheduling_constraints_doc;
        // Load selected fields of SUDrafts of the selected SU Set and filter the SUDrafts of selected observation strategy;
        let schedulingUnitList = (await ScheduleService.getSchedulingBySet(this.state.selectedSchedulingSetId, this.SU_DRAFT_FIELDS, this.SU_DRAFT_EXPAND))
            .filter(su => su.observation_strategy_template_id === observStrategy.id)
            .map(su => {
                su.taskDrafts = su.task_drafts;
                return su
            });

        this.agSUWithDefaultValue = {'id': 0, 'suname': '', 'sudesc': '', isDirty: false};

        /** Get Column details */
        await this.createGridCellDetails(observStrategy,paramsSchema,defaultStationGroups);
        let observationPropsList = [];
        this.tmpRowData = [];
        let totalSU = this.state.noOfSU;
        let lastRow = {};
        this.agSUWithDefaultValue['scheduler'] = observStrategyContraints?.scheduler ? observStrategyContraints.scheduler : this.resolvedConstraintSchema.properties.scheduler.default;
        this.agSUWithDefaultValue['timeat'] = observStrategyContraints?.time ? this.isNotEmpty(observStrategyContraints.time.at) ? moment.utc(observStrategyContraints.time.at).format(UIConstants.CALENDAR_DATETIME_FORMAT) : '' : '';
        this.agSUWithDefaultValue['timeafter'] = observStrategyContraints?.time ? this.isNotEmpty(observStrategyContraints.time.after) ? moment.utc(observStrategyContraints.time.after).format(UIConstants.CALENDAR_DATETIME_FORMAT) : '' : '';
        this.agSUWithDefaultValue['timebefore'] = observStrategyContraints?.time ? this.isNotEmpty(observStrategyContraints.time.before) ? moment.utc(observStrategyContraints.time.before).format(UIConstants.CALENDAR_DATETIME_FORMAT) : '' : '';
        if (observStrategyContraints?.time && observStrategyContraints?.time?.between) {
            this.agSUWithDefaultValue['between'] = this.getBetweenStringValue(observStrategyContraints.time.between);
        }
        if (observStrategyContraints?.time && observStrategyContraints?.time?.between) {
            this.agSUWithDefaultValue['notbetween'] = this.getBetweenStringValue(observStrategyContraints.time.not_between);
        }
        this.agSUWithDefaultValue['min_target_elevation'] = observStrategyContraints?.sky?.min_elevation && observStrategyContraints.sky.min_elevation.target ? ((observStrategyContraints.sky.min_elevation.target * 180) / Math.PI).toFixed(2) : ((this.resolvedConstraintSchema.properties.sky.properties.min_elevation.properties.target.default * 180) / Math.PI).toFixed(2);
        this.agSUWithDefaultValue['min_calibrator_elevation'] = observStrategyContraints?.sky?.min_elevation && observStrategyContraints.sky.min_elevation.calibrator ? ((observStrategyContraints.sky.min_elevation.calibrator * 180) / Math.PI).toFixed(2) : ((this.resolvedConstraintSchema.properties.sky.properties.min_elevation.properties.calibrator.default * 180) / Math.PI).toFixed(2);
        this.agSUWithDefaultValue['offset_from'] = observStrategyContraints?.sky?.transit_offset.from ? (observStrategyContraints.sky.transit_offset.from < 0 ? '-' : '') + UnitConverter.getSecsToHHmmss(observStrategyContraints.sky.transit_offset.from) : "00:00:00";
        this.agSUWithDefaultValue['offset_to'] = observStrategyContraints?.sky?.transit_offset.to ? (observStrategyContraints.sky.transit_offset.to < 0 ? '-' : '') + UnitConverter.getSecsToHHmmss(observStrategyContraints.sky.transit_offset.to) : "00:00:00";
        if (this.resolvedConstraintSchema.properties.sky.properties.reference_pointing) {
            this.agSUWithDefaultValue['ref_pointing_enabled'] = observStrategyContraints?.sky?.reference_pointing?.enabled || this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.enabled.default;
            this.agSUWithDefaultValue['ref_pointing_angle1'] = observStrategyContraints?.sky?.reference_pointing?.pointing?.angle1 ? UnitConverter.getAngleInput(observStrategyContraints.sky.reference_pointing.pointing.angle1, false) : UnitConverter.getAngleInput(this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.pointing.default.angle1, false);
            this.agSUWithDefaultValue['ref_pointing_angle2'] = observStrategyContraints?.sky?.reference_pointing?.pointing?.angle2 ? UnitConverter.getAngleInput(observStrategyContraints.sky.reference_pointing.pointing.angle2, true) : UnitConverter.getAngleInput(this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.pointing.default.angle2, true);
            this.agSUWithDefaultValue['ref_pointing_direction_type'] = observStrategyContraints?.sky?.reference_pointing?.pointing?.direction_type || this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.pointing.default.direction_type;
            this.agSUWithDefaultValue['ref_pointing_target'] = observStrategyContraints?.sky?.reference_pointing?.pointing?.target || this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.pointing.default.target || this.resolvedConstraintSchema.definitions.pointing.properties.target.default;
        }
        this.agSUWithDefaultValue['md_sun'] = observStrategyContraints?.sky?.min_distance && !isNaN(observStrategyContraints.sky.min_distance.sun) ? ((observStrategyContraints.sky.min_distance.sun * 180) / Math.PI).toFixed(2) : ((this.resolvedConstraintSchema.properties.sky.properties.min_distance.properties.sun.default * 180) / Math.PI).toFixed(2);
        this.agSUWithDefaultValue['md_moon'] = observStrategyContraints?.sky?.min_distance && !isNaN(observStrategyContraints.sky.min_distance.moon) ? ((observStrategyContraints.sky.min_distance.moon * 180) / Math.PI).toFixed(2) : ((this.resolvedConstraintSchema.properties.sky.properties.min_distance.properties.moon.default * 180) / Math.PI).toFixed(2);
        this.agSUWithDefaultValue['md_jupiter'] = observStrategyContraints?.sky?.min_distance && !isNaN(observStrategyContraints.sky.min_distance.jupiter) ? ((observStrategyContraints.sky.min_distance.jupiter * 180) / Math.PI).toFixed(2) : ((this.resolvedConstraintSchema.properties.sky.properties.min_distance.properties.jupiter.default * 180) / Math.PI).toFixed(2);
        this.agSUWithDefaultValue['daily'] = getDailyColumns(observStrategyContraints?.daily ? observStrategyContraints.daily : this.resolvedConstraintSchema.properties.daily);

        if (schedulingUnitList && schedulingUnitList.length > 0) {
            this.isNewSet = false;
            for (const scheduleunit of schedulingUnitList) {
                let observationProps = {
                    id: scheduleunit.id,
                    suname: scheduleunit.name,
                    sudesc: scheduleunit.description,
                    rank: scheduleunit.rank,
                    priority_queue: scheduleunit.priority_queue_value,
                    //set default TRUE and it will reset this value while validating the row and will skip the invalid rows when save the row data
                    isValid: true, isDirty: false
                };

                if (scheduleunit.observation_strategy_template_id) {
                    let parameters = await this.getObservationValueFromTask(scheduleunit);
                    let parametersName = Object.keys(parameters);
                    for (const parameter of parametersName) {
                        let valueItem = parameters[parameter];
                        let excelColumns = this.state.columnMap[parameter];
                        if (excelColumns) {
                            let excelColumnsKeys = Object.keys(excelColumns);
                            for (const eColKey of excelColumnsKeys) {
                                if (eColKey === 'angle1') {
                                    observationProps[excelColumns[eColKey]] = UnitConverter.getAngleInput(valueItem[eColKey], false);
                                } else if (eColKey === 'angle2') {
                                    observationProps[excelColumns[eColKey]] = UnitConverter.getAngleInput(valueItem[eColKey], true);
                                } else if (_.includes(this.durationList, eColKey)) {
                                    observationProps[excelColumns[eColKey]] = UnitConverter.getSecsToHHmmss(valueItem, true);
                                } else if (_.includes(this.stationGroupsName, eColKey)) {
                                    observationProps[excelColumns[eColKey]] = await this.updateStationGroups(scheduleunit, eColKey);
                                } else {
                                    let keys = Object.keys(valueItem);
                                    if (_.includes(keys, eColKey)) {
                                        observationProps[excelColumns[eColKey]] = valueItem[eColKey];
                                    } else {
                                        observationProps[excelColumns[eColKey]] = valueItem;
                                    }
                                }
                            }
                        }
                    }
                } else {
                    let parameters = observStrategy.template.parameters;
                    for (const parameter of parameters) {
                        let refUrl = parameter['refs'];
                        let valueItem = (await $RefParser.resolve(observStrategy.template)).get(refUrl[0]);
                        let excelColumns = this.state.columnMap[parameter.name];
                        if (excelColumns) {
                            let excelColumnsKeys = Object.keys(excelColumns);
                            for (const eColKey of excelColumnsKeys) {
                                if (eColKey === 'angle1') {
                                    observationProps[excelColumns[eColKey]] = UnitConverter.getAngleInput(valueItem[eColKey], false);
                                } else if (eColKey === 'angle2') {
                                    observationProps[excelColumns[eColKey]] = UnitConverter.getAngleInput(valueItem[eColKey], true);
                                } else if (_.includes(this.durationList, eColKey)) {
                                    observationProps[excelColumns[eColKey]] = UnitConverter.getSecsToHHmmss(valueItem[eColKey], true);
                                } else if (_.includes(this.stationGroupsName, eColKey)) {
                                    observationProps[excelColumns[eColKey]] = await this.updateStationGroups(scheduleunit, eColKey);
                                } else {
                                    observationProps[excelColumns[eColKey]] = valueItem[eColKey];
                                }
                            }
                        }
                    }
                }
                // Get Station details
                //  observationProps['stations'] = await this.getStationGrops(scheduleunit);
                let constraint = scheduleunit.id ? scheduleunit.scheduling_constraints_doc : null;
                if (constraint) {
                    if (constraint.scheduler) {
                        observationProps['scheduler'] = constraint.scheduler;
                    }

                    observationProps['timeat'] = this.isNotEmpty(constraint.time?.at) ? moment.utc(constraint.time.at).format(UIConstants.CALENDAR_DATETIME_FORMAT) : '';
                    observationProps['timeafter'] = this.isNotEmpty(constraint.time?.after) ? moment.utc(constraint.time.after).format(UIConstants.CALENDAR_DATETIME_FORMAT) : '';
                    observationProps['timebefore'] = this.isNotEmpty(constraint.time?.before) ? moment.utc(constraint.time.before).format(UIConstants.CALENDAR_DATETIME_FORMAT) : '';
                    if (constraint.time?.between) {
                        observationProps['between'] = this.getBetweenStringValue(constraint.time.between);
                    }
                    if (constraint.time?.not_between) {
                        observationProps['notbetween'] = this.getBetweenStringValue(constraint.time.not_between);
                    }

                    observationProps['daily'] = getDailyColumns(constraint.daily);
                    UnitConverter.radiansToDegree(constraint.sky);
                    if (constraint.sky.min_elevation) {
                        observationProps['min_target_elevation'] = constraint.sky.min_elevation.target;
                        observationProps['min_calibrator_elevation'] = constraint.sky.min_elevation.calibrator;
                    }
                    if (constraint.sky.transit_offset) {
                        observationProps['offset_from'] = constraint.sky.transit_offset.from ? (constraint.sky.transit_offset.from < 0 ? '-' : '') + UnitConverter.getSecsToHHmmss(constraint.sky.transit_offset.from) : "00:00:00";
                        observationProps['offset_to'] = constraint.sky.transit_offset.to ? (constraint.sky.transit_offset.to < 0 ? '-' : '') + UnitConverter.getSecsToHHmmss(constraint.sky.transit_offset.to) : "00:00:00";
                        observationProps['offset_from_max'] = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.from.maximum;
                        observationProps['offset_from_min'] = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.from.minimum;
                        observationProps['offset_to_max'] = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.to.maximum;
                        observationProps['offset_to_min'] = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.to.minimum;
                    }
                    if (constraint.sky.min_distance) {
                        observationProps['md_sun'] = constraint.sky.min_distance.sun;//constraint.sky.min_distance.sun:0;
                        observationProps['md_moon'] = constraint.sky.min_distance.moon; //constraint.sky.min_distance.moon:0;
                        observationProps['md_jupiter'] = constraint.sky.min_distance.jupiter;//constraint.sky.min_distance.jupiter:0;
                    }
                    if (constraint.sky.reference_pointing) {
                        observationProps['ref_pointing_enabled'] = constraint.sky.reference_pointing.enabled;
                        observationProps['ref_pointing_angle1'] = UnitConverter.getAngleInput(constraint.sky.reference_pointing.pointing.angle1, false);
                        observationProps['ref_pointing_angle2'] = UnitConverter.getAngleInput(constraint.sky.reference_pointing.pointing.angle2, true);
                        observationProps['ref_pointing_direction_type'] = constraint.sky.reference_pointing.pointing.direction_type;
                        observationProps['ref_pointing_target'] = constraint.sky.reference_pointing.pointing.target;
                    } else {
                        observationProps['ref_pointing_enabled'] = this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.enabled.default;
                        observationProps['ref_pointing_angle1'] = UnitConverter.getAngleInput(this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.pointing.default.angle1, false);
                        observationProps['ref_pointing_angle2'] = UnitConverter.getAngleInput(this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.pointing.default.angle2, true);
                        observationProps['ref_pointing_direction_type'] = this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.pointing.default.direction_type;
                        observationProps['ref_pointing_target'] = this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.pointing.default.target;
                    }
                }
                observationProps['custId'] = this.custId++;
                observationPropsList.push(observationProps);
                //Set values for global row if all rows has same value
                if (_.isEmpty(lastRow)) {
                    lastRow = observationProps;
                    defaultCommonRowData = _.cloneDeep(observationProps);
                } else /* if (!_.isEqual(
                        _.omit(lastRow, ['id']),
                        _.omit(observationProps, ['id'])
                    )) */ {

                    const keys = Object.keys(lastRow);
                    for (const key of keys) {
                        if (key === 'daily') {
                            // console.log("key =>",key,lastRow[key], observationProps[key])
                        }
                        if (!_.isEqual(lastRow[key], observationProps[key])) {
                            defaultCommonRowData[key] = '';
                        }
                    }
                }
            }
        } else {
            this.isNewSet = true;
        }
        this.tmpRowData = observationPropsList;
        // find No. of rows filled in array
        let totalCount = this.tmpRowData.length;
        // Prepare No. Of SU for rows for UI
        if (this.tmpRowData && this.tmpRowData.length > 0) {
            const paramsOutputKey = Object.keys(this.tmpRowData[0]);
            let availableCount = this.tmpRowData.length;
            if (this.isNewSet) {
                availableCount = 0;
                this.tmpRowData = [];
            }
            if (availableCount >= totalSU) {
                totalSU = availableCount + 1;
            }
            for (var i = availableCount; i < totalSU; i++) {
                let emptyRow = {};
                paramsOutputKey.forEach(key => {
                    if (key === 'id') {
                        emptyRow[key] = 0;
                    } else {
                        emptyRow[key] = '';
                    }
                });
                let tmpRow = _.cloneDeep( this.state.agSUWithDefaultValue  || this.agSUWithDefaultValue); /// why is this not   this.agSUWithDefaultValue  // was this.state.agSUWithDefaultValue
                tmpRow['custId'] = this.custId++;
                this.tmpRowData.push(tmpRow);
            }
        } else {
            let availableCount = this.tmpRowData.length;
            for (let i = availableCount; i < totalSU; i++) {
                let tmpRow = _.cloneDeep(this.state.agSUWithDefaultValue || this.agSUWithDefaultValue); // was  this.state.agSUWithDefaultValue 
                tmpRow['custId'] = this.custId++;
                this.tmpRowData.push(tmpRow);
            }
        }
        if (this.isNewSet) {
            defaultCommonRowData = this.tmpRowData[this.tmpRowData.length - 1];
        }
        this.setState({
            schedulingUnitList: schedulingUnitList,
            rowData: this.tmpRowData,
            totalCount: totalCount,
            noOfSU: this.tmpRowData.length,
            emptyRow: this.tmpRowData[this.tmpRowData.length - 1],
            isAGLoading: false,
            commonRowData: [defaultCommonRowData],
            defaultCommonRowData: defaultCommonRowData,
            isFetchingData: true
            // hasSameValue: hasSameValue
        });
        if (this.state.gridApi) {
            this.state.gridApi.setRowData(this.state.rowData);
        }
        // Lazy fetching of specifications_doc so that grid loads faster and data required while updating is loaded later
        ScheduleService.getSchedulingBySet(this.state.selectedSchedulingSetId, ["id", "specifications_doc"], null)
            .then(specList => {
                let rowData = this.state.schedulingUnitList;
                rowData = _.map(rowData, data => {
                    let specData = _.find(specList, {id: data.id});
                    data.specifications_doc = specData ? specData.specifications_doc : {};
                    return data;
                });
                this.setState({schedulingUnitList: rowData, isFetchingData: false});
            });
    }


    /**
     * Get Station details from Scheduling Unit
     * @param {*} schedulingUnit
     */
    async updateStationGroups(schedulingUnit, taskName) {
        let stationValue = '';
        if (schedulingUnit && schedulingUnit.id > 0) {
            if (schedulingUnit && schedulingUnit.observation_strategy_template_id) {
                let paramterRefs = this.state.observStrategy.template.parameters.find(parameter => {
                    return parameter.name === taskName
                });
                let taskPaths = paramterRefs.refs[0].split("/");
                stationValue = this.getStationGroupForCell(taskPaths[2]);
            }
        }
        return stationValue;
    }

    getStationGroupForCell(taskName) {
        let stationValue = '';
        let task = this.currentTaskDrafts.find(task => {
            return task.name === taskName
        });
        let stationGroups = task ? task.specifications_doc.station_configuration?.station_groups : [];
        if (stationGroups) {
            stationGroups.map(stationGroup => {
                stationValue += stationGroup.stations + ':' + stationGroup.max_nr_missing + "|";
                return stationGroup;
            });
            return stationValue;
        }
        return [];
    }


    /**
     * Get Observation details from Scheduling->Task
     * @param {Object} scheduleunit - Scheduling Unit
     * @returns
     */
    async getObservationValueFromTask(scheduleunit) {
        this.currentTaskDrafts = [];
        if (scheduleunit.id) {
            this.currentTaskDrafts = _.clone(scheduleunit.taskDrafts);
            scheduleunit.task_drafts = _.map(scheduleunit.taskDrafts, "url");
        }
        const observStrategy = _.cloneDeep(_.find(this.state.observStrategies, {'id': scheduleunit.observation_strategy_template_id}));
        ParserUtility.addStationParameters(observStrategy);
        const tasks = observStrategy.template.tasks;
        let paramsOutput = [];
        for (const taskName in tasks) {
            const task = tasks[taskName];
            const taskDraft = this.currentTaskDrafts.find(taskD => taskD.name === taskName);
            if (taskDraft) {
                const taskKeys = Object.keys(task);
                for (const taskKey of taskKeys) {
                    if (taskKey !== 'specifications_template') {
                        task[taskKey] = taskDraft[taskKey];
                    }
                }
            }
            //Resolve task from the strategy template
            const $taskRefs = await $RefParser.resolve(task);

            // Identify the task specification template of every task in the strategy template
            const taskTemplate = _.find(this.taskTemplates, {
                'name': task['specifications_template']['name'],
                'version': task.specifications_template.version
            });
            if (!taskTemplate) {
                this.growl.show({
                    severity: 'error',
                    summary: 'Task template missing',
                    detail: 'Unable to find task template defined in the strategy template'
                });
            }
            const taskParams = _.filter(observStrategy.template.parameters, param => param.refs[0].indexOf(`/tasks/${taskName}`) > 0);
            for (const param of taskParams) {
                // Resolve the identified template
                //const $templateRefs = await $RefParser.resolve(taskTemplate);
                let property = {};
                let tempProperty = null;
                const taskPaths = param.refs[0].split("/");
                tempProperty = _.cloneDeep(taskTemplate.ref_resolved_schema.properties[taskPaths.length === 4 ? taskPaths[3] : taskPaths[4]]);
                if (tempProperty && tempProperty.type === 'array') {
                    tempProperty = tempProperty.items.properties[taskPaths[6]];
                }
                property = tempProperty;
                if (property) {
                    property.title = param.name;
                } else {
                    property = {};
                    property.title = param.name;
                }

                try {
                    if (_.endsWith(param.refs[0], 'short_description')) {
                        property.default = taskDraft['short_description'] ? taskDraft['short_description'] : '';
                    } else {
                        property.default = $taskRefs.get(param.refs[0].replace(`#/tasks/${taskName}`, '#'));
                    }
                    paramsOutput[param.name] = property.default;
                } catch (err) { /* empty */ }
            }
        }
        return paramsOutput;
    }

    /**
     * Define AG Grid column properties
     */
    getAGGridAngelColumnsDefinition(schema) {
        let cellProps = [];
        cellProps['angle1'] = {/*isgroup: true,*/
            type: 'numberValueColumn',
            cellRenderer: 'timeInputMask',
            cellEditor: 'timeInputMask',
            valueSetter: 'valueSetter',
            cellStyle: function (params) {
                if (params.value && !Validator.validateTimeAndAngle(params.value)) {
                    return {backgroundColor: BG_COLOR};
                } else {
                    return {backgroundColor: ''};
                }
            },
        };
        cellProps['angle2'] = {/*isgroup: true,*/
            type: 'numberValueColumn',
            cellRenderer: 'degreeInputMask',
            cellEditor: 'degreeInputMask',
            valueSetter: 'valueSetter',
            cellStyle: function (params) {
                if (params.value && !Validator.validateAngle(params.value)) {
                    return {backgroundColor: BG_COLOR};
                } else {
                    return {backgroundColor: ''};
                }
            },
        };
        cellProps['angle3'] = {/*isgroup: true,*/ cellEditor: 'numericEditor', cellStyle: function (params) {
                if (isNaN(params.value)) {
                    return {backgroundColor: BG_COLOR};
                } else {
                    return {backgroundColor: ''};
                }
            }
        };
        cellProps['direction_type'] = {/*isgroup: true,*/
            cellEditor: 'agSelectCellEditor',
            default: schema.definitions.pointing.properties.direction_type.default,
            cellEditorParams: {
                values: schema.definitions.pointing.properties.direction_type.enum,
            },
        };
        cellProps['filter'] = {/*isgroup: false,*/
            cellEditor: 'agSelectCellEditor',
            default: schema.definitions.filter.default,
            cellEditorParams: {
                values: schema.definitions.filter.enum,
            },
        };
        cellProps['antenna_set'] = {/*isgroup: false,*/
            cellEditor: 'agSelectCellEditor',
            default: schema.definitions.antenna_set.default,
            cellEditorParams: {
                values: schema.definitions.antenna_set.enum,
            },
        };
        this.durationList = Object.keys(schema.properties).filter(durationKey => _.endsWith(schema.properties[durationKey]['$ref'], 'timedelta'));
        if (this.durationList) {
            for (const durationKey of this.durationList) {
                this.getColumnDefinition(durationKey + '~' + durationKey, {
                    type: 'string', format: 'hh:mm:ss',
                    schema: {
                        minimum: schema.properties[durationKey]['minimum'] || 0,
                        maximum: schema.properties[durationKey]['maximum'] || 86400
                    }
                });
                cellProps[durationKey + '~' + durationKey] = {cellEditor: 'offsetTimeInputmask'};
            }
        }

        this.getColumnDefinition('Beamformers~Beamformers', {type: 'object', format: 'json'});
        cellProps['beamformers'] = {cellRenderer: 'beamformersRenderer', cellEditor: 'beamformer'};
        for (const paramKey of _.keys(schema.properties)) {
            const param = schema.properties[paramKey];
            if (param.type) {
                const paramType = param.type.toLowerCase();
                if (paramType === 'integer') {
                    cellProps[paramKey.toLowerCase()] = {/*isgroup: false,*/
                        cellEditor: 'numericEditor',
                        cellStyle: function (params) {
                            if (isNaN(params.value)) {
                                return {backgroundColor: BG_COLOR};
                            } else {
                                return {backgroundColor: ''};
                            }
                        }
                    };
                } else if (paramType === 'boolean') {
                    cellProps[paramKey.toLowerCase()] = {/*isgroup: false,*/
                        cellEditor: 'agSelectCellEditor',
                        default: param.default ? param.default : false,
                        cellEditorParams: {
                            values: [true, false],
                        },
                    };
                } else if (paramType === 'object' && param.properties) {
                    for (const subProp of Object.keys(param.properties)) {
                        if (param.properties[subProp].type && param.properties[subProp].type === 'boolean') {
                            cellProps[subProp.toLowerCase()] = {
                                cellEditor: 'agSelectCellEditor',
                                default: param.default && param.default[subProp] ? param.default[subProp] : false,
                                cellEditorParams: {values: [true, false]},
                                'headerName': param.properties[subProp].title,
                                'headerTooltip': param.properties[subProp].title,
                            }
                        } else if (param.properties[subProp].type && param.properties[subProp].type === 'integer') {
                            cellProps[subProp.toLowerCase()] = {
                                cellEditor: 'numericEditor',
                                cellStyle: function (params) {
                                    if (isNaN(params.value)) {
                                        return {backgroundColor: BG_COLOR};
                                    } else {
                                        return {backgroundColor: ''};
                                    }
                                },
                                'headerName': param.properties[subProp].title,
                                'headerTooltip': param.properties[subProp].title,
                                type: 'integer', //TODO: this is not allowed
                                maximum: param.properties[subProp].maximum ? param.properties[subProp].maximum : undefined,
                                minimum: param.properties[subProp].minimum ? param.properties[subProp].minimum : undefined,
                                default: param.default && param.default[subProp] ? param.default[subProp] : undefined
                            };
                        } else if (param.properties[subProp].type && param.properties[subProp].type === 'number') {
                            cellProps[subProp.toLowerCase()] = {
                                cellStyle: function (params) {
                                    if (isNaN(params.value)) {
                                        return {backgroundColor: BG_COLOR};
                                    } else {
                                        return {backgroundColor: ''};
                                    }
                                },
                                'headerName': param.properties[subProp].title,
                                'headerTooltip': param.properties[subProp].title,
                                type: 'number', //TODO: not allowed
                                maximum: param.properties[subProp].maximum ? param.properties[subProp].maximum : undefined,
                                minimum: param.properties[subProp].minimum ? param.properties[subProp].minimum : undefined,
                                default: param.default && param.default[subProp] ? param.default[subProp] : undefined
                            };
                        } else {
                            cellProps[subProp.toLowerCase()] = {default: param.default && param.default[subProp] ? param.default[subProp] : undefined}
                        }
                    }
                }
            }
        }
        return cellProps;
    }

    /**
     *
     * @param {*} predefineCellProps
     * @param {*} childCellProps
     * @param {*} cellName
     * @returns
     */
    getAGGridAngelColumnsProperty(predefineCellProps, childCellProps, cellName) {
        let cellProperty = predefineCellProps[cellName];
        if (cellProperty) {
            let cellKeys = Object.keys(cellProperty);
            for (const cellKey of cellKeys) {
                childCellProps[cellKey] = predefineCellProps[cellName][cellKey];
            }
        }
        return childCellProps;
    }

    async createGridCellDetails(observStrategy,paramsSchema,defaultStationGroups) {
        let columnMap = [];
        let colProperty = {};
        this.colKeyOrder = [];
        let columnDefs = [
            { // Row Index
                headerName: '#',
                editable: false,
                maxWidth: 60,
                cellRenderer: 'rowIdRenderer',
                pinned: 'left',
                lockPosition: true,
                suppressSizeToFit: true,
                headerTooltip: 'Testing'
            },
            {
                headerName: 'Scheduling Unit', headerTooltip: 'Scheduling Unit', children: [
                    {headerName: 'Name', field: 'suname', headerTooltip: "Name"},
                    {
                        headerName: 'Description',
                        field: 'sudesc',
                        headerTooltip: "Description",
                        cellStyle: function (params) {
                            if (params.data && params.data.suname && (params.data.suname !== '' && (!params.value || params.value === ''))) {
                                return {backgroundColor: BG_COLOR};
                            } else {
                                return {backgroundColor: ''};
                            }
                        },
                    },
                    {
                        headerName: 'Rank',
                        field: 'rank',
                        cellEditor: 'numericEditor',
                        headerTooltip: "Rank",
                        cellStyle: function (params) {
                            let value = params.data.rank ? params.data.rank : params.data.gdef_rank ? params.data.gdef_rank : '';
                            if (value && value !== '') {
                                const splitValue = _.split((value + ''), ".");
                                if (value < 0 || value > 1 || (splitValue.length > 1 && splitValue[1].length > 4)) {
                                    return {backgroundColor: BG_COLOR};
                                } else {
                                    return {backgroundColor: ''};
                                }
                            } else {
                                return {backgroundColor: ''};
                            }
                        },
                        valueFormatter: function (params) {
                            if (params.value !== '') {
                                const value = parseFloat(params.value);
                                return value >= 0 ? value.toFixed(4) : params.value;
                            } else {
                                return "0.0000";
                            }
                        }
                    },
                    {
                        headerName: 'Priority Queue',
                        headerTooltip: "Priority Queue",
                        field: 'priority_queue',
                        cellEditor: 'agSelectCellEditor',
                        cellEditorParams: {values: this.state.priorityQueuelist}
                    },]
            }
        ];
        //this.columnDefinition = {'priority_queue': {'required': false, 'type': 'string', 'enum': ['A', 'B'], multiselect: false}}
        colProperty = {
            'ID': 'id',
            'Name': 'suname',
            'Description': 'sudesc',
            'Rank': 'rank',
            'Priority Queue': 'priority_queue'
        };
        columnMap['Scheduling Unit'] = colProperty;
        this.colKeyOrder.push("suname");
        this.colKeyOrder.push("sudesc");
        this.colKeyOrder.push("rank");
        this.colKeyOrder.push("priority_queue");
        // Create Constraint Column for AG Grid
        columnDefs = await this.getConstraintColumns(columnDefs,observStrategy);
        let cellProps = {};
        this.strategyVariables = [];
        //Observation Schema
        const schema = paramsSchema;
        if (schema.properties && !_.isEmpty(schema.properties)) {
            let predefineCellProps = this.getAGGridAngelColumnsDefinition(schema);
            let propKeys = Object.keys(schema.properties);
            for (const prop of propKeys) {
                colProperty = {};
                cellProps = {};
                let property = schema.properties[prop];
                if (property && property.$ref) {
                    cellProps['headerName'] = property.title;
                    cellProps['headerTooltip'] = property.title;
                    if (property.title === 'Filter' || property.title === 'Antenna Set') {
                        colProperty = {};
                        cellProps['headerName'] = property.title;
                        cellProps['headerTooltip'] = property.title;
                        this.strategyVariables.push(prop + "~" + property.title);
                        this.agSUWithDefaultValue[prop + "~" + property.title] = property.default;
                        cellProps['field'] = prop + "~" + property.title;
                        cellProps = this.getAGGridAngelColumnsProperty(predefineCellProps, cellProps, property.title === 'Filter' ? _.lowerCase(property.title) : 'antenna_set');
                        colProperty[property.title] = prop + "~" + property.title;
                        columnMap[property.title] = colProperty;
                        columnDefs.push(cellProps);
                    } else if (_.includes(this.stationGroupsName, property.title)) {
                        //Configure the station at end of the row
                    } else if (_.includes(this.durationList, property.title)) {
                        cellProps['headerName'] = property.title;
                        cellProps['headerTooltip'] = property.title;
                        this.strategyVariables.push(prop + "~" + property.title);
                        this.agSUWithDefaultValue[prop + "~" + property.title] = UnitConverter.getSecsToHHmmss(property.default);
                        cellProps['field'] = prop + "~" + property.title;
                        cellProps = this.getAGGridAngelColumnsProperty(predefineCellProps, cellProps, _.lowerCase(property.title));
                        colProperty[property.title] = prop + "~" + property.title;
                        columnMap[property.title] = colProperty;
                        columnDefs.push(cellProps);
                    } else {
                        let defaultKeys = Object.keys(property.default);
                        let children = [];
                        if (defaultKeys.length > 0 && !property.title.endsWith("Beamformers")) {
                            for (const defaultKey of defaultKeys) {
                                this.strategyVariables.push(prop + "~" + defaultKey);
                                if (defaultKey === 'angle1') {
                                    this.agSUWithDefaultValue[prop + "~" + defaultKey] = UnitConverter.getAngleInput(property.default[defaultKey], false);
                                } else if (defaultKey === 'angle2') {
                                    this.agSUWithDefaultValue[prop + "~" + defaultKey] = UnitConverter.getAngleInput(property.default[defaultKey], true);
                                } else {
                                    this.agSUWithDefaultValue[prop + "~" + defaultKey] = property.default[defaultKey];
                                }
                                let childCellProps = {
                                    headerName: _.startCase(defaultKey),
                                    headerTooltip: _.startCase(defaultKey),
                                    field: prop + "~" + defaultKey
                                };
                                childCellProps = this.getAGGridAngelColumnsProperty(predefineCellProps, childCellProps, defaultKey);
                                colProperty[defaultKey] = prop + "~" + defaultKey;
                                children.push(childCellProps);
                            }
                            columnMap[property.title] = colProperty;
                            cellProps['children'] = children;
                            columnDefs.push(cellProps);
                        } else {
                            cellProps['headerName'] = property.title;
                            cellProps['headerTooltip'] = property.title;
                            this.strategyVariables.push(prop + "~" + property.title);
                            this.agSUWithDefaultValue[prop + "~" + property.title] = property.default;
                            cellProps['field'] = prop + "~" + property.title;
                            cellProps = this.getAGGridAngelColumnsProperty(predefineCellProps, cellProps, _.lowerCase(property.title));
                            colProperty[property.title] = prop + "~" + property.title;
                            columnMap[property.title] = colProperty;
                            columnDefs.push(cellProps);
                        }
                    }
                } else {
                    colProperty = {};
                    if (property.type === 'array' && property.items.enum) {
                        await this.getDemixSourceColumnDef(property, prop, prop, colProperty, columnMap, columnDefs);
                    } else if (property.type === 'object' && property.properties) {
                        cellProps['headerName'] = property.title;
                        cellProps['headerTooltip'] = property.title;
                        let children = [];
                        for (const subProp of Object.keys(property.properties)) {
                            if (property.properties[subProp].type === 'array' && property.properties[subProp].items.enum) {
                                await this.getDemixSourceColumnDef(property.properties[subProp], prop, subProp, colProperty, columnMap, children);
                            } else {
                                this.strategyVariables.push(prop + "~" + subProp);
                                this.agSUWithDefaultValue[prop + "~" + subProp] = property.default[subProp];
                                let childCellProps = {
                                    headerName: _.startCase(subProp),
                                    headerTooltip: _.startCase(subProp),
                                    field: prop + "~" + subProp
                                };
                                childCellProps = this.getAGGridAngelColumnsProperty(predefineCellProps, childCellProps, subProp.toLowerCase());
                                colProperty[subProp] = prop + "~" + subProp;
                                children.push(childCellProps);
                            }
                        }
                        columnMap[property.title] = colProperty;
                        cellProps['children'] = children;
                        columnDefs.push(cellProps);
                    } else {
                        cellProps['headerName'] = property.title;
                        cellProps['headerTooltip'] = property.title;
                        if (property.max_length) {
                            cellProps['max_length'] = property.max_length;
                        }
                        this.strategyVariables.push(prop + "~" + property.title);
                        this.agSUWithDefaultValue[prop + "~" + property.title] = property.default !== undefined ? property.default : '';
                        cellProps['field'] = prop + "~" + property.title;
                        cellProps = this.getAGGridAngelColumnsProperty(predefineCellProps, cellProps, property.title.toLowerCase());
                        colProperty[property.title] = prop + "~" + property.title;
                        columnMap[property.title] = colProperty;
                        columnDefs.push(cellProps);
                    }
                }
            }
        }
        const stationKeys = Object.keys(schema.properties).filter(stationKey => _.includes(this.stationGroupsName, stationKey));
        if (stationKeys) {
            for (const stationKey of stationKeys) {
                let stationValue = '';
                let cellProp = {};
                defaultStationGroups[stationKey].map(stationGroup => {
                    let missingStation = (stationGroup.max_nr_missing) ? stationGroup.max_nr_missing : 0;
                    stationValue += stationGroup.stations + ':' + missingStation + "|";
                    return stationGroup;
                });
                let stationField = stationKey;
                this.agSUWithDefaultValue[stationField] = stationValue;
                columnDefs.push({
                    headerName: stationKey,
                    headerTooltip: stationKey,
                    field: stationField,
                    cellRenderer: 'betweenRenderer',
                    cellEditor: 'station',
                    valueSetter: 'newValueSetter'
                });
                this.strategyVariables.push(stationField);
                cellProp[stationField] = stationField;
                columnMap[stationField] = cellProp;
            }
        }


        this.getEmptyRow();
        this.colKeyOrder = [...this.colKeyOrder, ...this.strategyVariables];
        let globalColmunDef = _.cloneDeep(columnDefs);
        globalColmunDef = await this.createGlobalColumnDefs(globalColmunDef, schema);
        this.setState({
            colKeyOrder: this.colKeyOrder,
            globalColmunDef: globalColmunDef,
            columnDefs: columnDefs,
            columnMap: columnMap,
            agSUWithDefaultValue: this.agSUWithDefaultValue
        });
    }

    /**
     * Prepare Demix Source column definition
     */
    getDemixSourceColumnDef(property, prop, subProp, colProperty, columnMap, children) {
        let childCellProps = {};
        const title = property.title ? property.title : subProp;
        const tmpField = prop + "~" + title;
        this.getColumnDefinition(tmpField, {type: 'object', enum: property.items.enum, multiselect: true});
        let multiSelectorOptions = _.cloneDeep(this.state.multiSelectorOptions);
        childCellProps['headerName'] = _.startCase(title);
        childCellProps['headerTooltip'] = _.startCase(title);
        childCellProps['cellEditor'] = 'multiselector';
        childCellProps['field'] = prop + "~" + title;
        childCellProps.valueSetter = function (params) {
        } //This line required to set selected value in appropriate cell
        this.strategyVariables.push(prop + "~" + title);
        this.agSUWithDefaultValue[prop + "~" + title] = property.items.default;
        colProperty[title] = prop + "~" + title;
        columnMap[title] = colProperty;
        let option = []
        property.items.enum.forEach(prop => {
            option.push({'name': prop, 'value': prop});
        });
        multiSelectorOptions[childCellProps['field']] = option;
        children.push(childCellProps);
        this.setState({multiSelectorOptions: multiSelectorOptions});
    }

    /**
     * Create AG Grid column definition for top table
     * @param {*} globalColmunDef
     * @param {*} schema
     * @param {*} constraintSchema
     */
    createGlobalColumnDefs(globalColmunDef, schema) {
        let schedulerValues = [...' ', ...this.resolvedConstraintSchema.properties.scheduler.enum];
        let direction_type_Values = [...' ', ...(schema.definitions.pointing ? schema.definitions.pointing.properties.direction_type.enum : [])];
        globalColmunDef.forEach(colDef => {
            if (colDef.children) {
                colDef.children.forEach(childColDef => {
                    if (childColDef.field) {
                        if (childColDef.field.endsWith('direction_type')) {
                            childColDef.cellEditorParams.values = direction_type_Values;
                        } else if (childColDef.field === 'filter') {
                            childColDef.cellEditorParams.values = [...' ', ...schema.definitions.filter.enum];
                        }
                        childColDef.field = 'gdef_' + childColDef.field;
                        if (childColDef.default) {
                            childColDef.default = '';
                        }
                    }
                });
            } else {
                if (colDef.headerName === '#') {
                    colDef['hide'] = true;
                }
                if (colDef.field) {
                    if (colDef.field.endsWith('scheduler')) {
                        colDef.cellEditorParams.values = schedulerValues;
                    }
                    colDef.field = 'gdef_' + colDef.field;
                    if (colDef.default) {
                        colDef.default = '';
                    }
                }
            }
        });
        return globalColmunDef;
    }

    /**
     *
     */
    getEmptyRow() {
        this.emptyAGSU = {};
        let keys = Object.keys(this.agSUWithDefaultValue);
        for (const key of keys) {
            if (key === 'id') {
                this.emptyAGSU[key] = 0;
            } else {
                this.emptyAGSU[key] = '';
            }
        }
    }

    /**
     * Column definition to validate the cell
     * @param {*} key
     * @param {*} props
     */
    async getColumnDefinition(key, props) {
        this.columnDefinition[key] = {};
        if (props.required) {
            this.columnDefinition[key]['required'] = props.required;
        }
        if (props.type) {
            this.columnDefinition[key]['type'] = props.type;
        }
        if (props.enum) {
            this.columnDefinition[key]['enum'] = props.enum;
        }
        if (props.minimum) {
            this.columnDefinition[key]['minimum'] = props.minimum;
        }
        if (props.maximum) {
            this.columnDefinition[key]['maximum'] = props.maximum;
        }
        if (props.multiselect) {
            this.columnDefinition[key]['multiselect'] = props.multiselect;
        }

        switch (key.toLowerCase()) {
            case 'offset_from':
            case 'offset_to':
                this.columnDefinition[key]['format'] = '-hh:mm:ss';
                break;
            case 'timeat':
            case 'timeafter':
            case 'timebefore':
                this.columnDefinition[key]['format'] = 'date-time';
                break;
            case 'notbetween':
            case 'between':
                this.columnDefinition[key]['format'] = 'date-time-array';
                break;
            default:
                if (_.includes(this.durationList, key)) {
                    this.columnDefinition[key]['format'] = 'hh:mm:ss';
                    this.columnDefinition[key]['schema'] = props.schema;
                } else if (props.format) {
                    this.columnDefinition[key]['format'] = props.format;
                }
                break;
        }
    }

    /**
     * Create Constraint columns for AG Grid
     * @param {*} columnDefs
     * @returns
     */
    async getConstraintColumns(columnDefs,observStrategy) {
        this.constraintVariables = [];
        // currently only one constraint schema available and not propvided UI to choose constraints, so assign directly
        const currentConstraintsTemplate = observStrategy.template.scheduling_constraints_template;
        this.constraintSchema = getConstraintTemplate(this.constraintTemplates, currentConstraintsTemplate.name, currentConstraintsTemplate.version)
        if (!this.constraintSchema) {
            this.growl.show({
                severity: 'error',
                summary: 'Constraint template missing',
                detail: 'Unable to find constraint template defined in the strategy template'
            });
        }
        this.resolvedConstraintSchema = this.constraintSchema?.ref_resolved_schema;
       
        /**     AG Grid Cell Specific Properties
         In Excel View - expected column order is ['scheduler', 'time', 'daily', 'sky'] */
        let dailyProps = Object.keys(this.resolvedConstraintSchema.properties.daily.properties);
        let multiSelectorOptions = _.cloneDeep(this.state.multiSelectorOptions);
        this.columnDefinition['daily'] = {required: false, type: 'object', enum: dailyProps, multiselect: true}
        multiSelectorOptions['daily'] = dailyProps;
        this.daily = [];
        let dailyOption = [];
        dailyProps.forEach(prop => {
            dailyOption.push({'name': prop, 'value': prop});
            this.daily.push(prop);
        });
        multiSelectorOptions['daily'] = dailyOption;
        this.setState({daily: this.daily, multiSelectorOptions: multiSelectorOptions});

        // move this variable to class variable
        //Ag-grid Colums definition
        // Column order to use clipboard copy
        this.constraintVariables.push('scheduler');
        let tmpProps = this.resolvedConstraintSchema.properties.scheduler;
        tmpProps['multiselect'] = false;
        this.getColumnDefinition('scheduler', tmpProps);
        this.getColumnDefinition('timeat', {type: 'string', format: 'date-time'});
        this.getColumnDefinition('timeafter', {type: 'string', format: 'date-time'});
        this.getColumnDefinition('timebefore', {type: 'string', format: 'date-time'});
        this.getColumnDefinition('between', {type: 'string', format: 'date-time-array'});
        this.getColumnDefinition('notbetween', {type: 'string', format: 'date-time-array'});

        this.agSUWithDefaultValue['scheduler'] = this.resolvedConstraintSchema.properties.scheduler.default;
        this.agSUWithDefaultValue['min_target_elevation'] = ((this.resolvedConstraintSchema.properties.sky.properties.min_elevation.properties.target.default * 180) / Math.PI).toFixed(2);
        const elevationMax = this.resolvedConstraintSchema.definitions.elevation.maximum;
        const elevationMin = this.resolvedConstraintSchema.definitions.elevation.minimum;
        this.agSUWithDefaultValue['elevation_max'] = elevationMax;
        this.agSUWithDefaultValue['elevation_min'] = elevationMin;
        this.agSUWithDefaultValue['min_calibrator_elevation'] = ((this.resolvedConstraintSchema.properties.sky.properties.min_elevation.properties.calibrator.default * 180) / Math.PI).toFixed(2);
        this.agSUWithDefaultValue['offset_from'] = (this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.from.default < 0 ? '-' : '') + UnitConverter.getSecsToHHmmss(this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.from.default);
        this.agSUWithDefaultValue['offset_to'] = (this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.to.default < 0 ? '-' : '') + UnitConverter.getSecsToHHmmss(this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.to.default);
        const maxOffsetFrom = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.from.maximum;
        const minOffsetFrom = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.from.minimum;
        this.agSUWithDefaultValue['offset_from_max'] = maxOffsetFrom;
        this.agSUWithDefaultValue['offset_from_min'] = minOffsetFrom;
        const maxOffsetTo = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.to.maximum;
        const minOffsetTo = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.to.minimum;
        this.agSUWithDefaultValue['offset_to_max'] = maxOffsetTo;
        this.agSUWithDefaultValue['offset_to_min'] = minOffsetTo;
        this.agSUWithDefaultValue['md_sun'] = ((this.resolvedConstraintSchema.properties.sky.properties.min_distance.properties.sun.default * 180) / Math.PI).toFixed(2);
        this.agSUWithDefaultValue['md_moon'] = ((this.resolvedConstraintSchema.properties.sky.properties.min_distance.properties.moon.default * 180) / Math.PI).toFixed(2);
        this.agSUWithDefaultValue['md_jupiter'] = ((this.resolvedConstraintSchema.properties.sky.properties.min_distance.properties.jupiter.default) / Math.PI).toFixed(2);
        columnDefs.push({
            headerName: 'Scheduler',
            headerTooltip: 'Scheduler',
            field: 'scheduler',
            cellEditor: 'agSelectCellEditor',
            default: this.resolvedConstraintSchema.properties.scheduler.default,
            cellEditorParams: {values: this.resolvedConstraintSchema.properties.scheduler.enum,},
        });
        columnDefs.push({
            headerName: 'Time', headerTooltip: 'Time',
            children: [
                {
                    headerName: 'At',
                    headerTooltip: 'At',
                    field: 'timeat',
                    editable: true,
                    cellRenderer: 'betweenRenderer',
                    cellEditor: 'agDateInput',
                    valueSetter: 'newValueSetter'
                },
                {
                    headerName: 'After',
                    headerTooltip: 'After',
                    field: 'timeafter',
                    editable: true,
                    cellRenderer: 'betweenRenderer',
                    cellEditor: 'agDateInput',
                    valueSetter: 'newValueSetter'
                },
                {
                    headerName: 'Before',
                    headerTooltip: 'Before',
                    field: 'timebefore',
                    editable: true,
                    cellRenderer: 'betweenRenderer',
                    cellEditor: 'agDateInput',
                    valueSetter: 'newValueSetter'
                },
            ],
        });
        this.constraintVariables.push('timeat');
        this.constraintVariables.push('timeafter');
        this.constraintVariables.push('timebefore');
        this.constraintVariables.push('between');
        this.constraintVariables.push('notbetween');
        this.constraintVariables.push('daily');
        columnDefs.push({
            headerName: 'Between',
            headerTooltip: 'Between',
            field: 'between',
            cellRenderer: 'betweenRenderer',
            cellEditor: 'betweenEditor',
            valueSetter: 'newValueSetter'
        });
        columnDefs.push({
            headerName: 'Not Between',
            headerTooltip: 'Not Between',
            field: 'notbetween',
            cellRenderer: 'betweenRenderer',
            cellEditor: 'betweenEditor',
            valueSetter: 'newValueSetter'
        });
        this.constraintVariables.push('min_target_elevation');
        this.constraintVariables.push('min_calibrator_elevation');
        this.constraintVariables.push('offset_from');
        this.constraintVariables.push('offset_to');
        this.constraintVariables.push('ref_pointing_enabled');
        this.constraintVariables.push('ref_pointing_angle1');
        this.constraintVariables.push('ref_pointing_angle2');
        this.constraintVariables.push('ref_pointing_direction_type');
        this.constraintVariables.push('ref_pointing_target');
        columnDefs.push({
                headerName: 'Daily',
                headerTooltip: 'Daily',
                field: 'daily',
                cellEditor: 'multiselector',
                valueSetter: function (params) {
                }
            },
            {
                headerName: 'Sky', headerTooltip: 'Sky',
                children: [
                    {
                        headerName: 'Min Target Elevation (Degrees)',
                        headerTooltip: 'Min Target Elevation (Degrees)',
                        field: 'min_target_elevation',
                        cellEditor: 'numericEditor',
                        cellStyle: function (params) {
                            if (params.value) {
                                if (params.value === undefined || params.value === null || isNaN(params.value)) {
                                    return {backgroundColor: BG_COLOR};
                                } else if (Number(params.value) < 0 || Number(params.value) > 90) {
                                    return {backgroundColor: BG_COLOR};
                                } else {
                                    return {backgroundColor: ''};
                                }
                            }
                        },
                        valueFormatter: params => {
                            if (params.value !== '') {
                                const value = parseFloat(params.value);
                                return value >= 0 ? value.toFixed(2) : '0.00';
                            } else {
                                return "0.00";
                            }
                        }
                    },
                    {
                        headerName: 'Min Calibrator Elevation (Degrees)',
                        headerTooltip: 'Min Calibrator Elevation (Degrees)',
                        field: 'min_calibrator_elevation',
                        cellEditor: 'numericEditor',
                        cellStyle: function (params) {
                            if (params.value) {
                                if (params.value === undefined || params.value === null || isNaN(params.value)) {
                                    return {backgroundColor: BG_COLOR};
                                } else if (Number(params.value) < 0 || Number(params.value) > 90) {
                                    return {backgroundColor: BG_COLOR};
                                } else {
                                    return {backgroundColor: ''};
                                }
                            }
                        },
                        valueFormatter: params => {
                            if (params.value !== '') {
                                const value = parseFloat(params.value);
                                return value >= 0 ? value.toFixed(2) : '0.00';
                            } else {
                                return "0.00";
                            }
                        }
                    },
                    {
                        headerName: 'Offset Window From',
                        headerTooltip: 'Offset Window From',
                        field: 'offset_from',
                        cellRenderer: 'betweenRenderer',
                        cellEditor: 'offsetTimeInputmask',
                        valueSetter: 'newValueSetter',
                        cellStyle: function (params) {
                            const cellValue = _.split(params.value, ":");
                            if (typeof params.value === 'undefined' ||
                                (cellValue.length !== 3 || isNaN(cellValue[1]) || cellValue[1] > 59 || isNaN(cellValue[2]) || cellValue[2] > 59)) {
                                return {backgroundColor: BG_COLOR};
                            } else if (UnitConverter.getHHmmssToSecs(params.value) < minOffsetFrom ||
                                UnitConverter.getHHmmssToSecs(params.value) > maxOffsetFrom) {
                                return {backgroundColor: BG_COLOR};
                            } else {
                                return {backgroundColor: ''};
                            }
                        }
                    },
                    {
                        headerName: 'Offset Window To',
                        headerTooltip: 'Offset Window To',
                        field: 'offset_to',
                        cellRenderer: 'betweenRenderer',
                        cellEditor: 'offsetTimeInputmask',
                        valueSetter: 'newValueSetter',
                        cellStyle: function (params) {
                            const cellValue = _.split(params.value, ":");
                            if (typeof params.value === 'undefined' ||
                                (cellValue.length !== 3 || isNaN(cellValue[1]) || cellValue[1] > 59 || isNaN(cellValue[2]) || cellValue[2] > 59)) {
                                return {backgroundColor: BG_COLOR};
                            } else if (UnitConverter.getHHmmssToSecs(params.value) < minOffsetTo ||
                                UnitConverter.getHHmmssToSecs(params.value) > maxOffsetTo) {
                                return {backgroundColor: BG_COLOR};
                            } else {
                                return {backgroundColor: ''};
                            }
                        }
                    }
                ],
            });
        // Add reference pointing constraint
        if (this.resolvedConstraintSchema.properties.sky.properties.reference_pointing) {
            columnDefs.push({
                headerName: `${this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.title}`,
                headerTooltip: `${this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.description}`,
                children: [
                    {
                        headerName: 'Enabled',
                        headerTooltip: `${this.resolvedConstraintSchema.properties.sky.properties.reference_pointing.properties.enabled.description}`,
                        field: 'ref_pointing_enabled',
                        cellEditor: 'agSelectCellEditor',
                        cellEditorParams: {
                            values: [true, false],
                        }
                    },
                    {
                        headerName: 'Angle1',
                        headerTooltip: 'Reference Pointing - Angle1',
                        field: 'ref_pointing_angle1',
                        type: 'numberValueColumn',
                        cellRenderer: 'timeInputMask',
                        cellEditor: 'timeInputMask',
                        valueSetter: 'valueSetter',
                        cellStyle: function (params) {
                            if (params.value && !Validator.validateTimeAndAngle(params.value)) {
                                return {backgroundColor: BG_COLOR};
                            } else {
                                return {backgroundColor: ''};
                            }
                        }
                    },
                    {
                        headerName: 'Angle2',
                        headerTooltip: 'Reference Pointing - Angle2',
                        field: 'ref_pointing_angle2',
                        type: 'numberValueColumn',
                        cellRenderer: 'degreeInputMask',
                        cellEditor: 'degreeInputMask',
                        valueSetter: 'valueSetter',
                        cellStyle: function (params) {
                            if (params.value && !Validator.validateAngle(params.value)) {
                                return {backgroundColor: BG_COLOR};
                            } else {
                                return {backgroundColor: ''};
                            }
                        }
                    },
                    {
                        headerName: 'Reference Frame',
                        headerTooltip: 'Reference Pointing - Reference Frame',
                        field: 'ref_pointing_direction_type',
                        cellEditor: 'agSelectCellEditor',
                        cellEditorParams: {
                            values: this.resolvedConstraintSchema.definitions.pointing.properties.direction_type.enum,
                        }
                    },
                    {headerName: 'Target', headerTooltip: 'Refernce Pointing - Target', field: 'ref_pointing_target'}
                ]
            });
        }
        this.constraintVariables.push('md_sun');
        this.constraintVariables.push('md_moon');
        this.constraintVariables.push('md_jupiter');
        columnDefs.push({
            headerName: 'Min_distance', headerTooltip: 'Min_distance', children: [
                {
                    headerName: 'Sun (Degrees)',
                    headerTooltip: 'Sun (Degrees)',
                    field: 'md_sun',
                    cellEditor: 'numericEditor',
                    cellStyle: function (params) {
                        if (params.value) {
                            if (params.value === undefined || params.value === null || isNaN(params.value)) {
                                return {backgroundColor: BG_COLOR};
                            } else if (Number(params.value) < 0 || Number(params.value) > 180) {
                                return {backgroundColor: BG_COLOR};
                            } else {
                                return {backgroundColor: ''};
                            }
                        }
                    },
                    valueFormatter: params => {
                        if (params.value !== '') {
                            const value = parseFloat(params.value);
                            return value >= 0 ? value.toFixed(2) : '0.00';
                        } else {
                            return "0:00";
                        }
                    }
                },
                {
                    headerName: 'Moon (Degrees)',
                    headerTooltip: 'Moon (Degrees)',
                    field: 'md_moon',
                    cellEditor: 'numericEditor',
                    cellStyle: function (params) {
                        if (params.value) {
                            if (params.value === undefined || params.value === null || isNaN(params.value)) {
                                return {backgroundColor: BG_COLOR};
                            } else if (Number(params.value) < 0 || Number(params.value) > 180) {
                                return {backgroundColor: BG_COLOR};
                            } else {
                                return {backgroundColor: ''};
                            }
                        }
                    },
                    valueFormatter: params => {
                        if (params.value !== '') {
                            const value = parseFloat(params.value);
                            return value >= 0 ? value.toFixed(2) : '0.00';
                        } else {
                            return "0:00";
                        }
                    }
                },
                {
                    headerName: 'Jupiter (Degrees)',
                    headerTooltip: 'Jupiter (Degrees)',
                    field: 'md_jupiter',
                    cellEditor: 'numericEditor',
                    cellStyle: function (params) {
                        if (params.value) {
                            if (params.value === undefined || params.value === null || isNaN(params.value)) {
                                return {backgroundColor: BG_COLOR};
                            } else if (Number(params.value) < 0 || Number(params.value) > 180) {
                                return {backgroundColor: BG_COLOR};
                            } else {
                                return {backgroundColor: ''};
                            }
                        }
                    },
                    valueFormatter: params => {
                        if (params.value !== '') {
                            const value = parseFloat(params.value);
                            return value >= 0 ? value.toFixed(2) : '0.00';
                        } else {
                            return "0:00";
                        }
                    }
                },],
        });
        this.colKeyOrder = [...this.colKeyOrder, ...this.constraintVariables];
        return columnDefs;
    }


    /**
     * Function called back from Degree/Time Input Mask to set value in row data.
     *
     * @param {Stirng} cell -> contains Row ID, Column Name, Value, isDegree
     */
    async updateAngle(rowIndex, field, value, isDegree, isValid) { //TODO: refactor this to spreadsheet helper
        let row = {};
        let tmpRowData = [];
        if (field.startsWith('gdef_')) {
            row = this.state.commonRowData[0];
            row[field] = value;
            row['isValid'] = isValid;
            /* - this field is nolonger
            row[field+'value'] = UnitConverter.parseAngle(value);
            */
            tmpRowData = this.state.commonRowData;
            tmpRowData[0] = row;
            await this.setState({commonRowData: tmpRowData});
        } else {
            row = this.state.rowData[rowIndex];
            row[field] = value;
            row['isValid'] = isValid;
            row['isDirty'] = true;
            row['isValidSpecDoc'] = isValid;
            
            tmpRowData = this.state.rowData;
            tmpRowData[rowIndex] = row;
            await this.setState({rowData: tmpRowData, isDirty: true});
            publish('edit-dirty', true);
        }
    }

    /**
     * CallBack Function : update time value in master grid
     */
    async updateTime(rowIndex, field, value, isSetFocus, doBEValidation) {
        let row = {};
        let tmpRowData = [];
        if (field.startsWith('gdef_')) {
            row = this.state.commonRowData[0];
            row[field] = value;
            tmpRowData = this.state.commonRowData;
            tmpRowData[0] = row;
            await this.setState({commonRowData: tmpRowData});

        } else {
            row = this.state.rowData[rowIndex];
            row[field] = value;
            row["isDirty"] = true;
            tmpRowData = this.state.rowData;
            tmpRowData[rowIndex] = row;
            await this.setState({rowData: tmpRowData, isDirty: true});
            publish('edit-dirty', true);
        }
        if (isSetFocus === true) {
            if (field.startsWith('gdef_')) {
                this.state.topGridApi.stopEditing();
                let focusedCell = this.state.topGridColumnApi.getColumn(field)
                this.state.topGridApi.ensureColumnVisible(focusedCell);
                this.state.topGridApi.setFocusedCell(rowIndex, focusedCell);
            } else {
                this.state.gridApi.stopEditing();
                let focusedCell = this.state.gridColumnApi.getColumn(field)
                this.state.gridApi.ensureColumnVisible(focusedCell);
                this.state.gridApi.setFocusedCell(rowIndex, focusedCell);
            }
        }

        if (doBEValidation) {
            if (_.includes(this.constraintVariables, field)) {
                await this.validateSchedulingConstraints(rowIndex, row);
            }
            if (_.includes(this.strategyVariables, field)) {
                await this.validateSpecificationsDoc(rowIndex, row);
            }
        }
    }

    /**
     * Update the Row columns value from external component
     * @param {*} rowIndex
     * @param {*} field
     * @param {*} value
     * @param {Boolean} isMultiselect - It check if particular cell is Multiselect or not
     */
    async updateCell(rowIndex, field, value, isMultiselect, doBEValidation) {
        let row = {};
        let tmpRowData = [];
        if (field.startsWith('gdef_')) {
            row = this.state.commonRowData[0];
            row[field] = value;
            tmpRowData = this.state.commonRowData;
            tmpRowData[0] = row;
            await this.setState({commonRowData: tmpRowData});
            if (isMultiselect) {
                var focusedCell = this.state.topGridColumnApi.getColumn(field)
                this.state.topGridApi.ensureColumnVisible(focusedCell);
                this.state.topGridApi.setFocusedCell(rowIndex, focusedCell);
            }
        } else {
            row = this.state.rowData[rowIndex];
            row[field] = value && value.param_0 ? value.param_0 : value;
            row["isDirty"] = true;
      
            tmpRowData = this.state.rowData;
            tmpRowData[rowIndex] = row;
            await this.setState({rowData: tmpRowData, isDirty: true});
            publish('edit-dirty', true);
            if (isMultiselect) {

                let focusedCell = this.state.gridColumnApi.getColumn(field)
                this.state.gridApi.ensureColumnVisible(focusedCell);
                this.state.gridApi.setFocusedCell(rowIndex, focusedCell);
            }
        }

        if (doBEValidation) {
            if (_.includes(this.constraintVariables, field)) {
                await this.validateSchedulingConstraints(rowIndex, row);
            }
            if (_.includes(this.strategyVariables, field)) {
                await this.validateSpecificationsDoc(rowIndex, row);
            }
        }
    }

    /**
     * Save Scheduling Unit(s) form Excel table
     */
    async saveSchedulingUnit() {
        this.validateGridAndSave();
    }

    /**
     * Validate Grid values on click Save button from UI
     */
    async validateGridAndSave() {
        let validCount = 0;
        let inValidCount = 0;
        let isValidRow = true;
        let errorDisplay = [];
        let warningClipboardData = '';
        const mandatoryKeys = ['suname', 'sudesc', 'priority_queue', 'scheduler', 'min_target_elevation', 'min_calibrator_elevation', 'offset_from', 'offset_to', 'md_sun', 'md_moon', 'md_jupiter', 'param_0~angle1', 'param_0~angle2', 'param_0~direction_type', 'param_1~angle1', 'param_1~angle2', 'param_1~direction_type', 'param_2~angle1', 'param_2~angle2', 'param_2~direction_type'];
        let tmpMandatoryKeys = [];
        let tmpRowData = this.state.rowData;
        const schema = this.state.paramsSchema;

        let columnDefinition = this.columnDefinition;
        let columnDefs = this.state.columnDefs;

        this.state.gridApi.forEachNode(function (node) {
            isValidRow = true;
            let errorMsg = 'Row # [' + (Number(node.rowIndex) + 1) + ']';
            tmpMandatoryKeys = [];
            const rowData = node.data;
            let isFixedTimeScheduler = false;
            let hasData = true;
            if (rowData) {
                errorMsg = errorMsg + ` (${rowData['suname']}) : `
                for (const key of mandatoryKeys) {
                    if (rowData[key] === '') {
                        if (key === 'suname') {
                            if (rowData['sudesc'] !== '') {
                                tmpMandatoryKeys.push(key);
                            } else {
                                hasData = false;
                            }
                        } else if (key === 'sudesc') {
                            if (rowData['suname'] !== '') {
                                tmpMandatoryKeys.push(key);
                            }
                        } else {
                            tmpMandatoryKeys.push(key);
                        }
                    } else if (key === 'scheduler' && rowData[key] === 'fixed_time') {
                        isFixedTimeScheduler = true;
                    }
                }
                const colDefkeys = Object.keys(columnDefinition);
                for (const colDefkey of colDefkeys) {
                    let column = _.find(columnDefs, {'field': colDefkey});
                    if (!column) {
                        const children = _.find(columnDefs, _.flow(
                            _.property('children'),
                            _.partialRight(_.some, {field: colDefkey})
                        ));
                        if (children) {
                            column = _.find(children.children, function (o) {
                                return o.field === colDefkey;
                            });
                        }
                    }
                    if (!column) {
                        column = {header: _.capitalize(colDefkey)};
                    }
                    if (rowData.hasOwnProperty(colDefkey)) {
                        const defProperty = columnDefinition[colDefkey];
                        if (defProperty.hasOwnProperty('required') && defProperty.required) {
                            if (_.isEmpty(rowData[colDefkey] + '')) {
                                errorMsg += column.headerName + ", ";
                                isValidRow = false;
                            }
                        }
                        if (defProperty.hasOwnProperty('type')) {
                            if (rowData[colDefkey] && typeof rowData[colDefkey] === 'string') {
                                rowData[colDefkey] = rowData[colDefkey].trim()
                            }
                            if (rowData[colDefkey] && (typeof rowData[colDefkey] !== defProperty.type && typeof rowData[colDefkey] !== 'string')) {
                                errorMsg += column.headerName + ", ";
                                isValidRow = false;
                            } else {
                                if (defProperty.hasOwnProperty('minimum')) {
                                    if (rowData[colDefkey] < defProperty.minimum) {
                                        errorMsg += column.headerName + ", ";
                                        isValidRow = false;
                                    }
                                }
                                if (defProperty.hasOwnProperty('maximum')) {
                                    if (rowData[colDefkey] > defProperty.maximum) {
                                        errorMsg += column.headerName + ", ";
                                        isValidRow = false;
                                    }
                                }
                                if (defProperty.hasOwnProperty('enum')) {
                                    let containsAll = true;
                                    if (rowData[colDefkey] && rowData[colDefkey] !== '' && rowData[colDefkey].length > 0 && defProperty.hasOwnProperty('multiselect') && defProperty.multiselect) {
                                        if (typeof rowData[colDefkey] !== 'object') {
                                            rowData[colDefkey] = rowData[colDefkey].split(',');
                                        }
                                        containsAll = rowData[colDefkey].length > 0 ? rowData[colDefkey].every(element => {
                                            return defProperty.enum.indexOf(element) !== -1;
                                        }) : true;
                                    } else if (defProperty.hasOwnProperty('multiselect') && !defProperty.multiselect) {
                                        if (rowData[colDefkey] && rowData[colDefkey].length > 0 && !_.includes(defProperty.enum, rowData[colDefkey])) {
                                            containsAll = false;
                                        }
                                    } else {
                                        if (rowData[colDefkey] && rowData[colDefkey].length > 0 && !_.includes(defProperty.enum, rowData[colDefkey])) {
                                            containsAll = false;
                                        }
                                    }
                                    if (!containsAll) {
                                        errorMsg += column.headerName + ", ";
                                        isValidRow = false;
                                    }
                                }
                                if (defProperty.hasOwnProperty('format')) {
                                    if (defProperty.format === '-hh:mm:ss') {
                                        const result = Validator.validateTransitOffset(defProperty, rowData[colDefkey], {}, null);
                                        if (!_.isEmpty(result)) {
                                            errorMsg += column.headerName + ", ";
                                            isValidRow = false;
                                        }
                                    } else if (defProperty.format === 'hh:mm:ss') {
                                        if (defProperty.schema) {
                                            let errors = [];
                                            Validator.validateDurationHHmmss(defProperty.schema, rowData[colDefkey], errors, null);
                                            if (errors.length > 0) {
                                                errorMsg += errors[0].message + ", ";
                                                isValidRow = false;
                                            }
                                        } else {
                                            if (!Validator.isValidHHmmss(rowData[colDefkey], false)) {
                                                errorMsg += column.headerName + ", ";
                                                isValidRow = false;
                                            }
                                        }
                                    } else if (defProperty.format === 'dd:mm:ss') { // for angle2 , check the format and implement the validation
                                        const result = Validator.isValidHHmmss(rowData[colDefkey], false);
                                        if (!_.isEmpty(result)) {
                                            errorMsg += column.headerName + ", ";
                                            isValidRow = false;
                                        }
                                    } else if (defProperty.format === 'date-time') {
                                        if (rowData[colDefkey] !== '' && !moment(rowData[colDefkey], UIConstants.CALENDAR_DATETIME_FORMAT, true).isValid()) {
                                            errorMsg += column.headerName + ", ";
                                            isValidRow = false;
                                        }
                                    } else if (defProperty.format === 'json' && rowData[colDefkey].length > 0) {
                                        try {
                                            const element = typeof rowData[colDefkey] !== "string" ? JSON.stringify(rowData[colDefkey]) : rowData[colDefkey];
                                            JSON.parse(element);
                                        } catch (e) {
                                            errorMsg += column.headerName + ", ";
                                            isValidRow = false;
                                        }

                                    } else if (rowData[colDefkey] !== '' && defProperty.format === 'date-time-array') {
                                        let valid = true;
                                        const dateTimeArray = rowData[colDefkey] ? rowData[colDefkey].split("|") : [];
                                        dateTimeArray.forEach(dateTimeValues => {
                                            if (dateTimeValues !== '') {
                                                const dateTimeValueArray = dateTimeValues.split(",");
                                                if (dateTimeValueArray.length === 2) {
                                                    dateTimeValueArray.forEach(dateTime => {
                                                        if (!moment(dateTime, UIConstants.CALENDAR_DATETIME_FORMAT, true).isValid()) {
                                                            valid = false;
                                                        }
                                                    });
                                                } else {
                                                    valid = false;
                                                }
                                            }
                                        });
                                        if (!valid) {
                                            errorMsg += column.headerName + ", ";
                                            isValidRow = false;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if (tmpMandatoryKeys.length !== mandatoryKeys.length) {
                    for (var i = 0; i < node.columnController.gridColumns.length; i++) {
                        let column = node.columnController.gridColumns[i];
                        if (column.colId !== '0') {
                            if (rowData.hasOwnProperty(column.colId) && typeof rowData[column.colId] === 'string') {
                                rowData[column.colId] = rowData[column.colId].trim();
                            }
                            if (_.includes(tmpMandatoryKeys, column.colId)) {
                                isValidRow = false;
                                errorMsg += column.colDef.headerName + ", ";
                            } else {
                                if (column.colId === 'rank') {
                                    let value = rowData[column.colId] ? rowData[column.colId] : '0';
                                    rowData[column.colId] = value;
                                    const splitValue = _.split((value + ''), ".");
                                    if (rowData[column.colId] < 0 || rowData[column.colId] > 1 || (splitValue.length > 1 && splitValue[1].length > 4)) {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    }
                                } else if (isFixedTimeScheduler && (column.colId === 'timeat') && rowData[column.colId] === '') {
                                    //  isValidRow = false;
                                    //  errorMsg += column.colDef.headerName+", ";
                                } else if (column.colId === 'min_target_elevation' || column.colId === 'min_calibrator_elevation') {
                                    const skyElevationPattern = /^\d{1,2}(\.\d{1,2})?$/;
                                    if (skyElevationPattern.test(rowData[column.colId])) {
                                        const elevationValue = UnitConverter.convertToRadians(Number(rowData[column.colId]));
                                        if (elevationValue < rowData["elevation_min"] || elevationValue > rowData["elevation_max"]) {
                                            isValidRow = false;
                                            errorMsg += column.colDef.headerName + ", ";
                                        }
                                    } else {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    }
                                } else if (column.colId === 'offset_from') {
                                    const tmpTime = _.split(rowData[column.colId], ":");
                                    if (typeof rowData[column.colId] === 'undefined' ||
                                        (tmpTime.length !== 3 || isNaN(tmpTime[1]) || tmpTime[1] > 59 || isNaN(tmpTime[2]) || tmpTime[2] > 59)) {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    } else if (UnitConverter.getHHmmssToSecs(rowData[column.colId]) < rowData['offset_from_min'] || UnitConverter.getHHmmssToSecs(rowData[column.colId]) > rowData['offset_from_max']) {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    }
                                } else if (column.colId === 'offset_to') {
                                    const tmpTime = _.split(rowData[column.colId], ":");
                                    if (typeof rowData[column.colId] === 'undefined' ||
                                        (tmpTime.length !== 3 || tmpTime[1] > 59 || tmpTime[2] > 59)) {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    } else if (UnitConverter.getHHmmssToSecs(rowData[column.colId]) < rowData['offset_to_min'] || UnitConverter.getHHmmssToSecs(rowData[column.colId]) > rowData['offset_to_max']) {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    }
                                } else if (column.colId === 'md_sun' || column.colId === 'md_moon' || column.colId === 'md_jupiter') {
                                    const skyMinPattern = /^\d{1,3}(\.\d{1,2})?$/;
                                    if ((Number(rowData[column.colId]) < 0 || Number(rowData[column.colId]) > 180) || !skyMinPattern.test(rowData[column.colId])) {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    }
                                } else if (_.endsWith(column.colId, "angle1") && !Validator.validateTimeAndAngle(rowData[column.colId])) {
                                    isValidRow = false;
                                    errorMsg += column.colDef.headerName + ", ";
                                } else if (_.endsWith(column.colId, "angle2") && !Validator.validateAngle(rowData[column.colId])) {
                                    isValidRow = false;
                                    errorMsg += column.colDef.headerName + ", ";
                                } else if (_.endsWith(column.colId, "angle3")) {
                                    if (isNaN(rowData[column.colId])) {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    }
                                } else if (_.endsWith(column.colId, "stations")) {
                                    let sgCellValue = rowData[column.colId];
                                    if (!_.includes(sgCellValue, '|')) {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    } else {
                                        let stationGroups = _.split(sgCellValue, "|");
                                        stationGroups.map(stationGroup => {
                                            let sgValue = _.split(stationGroup, ":");
                                            if (rowData['suname'] !== '' && rowData['sudesc'] !== '' && (sgValue[1] === 'undefined' || sgValue[1] === 'NaN' || Number(sgValue[1]) < 0)) {
                                                isValidRow = false;
                                                errorMsg += column.colDef.headerName + ", ";
                                            }
                                            return stationGroup;
                                        });
                                    }
                                } else {
                                    if (rowData.hasOwnProperty(column.colId) && rowData[column.colId] !== '' && column.colDef.cellEditorParams) {
                                        if (!_.includes(column.colDef.cellEditorParams.values, rowData[column.colId])) {
                                            errorMsg += column.colDef.headerName + ", ";
                                            isValidRow = false;
                                        }
                                    } else if (rowData.hasOwnProperty(column.colId) && rowData[column.colId] !== '') {
                                        if (column.colDef.minimum && Number(rowData[column.colId]) < column.colDef.minimum) {
                                            errorMsg += column.colDef.headerName + ", ";
                                            isValidRow = false;
                                        }
                                        if (isValidRow && column.colDef.maximum && Number(rowData[column.colId]) >= column.colDef.maximum) {
                                            errorMsg += column.colDef.headerName + ", ";
                                            isValidRow = false;
                                        }

                                        if (column.colDef.type && column.colDef.type === 'integer') { //TODO: integer not allowed
                                            rowData[column.colId] = parseInt(rowData[column.colId]);
                                        } else if (column.colDef.type && column.colDef.type === 'number') {
                                            rowData[column.colId] = Number(rowData[column.colId]);
                                        }
                                    }
                                }

                                if (column.colDef.max_length) {
                                    let sgCellValue = rowData[column.colId];
                                    if (sgCellValue && sgCellValue !== '' && sgCellValue.length > column.colDef.max_length) {
                                        isValidRow = false;
                                        errorMsg += column.colDef.headerName + ", ";
                                    }
                                }
                            }
                        }
                        let prefix = column.colId.split("~");
                        if (schema.properties && prefix[0] && schema.properties[prefix[0]]) {
                            const type = schema.properties[prefix[0]].type;
                            const minvalue = schema.properties[prefix[0]].minimum;
                            if (type && type === 'integer') {
                                if (isNaN(rowData[column.colId])) {
                                    isValidRow = false;
                                    errorMsg += column.colDef.headerName + ", ";
                                } else if (minvalue && rowData[column.colId] < minvalue) {
                                    isValidRow = false;
                                    errorMsg += column.colDef.headerName + ", ";
                                }
                            }
                        }
                    }
                    
                }
            }
            if (hasData) {
                if (isValidRow) {
                    validCount++;
                    tmpRowData[node.rowIndex]['isValid'] = true;
                } else {
                    inValidCount++;
                    tmpRowData[node.rowIndex]['isValid'] = false;
                    warningClipboardData = warningClipboardData + errorMsg.slice(0, -2) + '\n';
                    errorDisplay.push(errorMsg.slice(0, -2));
                }
            }
        });
        this.warningClipboardData = warningClipboardData;
        if (validCount > 0 && inValidCount === 0) {
            // save SU directly
            this.saveSU();
        } else if (validCount === 0 && inValidCount === 0) {
            // leave with no change
            this.showIcon = true;
            this.dialogMsg = 'No valid Scheduling Unit found !';
            this.dialogType = 'warning';
            this.onClose = () => {
                this.setState({confirmDialogVisible: false});
            };
            this.setState({confirmDialogVisible: true});
        } else {
            this.setState({
                validCount: validCount,
                inValidCount: inValidCount,
                tmpRowData: tmpRowData,
                errorDisplay: errorDisplay,
                confirmDialogVisible: true,
            });
            this.callBackFunction = this.saveSU;
            this.state.gridApi.redrawRows();
            this.showIcon = true;
            this.onCancel = () => {
                this.setState({confirmDialogVisible: false});
            };
            this.onClose = () => {
                this.setState({confirmDialogVisible: false});
            };
            this.actions = [
                {
                    id: "copy_warning_btn",
                    style: {float: 'left', backgroundColor: '#ffbb08', borderColor: '#ffbb08'},
                    title: "Copy Warning",
                    callback: this.copyWarningContent
                },
                {id: "yes_btn", title: "Yes", callback: this.saveSU},
                {id: "cancel_btn", title: "No", className: 'act-btn-cancel', callback: this.onCancel},];
            this.dialogType = "confirmation";
            this.dialogHeader = "Save Scheduling Unit(s)";
            this.dialogMsg = "Some of the Scheduling Unit(s) has invalid data, Do you want to ignore and save valid Scheduling Unit(s) only?";
            this.dialogContent = this.showDialogContent;
        }
    }

    /**
     * Copy warning content to clipboard
     */

    async copyWarningContent() {
        await navigator.clipboard.writeText(this.warningClipboardData);
        this.growl.show({severity: 'success', summary: '', detail: 'Warning copied to clipboard '});
    }

    /**
     * Show the content in custom dialog
     */
    showDialogContent() {
        if (typeof this.state.errorDisplay === 'undefined' || this.state.errorDisplay.length === 0) {
            return "";
        } else {
            return <> <br/>Invalid Rows:- Row # , Name and Invalid
                columns<br/>{this.state.errorDisplay && this.state.errorDisplay.length > 0 &&
                    this.state.errorDisplay.map((msg, index) => (
                        <React.Fragment key={index + 10}>
                            <span key={'label1-' + index}>{msg}</span> <br/>
                        </React.Fragment>
                    ))} </>
        }
    }

    /**
     * prepare Constraint Error dialog
     * @returns
     */
    showConstraintErrorDialog() {
        const keys = Object.keys(this.constraintsValidationMessage);
        return <>
            <React.Fragment>
                <br/>
                <tr style={{backgroundColor: 'lightgray', textAlign: 'center'}}>
                    <th style={{width: '5em'}}><span>Row Id </span></th>
                    <th style={{width: '20em'}}><span>SchedulingUnit Name </span></th>
                    <th style={{width: '45em', textAlign: 'center'}}><span> Error Details</span></th>
                </tr>
            </React.Fragment>
            {keys.map((key, index) => (
                <React.Fragment key={index + 10}>
                    <tr style={{backgroundColor: index % 2 === 0 ? '' : '#e6e5e5'}}>
                        <td style={{width: '5em', paddingLeft: '7px'}}><span
                            key={'label1-' + index}>{(parseInt(key) + 1)} </span></td>
                        <td style={{width: '20em', paddingLeft: '7px'}}><span
                            key={'label1-' + index}>{this.constraintsValidationMessage[key]['suName']} </span></td>
                        <td style={{width: '45em'}}><span
                            key={'label1-' + index}>{this.constraintsValidationMessage[key]['message']}</span></td>
                    </tr>
                </React.Fragment>
            ))}
        </>
    }

    /**
     * prepare Specification Error dialog
     * @returns
     */
    showSpecificationErrorDialog() {
        const keys = Object.keys(this.specificationValidationMessage);
        return <>
            <React.Fragment>
                <br/>
                <tr style={{backgroundColor: 'lightgray', textAlign: 'center'}}>
                    <th style={{width: '5em'}}><span>Row Id </span></th>
                    <th style={{width: '20em'}}><span>SchedulingUnit Name </span></th>
                    <th style={{width: '45em', textAlign: 'center'}}><span> Error Details</span></th>
                </tr>
            </React.Fragment>
            {keys.map((key, index) => (
                <React.Fragment key={index + 10}>
                    <tr style={{backgroundColor: index % 2 === 0 ? '' : '#e6e5e5'}}>
                        <td style={{width: '5em', paddingLeft: '7px'}}><span
                            key={'label1-' + index}>{(parseInt(key) + 1)} </span></td>
                        <td style={{width: '20em', paddingLeft: '7px'}}><span
                            key={'label1-' + index}>{this.specificationValidationMessage[key]['suName']} </span></td>
                        <td style={{width: '45em'}}><span
                            key={'label1-' + index}>{this.specificationValidationMessage[key]['message']}</span></td>
                    </tr>
                </React.Fragment>
            ))}
        </>
    }

    /**
     * Prepare Scheduling Unit from Excel table
     * @param {*} suRow
     * @returns
     */
    async prepareObservStrategyFromExcelValue(suRow) {
        let colKeys = Object.keys(suRow);
        let paramsOutput = {};
        let parameters = _.map(this.state.observStrategy.template.parameters, 'name');
        const schema = this.state.paramsSchema;
        for (const colKey of colKeys) {
            if (_.includes(["true", "false", "FALSE", "TRUE"], suRow[colKey])) {
                suRow[colKey] = JSON.parse(_.lowerCase(suRow[colKey]));
            }
            let prefix = colKey.split("~");
            const durationKey = prefix.length === 2 ? prefix[1] : colKey;
            if (prefix.length > 1 && parameters.indexOf(prefix[0]) >= 0) {
                var res = Object.keys(suRow).filter(objKey => {
                    const tmp = objKey.split("~");
                    return tmp[0] === prefix[0];
                });
                if (res && res.length > 1) {
                    let res = paramsOutput[prefix[0]];
                    if (prefix[1] === 'angle1' || prefix[1] === 'angle2') {
                        suRow[colKey] = UnitConverter.parseAngle(suRow[colKey]);
                    }
                    if (res) {
                        res[prefix[1]] = suRow[colKey];
                    } else {
                        res = {};
                        res[prefix[1]] = suRow[colKey];
                        paramsOutput[prefix[0]] = res;
                    }
                } else {
                    if (colKey.endsWith('Beamformers')) {
                        let result = suRow[colKey];
                        if (result['param_0']) {
                            paramsOutput[prefix[0]] = result['param_0'];
                        } else {
                            paramsOutput[prefix[0]] = result;
                        }
                    } else if (_.includes(this.durationList, durationKey)) {
                        paramsOutput[prefix[0]] = suRow[colKey] === undefined ? 0 : UnitConverter.getHHmmssToSecs(suRow[colKey]);
                    } else {
                        if (schema.properties && schema.properties[prefix[0]]) {
                            const type = schema.properties[prefix[0]].type;
                            if (type === 'array' && schema.properties[prefix[0]].items 
                            && schema.properties[prefix[0]]['items'].type && schema.properties[prefix[0]]['items'].type === 'integer') {
                                paramsOutput[prefix[0]] = Array.isArray(suRow[colKey]) ? suRow[colKey] : UnitConverter.getSubbandOutput(suRow[colKey])
                            }   else if(type === 'array' && !Array.isArray(suRow[colKey])) {
                                paramsOutput[prefix[0]] = suRow[colKey] === '' ? []:suRow[colKey].split(",");
                            }   else if(type === 'integer' && !isNaN(parseInt(suRow[colKey]))) {
                                paramsOutput[prefix[0]] = parseInt(suRow[colKey]);
                            } else if (type === 'numeric' && !isNaN(parseInt(suRow[colKey]))) {
                                paramsOutput[prefix[0]] = parseInt(suRow[colKey]);
                            } else {
                                paramsOutput[prefix[0]] = suRow[colKey];
                            }
                        } else {
                            paramsOutput[prefix[0]] = suRow[colKey];
                        }
                    }
                }
            } else {
                paramsOutput[prefix[0]] = suRow[colKey];
            }
        }
        this.setState({paramsOutput: paramsOutput})
        let observationIdSet = this.state.observationIdSet;
        let observStrategy = _.cloneDeep(this.state.observStrategy);
        observationIdSet.push(observStrategy.id);
        const $refs = await $RefParser.resolve(observStrategy.template);
        observStrategy.template.parameters.forEach(async (param, index) => {
            let refIndex = 0;
            param.refs.forEach(() => {
                $refs.set(observStrategy.template.parameters[index]['refs'][refIndex], this.state.paramsOutput[param.name]);
                refIndex++;
            });
        });
        this.setState({observationIdSet: observationIdSet});
        return observStrategy;
    }

    /**
     * Prepare Constraint from Excel table
     * @param {*} suRow
     * @returns
     */
    async prepareConstraintFromExcelValue(suRow) {
        let between = this.getBetweenDateValue(suRow.between);
        let notbetween = this.getBetweenDateValue(suRow.notbetween);
        let constraint = null;
        if (suRow.id > 0) {
            let schedulingUnit = _.find(this.state.schedulingUnitList, {'id': suRow.id});
            constraint = schedulingUnit.scheduling_constraints_doc;
        }
        if (!constraint) {
                     constraint = this.defaultConstraintDoc;
        }
        //If No SU Constraint create default ( maintain default struc)
        constraint['scheduler'] = suRow.scheduler;
        // Remove empty time constraint properties
        if (this.isNotEmpty(suRow.timeat)) {
            constraint.time.at = `${moment(suRow.timeat).format(UIConstants.UTC_DATE_TIME_MS_FORMAT, {trim: false})}Z`;
        } else {
            delete constraint.time.at;
        }
        if (this.isNotEmpty(suRow.timeafter)) {
            constraint.time.after = `${moment(suRow.timeafter).format(UIConstants.UTC_DATE_TIME_MS_FORMAT, {trim: false})}Z`;
        } else {
            delete constraint.time.after;
        }
        if (this.isNotEmpty(suRow.timebefore)) {
            constraint.time.before = `${moment(suRow.timebefore).format(UIConstants.UTC_DATE_TIME_MS_FORMAT, {trim: false})}Z`;
        } else {
            delete constraint.time.before;
        }

        if (this.isNotEmpty(between)) {
            constraint.time.between = between;
        } else {
            delete constraint.time.between;
        }
        if (this.isNotEmpty(notbetween)) {
            constraint.time.not_between = notbetween;
        } else {
            delete constraint.time.not_between;
        }
        this.state.daily.forEach(daily => {
            if (_.includes(suRow.daily, daily)) {
                constraint.daily[daily] = true;
            } else {
                constraint.daily[daily] = false;
            }
        })
        let min_distance_res = {};
        min_distance_res['sun'] = isNaN(suRow.md_sun) ? 0 : suRow.md_sun;
        min_distance_res['moon'] = isNaN(suRow.md_moon) ? 0 : suRow.md_moon;
        min_distance_res['jupiter'] = isNaN(suRow.md_jupiter) ? 0 : suRow.md_jupiter;
        constraint.sky.min_distance = min_distance_res;

        let transit_offset_res = {};
        transit_offset_res['from'] = UnitConverter.getHHmmssToSecs(suRow.offset_from);
        transit_offset_res['to'] = UnitConverter.getHHmmssToSecs(suRow.offset_to);
        if (transit_offset_res) {
            constraint.sky.transit_offset = transit_offset_res;
        }

        constraint.sky.min_elevation.target = isNaN(suRow.min_target_elevation) ? 0 : suRow.min_target_elevation;
        constraint.sky.min_elevation.calibrator = isNaN(suRow.min_calibrator_elevation) ? 0 : suRow.min_calibrator_elevation;

        let ref_pointing = {enabled: false, pointing: {target: "-"}};
        ref_pointing.enabled = suRow.ref_pointing_enabled;
        ref_pointing.pointing.angle1 = UnitConverter.parseAngle(suRow.ref_pointing_angle1);
        ref_pointing.pointing.angle2 = UnitConverter.parseAngle(suRow.ref_pointing_angle2);
        ref_pointing.pointing.direction_type = suRow.ref_pointing_direction_type;
        ref_pointing.pointing.target = suRow.ref_pointing_target;
        constraint.sky.reference_pointing = ref_pointing;

        return constraint;
    }

    getStationGroupFromRowData(rowData, stationField) {
        let sgCellValue = rowData[stationField];
        let tmpStationGroups = [];
        let tmpStationGroup = {};
        let stationGroups = _.split(sgCellValue, "|");
        stationGroups.map(stationGroup => {
            tmpStationGroup = {};
            let sgValue = _.split(stationGroup, ":");
            if (sgValue && sgValue[0].length > 0) {
                let stationArray = _.split(sgValue[0], ",");
                tmpStationGroup['stations'] = stationArray;
                let missingStation = (sgValue[1]) ? sgValue[1] : 0;
                tmpStationGroup['max_nr_missing'] = Number(missingStation);
                tmpStationGroups.push(tmpStationGroup);
            }
            return stationGroup;
        });
        return tmpStationGroups;
    }

    /**
     * Save/Update Scheduling Unit(s)
     */
    async saveSU() {
        let newSUCount = 0;
        let existingSUCount = 0;
        let isUpdated = true;
        try {
            this.setState({
                confirmDialogVisible: false,
                showSpinner: true
            });
            let newSU = this.state.schedulingUnit;
            let suStatus = [];
            let processCount = 0;
            const validRows = _.filter(this.state.rowData, {"isValid": true, "isDirty": true});
            this.setState({progressPercent: 0});
            for (const validRow of validRows) {
                let suRow = _.cloneDeep(validRow);
                delete suRow['custId'];
                if (!suRow['isValid']) {
                    continue;
                }
                let observStrategy = await this.prepareObservStrategyFromExcelValue(suRow);

                const queue = _.find(this.priorityQueuelist, {value: suRow.priority_queue});
                if (queue) {
                    suRow.priority_queue = queue.value;
                } else {
                    delete suRow['priority_queue'];
                }
                //Stations

                let tmpObservStrategy = null;
                if (this.state.observStrategy) {
                    tmpObservStrategy = _.cloneDeep(this.state.observStrategy);
                    const $refs = await $RefParser.resolve(tmpObservStrategy.template);
                    tmpObservStrategy.template.parameters.forEach(async (param, index) => {
                        let refIndex = 0;
                        for (const ref of param.refs) {
                            let station_groups = [];
                            if (ref.includes("station_groups")) {
                                station_groups = this.getStationGroupFromRowData(suRow, param.name);//'Station Groups - '+taskName
                                $refs.set(tmpObservStrategy.template.parameters[index]['refs'][refIndex], station_groups);
                            } else {
                                $refs.set(tmpObservStrategy.template.parameters[index]['refs'][refIndex], this.state.paramsOutput[param.name]);
                            }
                            refIndex++;
                        }
                    });
                }

                let taskDrafts = [];
                if (suRow.id === 0) {
                    for (const taskName in observStrategy.template.tasks) {
                        let task = observStrategy.template.tasks[taskName];
                        if (task.specifications_doc.station_configuration?.station_groups) {
                            let sgCellValue = suRow['Station Groups - ' + taskName];
                            let tmpStationGroups = [];
                            let tmpStationGroup = {};
                            let stationGroups = _.split(sgCellValue, "|");
                            stationGroups.map(stationGroup => {
                                tmpStationGroup = {};
                                let sgValue = _.split(stationGroup, ":");
                                if (sgValue && sgValue[0].length > 0) {
                                    let stationArray = _.split(sgValue[0], ",");
                                    tmpStationGroup['stations'] = stationArray;
                                    let missingStation = (sgValue[1]) ? sgValue[1] : 0;
                                    tmpStationGroup['max_nr_missing'] = Number(missingStation);
                                    tmpStationGroups.push(tmpStationGroup);
                                }
                                return stationGroup;
                            });
                        }
                    }
                } else {
                    let taskData = _.find(this.state.schedulingUnitList, {'id': suRow.id});
                    if (taskData) {
                        taskDrafts = taskData.taskDrafts;
                    }
                    for (const task of taskDrafts) {
                        if (task.specifications_doc.station_configuration?.station_groups) {
                            let sgCellValue = suRow[('Station Groups - ' + task.name)];
                            let tmpStationGroups = [];
                            let tmpStationGroup = {};
                            let stationGroups = _.split(sgCellValue, "|");
                            stationGroups.map(stationGroup => {
                                tmpStationGroup = {};
                                let sgValue = _.split(stationGroup, ":");
                                if (sgValue && sgValue[0].length > 0) {
                                    let stationArray = _.split(sgValue[0], ",");
                                    tmpStationGroup['stations'] = stationArray;
                                    let missingStation = (sgValue[1]) ? sgValue[1] : 0;
                                    tmpStationGroup['max_nr_missing'] = Number(missingStation);
                                    tmpStationGroups.push(tmpStationGroup);
                                }
                                return stationGroup;
                            });
                        }
                    }
                }
                let isNewConstraint = false;
                let newConstraint = {};
                let constraint = await this.prepareConstraintFromExcelValue(suRow);
                if (suRow.id === 0) {
                    isNewConstraint = true;
                }

                UnitConverter.degreeToRadians(constraint.sky);

                if (isNewConstraint) {
                    newSU['scheduling_constraints_doc'] = constraint;
                }

                if (suRow.id === 0) {
                    newConstraint['scheduling_constraints_doc'] = constraint;
                    newConstraint['id'] = this.state.constraintId;
                    newConstraint['constraint'] = {'url': ''};
                    newConstraint.constraint.url = this.state.constraintUrl;
                }
                let suUpdateStatus = {};
                if (suRow.id > 0 && this.isNotEmpty(suRow.suname) && this.isNotEmpty(suRow.sudesc)) {
                    newSU = _.find(this.state.schedulingUnitList, {'id': suRow.id});
                    newSU['name'] = suRow.suname;
                    newSU['description'] = suRow.sudesc;
                    newSU['output_pinned'] = this.state.selectedProject[0].auto_pin;
                    newSU['rank'] = suRow.rank && suRow.rank.length > 0 ? _.round(parseFloat(suRow.rank), 4) : suRow.rank;
                    if (suRow.priority_queue) {
                        newSU['priority_queue'] = (_.find(this.priorityQueuelist, {"value": suRow.priority_queue})).url;
                    }

                    suUpdateStatus['suName'] = suRow.suname;
                    suUpdateStatus['action'] = 'Update';
                    newSU.specifications_doc['parameters'] = tmpObservStrategy.template.parameters;
                    newSU.specifications_doc['tasks'] = tmpObservStrategy.template.tasks;
                    let updateSu = await ScheduleService.updateSchedulingUnitDraft(newSU);
                    suUpdateStatus['suStatus'] = "Success";
                    suUpdateStatus['suId'] = updateSu.data.id;
                    if (updateSu && updateSu['message'] === 'Success') {
                        suUpdateStatus['suStatus'] = "Success";
                        existingSUCount++;
                    } else {
                        suUpdateStatus['suStatus'] = "Failed";
                    }

                } else if (suRow.id === 0 && this.isNotEmpty(suRow.suname) && this.isNotEmpty(suRow.sudesc)) {
                    let newSchedulueUnit = {
                        description: suRow.sudesc,
                        name: suRow.suname,
                        rank: suRow.rank && suRow.rank.length > 0 ? _.round(parseFloat(suRow.rank), 4) : suRow.rank,
                        priority_queue: suRow.priority_queue,
                        scheduling_constraints_template_id: newSU['scheduling_constraints_template_id'],
                        scheduling_set_id: this.state.selectedSchedulingSetId,
                        observation_strategy_template_id: this.state.observStrategy.id,
                        output_pinned: this.state.selectedProject[0].auto_pin
                    }
                    suUpdateStatus['suName'] = suRow.suname;
                    suUpdateStatus['action'] = 'Create';
                    let outputParam = {};
                    for (const parameter of this.state.observStrategy.template.parameters) {
                        for (const ref of parameter.refs) {
                            let paramPath = ref.replaceAll('#/', '');
                            paramPath = paramPath.replaceAll('/', '.');
                            const value = _.get(tmpObservStrategy.template, paramPath);
                            if (value !== undefined) {
                                outputParam = _.set(outputParam, paramPath, value);
                            }
                        }
                    }
                    outputParam['scheduling_constraints_doc'] = constraint;
                    let savedSU = await ScheduleService.saveSchedulingUnit(newSchedulueUnit, outputParam);
                    if (savedSU && savedSU['message'] === 'Success') {
                        suUpdateStatus['suStatus'] = "Success";
                        suUpdateStatus['suId'] = savedSU.data.id;
                        newSUCount++;
                    } else {
                        suUpdateStatus['suStatus'] = "Failed";
                    }
                }
                suStatus.push(suUpdateStatus);
                this.setState({progressPercent: Math.round((++processCount / validRows.length) * 100)});
            }

            if ((newSUCount + existingSUCount) > 0) {
                this.setState({suStatus: suStatus});
                this.dialogType = "success";
                this.dialogHeader = "Success";
                this.showIcon = true;
                this.dialogWidth = "60vw";
                if (isUpdated) {
                    this.dialogMsg = newSUCount + ' Scheduling Units are created & ' + existingSUCount + ' Scheduling Units are updated successfully.';
                } else {
                    this.dialogHeader = "Warning";
                    this.dialogMsg = newSUCount + ' Scheduling Units are created & ' + existingSUCount + ' Scheduling Units are updated successfully, and there are some Schedule Unit failed to create/update';
                }
                this.actions = null;
                this.dialogContent = this.getSchedulingDialogContent;
                this.onCancel = null;
                this.onClose = this.reset;
                this.callBackFunction = null;
                this.setState({
                    isDirty: false,
                    showSpinner: false,
                    confirmDialogVisible: true, /*dialog: dialog,*/
                    isAGLoading: true,
                    copyHeader: false,
                    rowData: []
                });
                publish('edit-dirty', false);
            } else {
                this.setState({isDirty: false, showSpinner: false,});
                publish('edit-dirty', false);
                this.growl.show({
                    severity: 'warn',
                    summary: 'Warning',
                    detail: 'No new/changes in the Scheduling Units list to create/update'
                });
            }
        } catch (err) {
            console.error(err);
            this.growl.show({
                severity: 'error',
                summary: 'Error Occured',
                detail: 'Unable to create/update Scheduling Units'
            });
            this.setState({showSpinner: false});
        }
    }

    /**
     * Prepare constraint from SU row
     * @param {Json Object} suRowData
     * @returns Json Object - constraint value
     */
    async getConstraintValueFromGrid(suRowData) {
        let constraint = await this.prepareConstraintFromExcelValue(suRowData);
        await UnitConverter.degreeToRadians(constraint.sky);
        return constraint;
    }

    /**
     * Prepare Specification doc from SU row
     * @param {Json Object} suRowData
     * @returns Json Object - specification value
     */
    async getUpdatedSpecificationDoc(suRowData) {
        //Prepare output Param
        await this.prepareObservStrategyFromExcelValue(suRowData);
        let tmpObservStrategy = null;
        if (this.state.observStrategy) {
            tmpObservStrategy = _.cloneDeep(this.state.observStrategy);
            const $refs = await $RefParser.resolve(tmpObservStrategy.template);
            tmpObservStrategy.template.parameters.forEach(async (param, index) => {
                let refIndex = 0;
                for (const ref of param.refs) {
                    let station_groups = [];
                    if (ref.includes("station_groups")) {
                        station_groups = this.getStationGroupFromRowData(suRowData, param.name);
                        $refs.set(tmpObservStrategy.template.parameters[index]['refs'][refIndex], station_groups);
                    } else {
                        $refs.set(tmpObservStrategy.template.parameters[index]['refs'][refIndex], this.state.paramsOutput[param.name]);
                    }
                    refIndex++;
                }
            });
        }
        return tmpObservStrategy;
    }

    /**
     * Common function validate specification docs
     * @param {String} template Template url
     * @param {Object} updatedSpecificationDoc Object which needs to be validated
     */
    async validateSpecificationsDoc(rowIndex, suRowData, isValid) {
        let validationMessage = '';
        let observStrategy = await this.getUpdatedSpecificationDoc(_.cloneDeep(suRowData));
        if (suRowData['suname'] !== "" && suRowData['sudesc'] !== "" && observStrategy) {
            const promises = [];
            promises.push(ScheduleService.validateSpecificationsDoc(observStrategy.scheduling_unit_template, observStrategy.template))
            const validation_result = await Promise.all(promises);
            if (validation_result.length > 0 && !validation_result[0]['valid']) {
                validationMessage = "Task Parameters specification is not valid";
                this.isSpecificationValid = false;
                this.setState({
                    validSpecification: validation_result[0]['valid'],
                    specificationValidationMessage: validationMessage
                });
                this.specificationValidationMessage[rowIndex] = {
                    message: validation_result[0]['message'].replace('Could not validate the specifications_doc. Error:', ''),
                    suName: suRowData.suname
                };
            } else {
                delete this.specificationValidationMessage[rowIndex];
                if (Object.keys(this.specificationValidationMessage).length > 0) {
                    validationMessage = "Task Parameters specification is not valid";
                }
                this.setState({specificationValidationMessage: validationMessage});
            }
        }
    }

    /**
     * Common function validate constraints
     */
    async validateSchedulingConstraints(rowIndex, suRowData) {
        let constraint = await this.getConstraintValueFromGrid(_.cloneDeep(suRowData));
        if (suRowData['suname'] !== "" && suRowData['sudesc'] !== "" && constraint) {
            const promises = [];
            let validationMessage = '';
            promises.push(ScheduleService.validateSpecificationsDoc(this.constraintSchema.url, constraint))
            const validation_result = await Promise.all(promises);
            if (validation_result.length > 0 && !validation_result[0]['valid']) {
                this.constraintsValidationMessage[rowIndex] = {
                    message: validation_result[0]['message'].replaceAll('Could not validate the specifications_doc. Error:', ''),
                    suName: suRowData.suname
                };
                validationMessage = "Constraints specification is not valid";
                this.isConstraintValid = false;
                this.setState({
                    validConstraints: validation_result[0]['valid'],
                    constraintsValidationMessage: validationMessage
                });
            } else {
                delete this.constraintsValidationMessage[rowIndex];
                if (Object.keys(this.constraintsValidationMessage).length > 0) {
                    validationMessage = "Constraints specification is not valid";
                }
                this.setState({constraintsValidationMessage: validationMessage});
            }
        }
    }

    /**
     * Prepare Scheduling Unit(s) details to show on confirmation dialog
     */
    getSchedulingDialogContent() {
        let suStatus = this.state.suStatus;
        return <>
            {suStatus.length > 0 &&
                <div style={{marginTop: '1em'}}>
                    <b>Scheduling Unit(s) status</b>
                    <DataTable value={suStatus} resizableColumns columnResizeMode="expand" className="card"
                               style={{paddingLeft: '0em'}}>
                        <Column field="suId" header="Scheduling Unit ID"></Column>
                        <Column field="suName" header="Scheduling Unit Name"></Column>
                        <Column field="action" header="Action"></Column>
                        <Column field="suStatus" header="Scheduling Unit Status"></Column>
                        {/* <Column field="taskStatus" header="Task(s) Status"></Column> */}
                    </DataTable>
                </div>
            }
        </>
    }

    /**
     * Convert the date to string value for Between And Not-Between Columns
     * @param {*} dates
     */
    getBetweenStringValue(dates) {
        let returnDate = '';
        if (dates) {
            dates.forEach(utcDateArray => {
                returnDate += moment.utc(utcDateArray.from).format(UIConstants.CALENDAR_DATETIME_FORMAT) + ",";
                returnDate += moment.utc(utcDateArray.to).format(UIConstants.CALENDAR_DATETIME_FORMAT) + "|";
            });
        }
        return returnDate;
    }


    /**
     * convert String to Date value for Between And Not-Between Columns
     */
    getBetweenDateValue(betweenValue) {
        let returnDate = [];
        if (betweenValue) {
            let rowDateArray = _.split(betweenValue, "|");
            rowDateArray.forEach(betweenDates => {
                let betweendate = _.split(betweenDates, ",");
                let dateres = {};
                if (betweendate && betweendate.length === 2) {
                    dateres['from'] = `${moment(betweendate[0]).format(UIConstants.UTC_DATE_TIME_MS_FORMAT, {trim: false})}Z`;
                    dateres['to'] = `${moment(betweendate[1]).format(UIConstants.UTC_DATE_TIME_MS_FORMAT, {trim: false})}Z`;
                    returnDate.push(dateres);
                }
            })
        }
        return returnDate;
    }

    /**
     * warn before cancel the page if any changes detected
     */
    checkIsDirty() {
        if (this.state.isDirty) {
            this.showIcon = true;
            this.dialogType = "confirmation";
            this.dialogHeader = "Add Multiple Scheduling Unit(s)";
            this.dialogMsg = "Do you want to discard your changes? Your changes to the Scheduling Unit(s) will not be saved.";
            this.dialogContent = "";
            this.dialogHeight = '5em';
            this.actions = [{id: "yes", title: 'Discard', callback: this.cancelCreate, className: 'act-btn-dispose'},
                {id: "no", title: 'Cancel', className: 'act-btn-cancel', callback: this.close}];
            this.onClose = this.close;
            this.setState({
                confirmDialogVisible: true,
            });
        } else {
            this.cancelCreate();
        }
    }

    /**
     * Refresh the grid with updated data
     */
    async reset() {
        if (this.state.confirmDialogVisible) {
            // Fetches only the observation_strategy_template_id of the SUDrafts of selected SUSet
            let schedulingUnitList = await ScheduleService.getSchedulingBySet(this.state.selectedSchedulingSetId, "observation_strategy_template_id");
            schedulingUnitList = _.filter(schedulingUnitList, {'observation_strategy_template_id': this.state.observStrategy.id});
            this.setState({
                schedulingUnitList: schedulingUnitList,
                confirmDialogVisible: false,
                isDirty: false
            });
            publish('edit-dirty', false);
            this.isNewSet = false;
            await this.prepareScheduleUnitListForGrid();
            this.state.gridApi.setRowData(this.state.rowData);
            this.state.gridApi.redrawRows();
        }
    }

    /**
     * Cancel SU creation and redirect
     */
    cancelCreate() {
        publish('edit-dirty', false);
        const pageTracker = JSON.parse(sessionStorage.getItem("pageTracker"));
        this.setState({redirect: pageTracker.prevPath});
    }

    /**
     * Set state to copy the table header to clipboard
     * @param {*} value
     */
    async copyHeader(value) {
        await this.setState({'copyHeader': value});
    }


    /**
     * Copy the table header to clipboard
     */
    async copyOnlyHeader() {
        this.setState({fade: true});
        let clipboardData = '';
        if (this.state.gridColumnApi) {
            var columnsName = this.state.gridColumnApi.getAllGridColumns();
            var line = '';
            if (columnsName) {
                columnsName.map(column => {
                    if (column.colId !== '0') {
                        line += column.colDef.headerName + '\t';
                    }
                    return column;
                });
            }
            line = _.trim(line);
            clipboardData += line + '\r\n';
            clipboardData = _.trim(clipboardData);
            await navigator.clipboard.writeText(clipboardData);
            this.growl.show({severity: 'success', summary: '', detail: 'Header copied to clipboard '});
        }
    }

    /**
     * Read Data from clipboard
     */
    async readClipBoard() {
        let data = await navigator.clipboard.readText();
        return data;
    }

    /**
     * Function to copy the data to clipboard
     */
    async copyToClipboard() {
        let  columnsName = this.state.gridColumnApi.getAllGridColumns();
        let selectedRows = this.state.gridApi.getSelectedRows();
        let clipboardData = '';
        let copySingleCell = false;
        let focusedCell = this.state.gridApi.getFocusedCell();
        if (focusedCell && focusedCell.column['colId'] === '0') {
            if (this.state.copyHeader) {
                let line = '';
                columnsName.map(column => {
                    if (column.colId !== '0') {
                        line += column.colDef.headerName + '\t';
                    }
                    return column;
                })
                line = _.trim(line);
                clipboardData += line + '\r\n';
            }
            for (var rowCount = 0; rowCount < selectedRows.length; rowCount++) {
                const rowData = selectedRows[rowCount];
                let line = '';
                for (var count = 0; count < this.state.colKeyOrder.length; count++) {
                    let value = ' ';
                    const key = this.state.colKeyOrder[count];
                    if (key.endsWith('Beamformers')) {
                        let tmp = rowData[key];
                        if (tmp['param_0']) {
                            value = JSON.stringify(tmp['param_0']);
                        } else {
                            value = JSON.stringify(tmp);
                        }
                    } else {
                        value = rowData[key];
                    }
                    if (value === undefined) {
                        value = ' ';
                    }
                    line += value + ((count === this.state.colKeyOrder.length - 1) ? '' : '\t');
                }
                clipboardData += line + ((rowCount === selectedRows.length - 1) ? '' : '\r\n');
            }
        } else {
            copySingleCell = true;
            if (focusedCell['column']['colId'].endsWith('Beamformers')) {
                let tmp = this.state.rowData[focusedCell.rowIndex][focusedCell['column']['colId']];
                if (tmp['param_0']) {
                    clipboardData = JSON.stringify(tmp['param_0']);
                } else {
                    clipboardData = focusedCell['column']['colId'] + '\t' + JSON.stringify(tmp);
                }
            } else {
                clipboardData = focusedCell['column']['colId'] + '\t' + this.state.rowData[focusedCell.rowIndex][focusedCell['column']['colId']];
            }
        }
        try {
            const queryOpts = {name: 'clipboard-write', allowWithoutGesture: true};
            await navigator.permissions.query(queryOpts);
            await navigator.clipboard.writeText(clipboardData);
        } catch (error) {
            try {
                const elem = document.createElement('textarea');
                elem.value = clipboardData;
                document.body.appendChild(elem);
                elem.select();
                document.execCommand('copy');
                document.body.removeChild(elem);
            } catch (error) {
                this.growl.show({
                    severity: 'error',
                    summary: 'Warning',
                    detail: 'Unable to copy the data to clipboard'
                });
            }
        }
        if (copySingleCell) {
            this.growl.show({
                severity: 'success',
                summary: '',
                detail: "Value '" + this.state.rowData[focusedCell.rowIndex][focusedCell['column']['colId']] + "' is copied"
            });
        } else {
            const headerText = (this.state.copyHeader) ? 'with Header' : '';
            this.growl.show({
                severity: 'success',
                summary: '',
                detail: selectedRows.length + ' row(s) copied to clipboard ' + headerText
            });
        }
    }

    /**
     * Copy data to/from clipboard
     * @param {*} e
     */
    async clipboardEvent(e) {
        let isError = false;
        if (navigator.appVersion.indexOf("Mac") >= 0) {
            this.isMacOS = true;
        }
        var key = e.which || e.keyCode;
        var ctrl = (e.ctrlKey || e.metaKey) ? true : false;
        if (ctrl && (key === 67 || key === 45)) {//Ctrl+C(Windows & linux) or Cmd+C(Mac) or Ctrl+Insert(windows & linux)
            this.copyToClipboard();
        } else if ((ctrl && key === 86) || (e.shiftKey && key === 45)) {    // Ctrl+V(windows & linux) or Cmd+V or Shift+Insert(windows & linux)
            if (!this.isMacOS) {
                try {
                    var clipText = await this.readClipBoard();
                    await this.setState({clipText: clipText});
                    this.copyFromClipboard();
                } catch (error) {
                    isError = true;
                }
            }

            if (this.isMacOS || isError) {
                this.callBackFunction = this.copyFromClipboard;
                this.onCancel = () => {
                    this.setState({confirmDialogVisible: false, clipText: ''});
                    this.actions = null;
                };
                this.onClose = () => {
                    this.setState({confirmDialogVisible: false, clipText: ''});
                    this.actions = null;
                };
                this.showIcon = false;
                this.dialogType = "confirmation";
                this.dialogHeader = "Paste Copied Data";
                this.dialogMsg = "Paste copied content in the field and click Ok to paste to the selected row(s)";
                this.dialogContent = this.pasteTextField;
                this.actions = [
                    {id: "ok_btn", title: "Ok", callback: this.copyFromClipboard},
                    {id: "cancel_btn", title: "Cancel", className: 'act-btn-cancel', callback: this.onCancel}];
                await this.setState({confirmDialogVisible: true});
                if (document.getElementById("clipTextField")) {
                    document.getElementById("clipTextField").focus();
                }
                if (!this.isMacOS) {
                    setTimeout(this.copyFromClipboard, 1);
                }
            }
        }
    }

    /**
     * Text area component passed to dialog to get the clipboard data from user
     * @returns Component
     */
    pasteTextField() {
        return <textarea id="clipTextField" value={this.state.clipText} style={{width: '100%'}} autoFocus
                         onChange={(e) => {
                             this.copyClipText(e)
                         }} placeholder="Paste your text here..."/>;
    }

    copyClipText(e) {
        this.setState({clipText: e.target.value})
    }

    /**
     * Function to copy the data from clipboard
     */
    async copyFromClipboard() {
        try {
            var selectedRows = this.state.gridApi.getSelectedNodes();
            this.tmpRowData = this.state.rowData;
            let dataRowCount = this.state.totalCount;
            //Read Clipboard Data
            let clipboardData = this.state.clipText ? this.state.clipText : null;
            let selectedRowIndex = 0;
            var focusedCell = this.state.gridApi.getFocusedCell();
            this.isSpecificationValid = true;
            this.isConstraintValid = true;
            if ((selectedRows && selectedRows.length > 0) || focusedCell) {
                const copiedValues = clipboardData.split('\t');
                if (copiedValues.length === 2 && focusedCell && focusedCell.column['colId'] === '0') {
                    if (selectedRows && selectedRows.length > 0) {
                        await selectedRows.map(async selectedRow => {
                            selectedRowIndex = selectedRow.rowIndex;
                            this.tmpRowData[selectedRowIndex]["isDirty"] = true;
                            this.tmpRowData[selectedRowIndex][copiedValues[0]] = await this.getValidCellValue(copiedValues[0], copiedValues[0],
                                _.includes(["true", "false", "FALSE", "TRUE"], copiedValues[1]) ? JSON.parse(_.lowerCase(copiedValues[1])) : _.lowerCase(copiedValues[1]));
                            if (_.includes(this.constraintVariables, copiedValues[0])) {
                                await this.validateSchedulingConstraints(selectedRowIndex, this.tmpRowData[selectedRowIndex]);
                            }
                            if (_.includes(this.strategyVariables, copiedValues[0])) {
                                await this.validateSpecificationsDoc(selectedRowIndex, this.tmpRowData[selectedRowIndex]);
                            }
                        });
                    }
                } else if (copiedValues.length === 2 && focusedCell && focusedCell.column['colId'] !== '0') {
                    if (focusedCell.rowIndex >= 0) {
                        this.tmpRowData[focusedCell.rowIndex][focusedCell.column['colId']] = this.getValidCellValue(focusedCell.column['colId'], copiedValues[0], copiedValues[1]);
                        this.tmpRowData[focusedCell.rowIndex]['isDirty'] = true;
                        if (selectedRows && selectedRows.length === 0) {
                            if (_.includes(this.constraintVariables, focusedCell.column['colId'])) {
                                this.validateSchedulingConstraints(focusedCell.rowIndex, this.tmpRowData[focusedCell.rowIndex]);
                            }
                            if (_.includes(this.strategyVariables, focusedCell.column['colId'])) {
                                this.validateSpecificationsDoc(focusedCell.rowIndex, this.tmpRowData[focusedCell.rowIndex]);
                            }
                        }
                    }
                    if (selectedRows && selectedRows.length > 0) {
                        await selectedRows.map(selectedRow => {
                            selectedRowIndex = selectedRow.rowIndex;
                            this.tmpRowData[selectedRowIndex][focusedCell.column['colId']] = this.getValidCellValue(focusedCell.column['colId'], copiedValues[0], copiedValues[1]);
                            this.tmpRowData[selectedRowIndex]['isDirty'] = true;
                            if (_.includes(this.constraintVariables, focusedCell.column['colId'])) {
                                this.validateSchedulingConstraints(selectedRowIndex, this.tmpRowData[selectedRowIndex]);
                            }
                            if (_.includes(this.strategyVariables, focusedCell.column['colId'])) {
                                this.validateSpecificationsDoc(selectedRowIndex, this.tmpRowData[selectedRowIndex]);
                            }
                            return selectedRow;
                        });
                    }
                } else if (selectedRows && selectedRows.length > 0) {
                    let noOfRowsToPaste = 1;
                    if (clipboardData) {
                        // Get the row index from where the paste option starts
                        selectedRowIndex = selectedRows[0].rowIndex - 1;
                        clipboardData = _.trimEnd(clipboardData);
                        let suGridRowData = this.state.emptyRow;
                        // Get the values of copied rows
                        let suRows = clipboardData.split("\n");
                        // If multiple rows are copied, iteration occurs once and all copied rows will be pasted.
                        // If only one row is copied, iteration occurs as per the no of selected rows.
                        noOfRowsToPaste = suRows.length === 1 ? selectedRows.length : 1;
                        _.range(1, noOfRowsToPaste + 1).forEach(() => {
                            // Paste all copied rows
                            suRows.map(async line => {
                                selectedRowIndex++;
                                suGridRowData = {};
                                suGridRowData['id'] = 0;
                                suGridRowData['isValid'] = true;
                                suGridRowData["isDirty"] = true;
                                if (this.tmpRowData.length <= selectedRowIndex) {
                                    this.tmpRowData.push(this.state.emptyRow);
                                }
                                let colCount = 0;
                                let suRow = line.split("\t");
                                for (const key of this.state.colKeyOrder) {
                                    if (key.endsWith('Beamformers')) {
                                        let cellValue = {};
                                        cellValue['param_0'] = JSON.parse(suRow[colCount]);
                                        suGridRowData[key] = cellValue;
                                    } else {
                                        if (_.includes(["true", "false", "FALSE", "TRUE"], suRow[colCount])) {
                                            suGridRowData[key] = JSON.parse(_.lowerCase(suRow[colCount]));
                                        } else {
                                            suGridRowData[key] = suRow[colCount];
                                        }

                                    }
                                    colCount++;
                                }
                                if (this.tmpRowData[selectedRowIndex].id > 0) {
                                    suGridRowData['id'] = this.tmpRowData[selectedRowIndex].id;
                                }
                                this.tmpRowData[selectedRowIndex] = (suGridRowData);
                                await this.validateSchedulingConstraints(selectedRowIndex, suGridRowData);
                                await this.validateSpecificationsDoc(selectedRowIndex, suGridRowData);
                            });
                        })
                    }
                }

                dataRowCount = selectedRowIndex;
                let emptyRow = this.state.emptyRow;
                let tmpNoOfSU = this.state.noOfSU;
                if (dataRowCount >= tmpNoOfSU) {
                    tmpNoOfSU = dataRowCount;
                    //Create additional empty row at the end
                    for (let i = this.tmpRowData.length; i <= tmpNoOfSU; i++) {
                        this.tmpRowData.push(emptyRow);
                    }
                }
                await this.setState({
                    rowData: this.tmpRowData,
                    noOfSU: this.tmpRowData.length,
                    totalCount: dataRowCount,
                    confirmDialogVisible: false,
                    clipText: "",
                    isDirty: true
                });
                this.actions = null;
                publish('edit-dirty', true);
                this.state.gridApi.setRowData(this.state.rowData);
                this.state.gridApi.redrawRows();
            }
        } catch (err) {
            console.error('Error: ', err);
        }
    }

    getValidCellValue(targetKey, key, value) {
        value = _.replace(value, '\r\n', '');
        if (key === 'stations') {
            return value;
        } else if (key.endsWith('Beamformers')) {
            if (targetKey === 'stations') {
                return JSON.stringify(value);
            } else {
                return JSON.parse(value);
            }
        } else if (_.includes(["true", "false", "FALSE", "TRUE"], value)) {
            return JSON.parse(_.lowerCase(value));
        } else if (_.includes(value, ',') && (key !== 'between' && key !== 'notbetween')) {
            return value.split(',');
        } else {
            return value;
        }
    }

    /**
     * Show warning messgae if any changes not saved when the AG grid reload or cancel the page
     * @param {*} functionName
     */
    showWarning(functionName) {
        this.showIcon = true;
        this.dialogType = "confirmation";
        this.dialogHeader = "Add Multiple Scheduling Unit(s)";
        this.dialogMsg = "Do you want to discard your changes? Your changes to the Scheduling Unit(s) will not be saved.";
        this.dialogContent = "";
        this.dialogHeight = '5em';
        this.actions = [{id: "yes", title: 'Discard', callback: functionName, className: 'act-btn-dispose'},
            {id: "no", title: 'Cancel', className: 'act-btn-cancel', callback: this.close}];
        this.callBackFunction = functionName;
        this.onClose = this.close;
        this.onCancel = this.close;
        this.setState({
            confirmDialogVisible: true,
        });
    }

    /**
     * Reset the top table values
     */
    async resetCommonData() {
        await this.setState({commonRowData: []});
        let tmpData = [this.state.defaultCommonRowData]; 
        let gRowData = {};
        for (const key of _.keys(tmpData[0])) {
            if (key === 'id') {
                gRowData[key] = tmpData[0][key];
            }
            //else if(this.state.hasSameValue) {
            gRowData['gdef_' + key] = tmpData[0][key];
            /*} else {
                gRowData['gdef_'+key] = '';
            }*/
        }
        gRowData['gdef_offset_from_max'] = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.from.maximum;
        gRowData['gdef_offset_from_min'] = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.from.minimum;
        gRowData['gdef_offset_to_max'] = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.to.maximum;
        gRowData['gdef_offset_to_min'] = this.resolvedConstraintSchema.properties.sky.properties.transit_offset.properties.to.minimum;
        await this.setState({commonRowData: [gRowData]});
    }

    /**
     * Reload the data from API
     */
    reload() {
        this.changeStrategy(this.state.observStrategy.id);
    }

    /**
     * Apply the changes to all rows
     */
    async applyToAll() {
        let isNotEmptyRow = true;
        if (!this.state.applyEmptyValue) {
            var row = this.state.commonRowData[0];
            Object.keys(row).forEach(key => {
                if (key !== 'id' && row[key] !== '') {
                    isNotEmptyRow = false;
                }
            });
        }
        if (!this.state.applyEmptyValue && isNotEmptyRow) {
            this.growl.show({
                severity: 'warn',
                summary: 'Warning',
                detail: 'Please enter value in the column(s) above to apply'
            });
        } else {
            this.dialogType = "confirmation";
            this.dialogHeader = "Warning";
            this.showIcon = true;
            this.dialogMsg = "Do you want to apply the above value(s) to all Scheduling Units?";
            this.dialogContent = "";
            this.actions = null;
            this.callBackFunction = this.applyChanges;
            this.applyToAllRow = true;
            this.applyToEmptyRowOnly = false;
            this.onClose = this.close;
            this.onCancel = this.close;
            this.setState({confirmDialogVisible: true});
        }
    }

    /**
     * Apply the changes to selected rows
     */
    async applyToSelected() {
        let isNotEmptyRow = true;
        let tmpRowData = this.state.gridApi.getSelectedRows();
        if (!this.state.applyEmptyValue) {
            var row = this.state.commonRowData[0];
            Object.keys(row).forEach(key => {
                if (key !== 'id' && row[key] !== '') {
                    isNotEmptyRow = false;
                }
            });
        }
        if (!this.state.applyEmptyValue && isNotEmptyRow) {
            this.growl.show({
                severity: 'warn',
                summary: 'Warning',
                detail: 'Please enter value in the column(s) above to apply'
            });
        } else if (tmpRowData && tmpRowData.length === 0) {
            this.growl.show({
                severity: 'warn',
                summary: 'Warning',
                detail: 'Please select at least one row to apply the changes'
            });
        } else {
            this.showIcon = true;
            this.dialogType = "confirmation";
            this.dialogHeader = "Warning";
            this.dialogMsg = "Do you want to apply the above value(s) to all selected Scheduling Unit(s) / row(s)?";
            this.dialogContent = "";
            this.actions = null;
            this.applyToAllRow = false;
            this.applyToEmptyRowOnly = false;
            this.callBackFunction = this.applyChanges;
            this.onClose = this.close;
            this.onCancel = this.close;
            this.setState({confirmDialogVisible: true});
        }
    }

    /**
     * Apply the changes to Empty rows
     */
    async applyToEmptyRows() {
        let isNotEmptyRow = true;
        if (!this.state.applyEmptyValue) {
            var row = this.state.commonRowData[0];
            Object.keys(row).forEach(key => {
                if (key !== 'id' && row[key] !== '') {
                    isNotEmptyRow = false;
                }
            });
        }
        if (!this.state.applyEmptyValue && isNotEmptyRow) {
            this.growl.show({
                severity: 'warn',
                summary: 'Warning',
                detail: 'Please enter value in the column(s) above to apply'
            });
        } else {
            this.showIcon = true;
            this.dialogType = "confirmation";
            this.dialogHeader = "Warning";
            this.dialogMsg = "Do you want to apply the above value(s) to all empty rows?";
            this.dialogContent = "";
            this.actions = null;
            this.applyToEmptyRowOnly = true;    // Allows only empty to make changes
            this.applyToAllRow = true;
            this.callBackFunction = this.applyChanges;
            this.onClose = this.close;
            this.onCancel = this.close;
            this.setState({confirmDialogVisible: true});
        }
    }

    /**
     * Make global changes in table data
     */
    async applyChanges() {
        const currentSortingColumn = this.state.gridApi.getSortModel();
        this.state.gridColumnApi.applyColumnState({
            state: [{colId: '0', sort: 'asc'}],
            defaultState: {sort: true},
        });

        await this.setState({
            confirmDialogVisible: false,
            isDirty: true
        });
        publish('edit-dirty', true);
        let selectedRows = [];
        let gridRows = [];
        this.state.gridApi.forEachNodeAfterFilterAndSort((node) => gridRows.push(node.data));
        if (!this.applyToAllRow) {
            selectedRows = this.state.gridApi.getSelectedRows();
        }
        var grow = _.cloneDeep(this.state.commonRowData[0]);
        delete grow['gdef_id'];
        if (gridRows.length > 0) {
            let tmpRows = (this.applyToAllRow) ? gridRows : selectedRows;
            for (const tmpRow of tmpRows) {
                if (this.applyToEmptyRowOnly && (tmpRow['id'] > 0 || (tmpRow['suname'] !== '' && tmpRow['sudesc'] !== ''))) {
                    continue;
                }
                let row = _.find(gridRows, {'custId': tmpRow.custId});
                if (row) {
                    for (const key of this.colKeyOrder) {
                        let value = grow['gdef_' + key];
                        if ((key === 'suname' || key === 'sudesc') && Validator.isEmpty(value)) {
                            continue;
                        }
                        if (grow.hasOwnProperty(['gdef_' + key])) {
                            if (this.state.applyEmptyValue) {
                                row[key] = value;
                            } else {
                                //Lodash is not checking the number, so append string
                                row[key] = (_.isEmpty(value + '')) ? row.hasOwnProperty([key]) ? row[key] : grow['gdef_' + key] : value;
                            }
                            row["isDirty"] = true;
                        }
                    }

                }
                this.validateSchedulingConstraints((tmpRow.custId - 1), tmpRow);
                this.validateSpecificationsDoc((tmpRow.custId - 1), tmpRow);
            }
            this.state.gridApi.setRowData(gridRows);
            this.state.gridColumnApi.applyColumnState({
                state: currentSortingColumn,
                defaultState: {sort: null},
            });
        }
    }


    /**
     * Callback function for JSON Editor to set the default constraint_doc value.
     * @param {Object} jsonOutput - default constraint values from the editor
     */
    setConstraintsEditorOutput(jsonOutput) {
        if (jsonOutput["sky"] && jsonOutput["sky"]["reference_pointing"]) {
            jsonOutput["sky"]["reference_pointing"]["pointing"]["angle1"] = parseFloat(jsonOutput["sky"]["reference_pointing"]["pointing"]["angle1"]);
            jsonOutput["sky"]["reference_pointing"]["pointing"]["angle2"] = parseFloat(jsonOutput["sky"]["reference_pointing"]["pointing"]["angle2"]);
        }
        this.defaultConstraintDoc = jsonOutput;
    }

    getStrategyFilterOptions(filterValues, filterId) {
        if (filterValues) {
            let optionList = [];
            filterValues.forEach(choice => {
                const tmpVar = {name: choice.value, value: choice.value};
                if (filterId === 'purpose') {
                    optionList.push(tmpVar);
                }
                if (filterId === 'state') {
                    optionList.push(tmpVar);
                }
            });
            return _.sortBy(optionList, ['name']);
        }
    }

    /**
     * Show Constraint Error dialog in UI
     */
    showConstraintError() {
        this.actions = [{id: "cancel_btn", title: "Close", callback: this.close},];
        this.onClose = this.close;
        this.dialogType = "error";
        this.dialogHeader = "Constraint Validation";
        this.dialogMsg = "One or more input for the constraint specification is invalid. See the details below and update them with correct values.";
        this.dialogContent = this.showConstraintErrorDialog;
        this.setState({confirmDialogVisible: true});
    }

    /**
     * Show Specification Error dialog in UI
     */
    showSpecificationError() {
        this.actions = [{id: "cancel_btn", title: "Close", callback: this.close},];
        this.onClose = this.close;
        this.dialogType = "error";
        this.dialogHeader = "Specification Doc Validation";
        this.dialogMsg = "One or more input for the task specification is invalid. See the details below and update them with correct values.";
        this.dialogContent = this.showSpecificationErrorDialog;
        this.setState({confirmDialogVisible: true});
    }

    render() {
        if (this.state.redirect) {
            return <Redirect to={{pathname: this.state.redirect}}></Redirect>
        }
        const {scheduleunit_draft} = this.state.userrole;
        return (
            <React.Fragment>
                {scheduleunit_draft && scheduleunit_draft.list ?
                    <>
                        <Toast ref={(el) => this.growl = el}/>
                        <PageHeader location={this.props.location} title={'Scheduling Unit(s) Add Multiple'}
                                    actions={[{
                                        icon: 'fa-window-close',
                                        title: 'Close',
                                        type: 'button',
                                        actOn: 'click',
                                        props: {callback: this.checkIsDirty}
                                    }]}
                        />
                        {this.state.isFetchingData &&
                            <TopProgressBar/>
                        }
                        {this.state.isLoading ? <AppLoader/> :
                            <>
                                {/* SchedulingConstraint editor to pass the scheduling_constraint schema and get the default
                        constraint_doc for new SUs if not constraint fields are edited */}
                                <div style={{display: "none"}}>
                                    <SchedulingConstraints constraintTemplate={this.constraintSchema} disable
                                                           formatOutput={false}
                                                           callback={this.setConstraintsEditorOutput}/>
                                </div>
                                <div>
                                    <div className="p-fluid">
                                        {<ProjectScheduleSetSelector
                                            selectedProject={this.state.schedulingUnit.project}
                                            projectDisabled={this.state.projectDisabled}
                                            allProjects={this.projects}
                                            projectErrors={this.state.errors.project}
                                            touchedProject={true}
                                            changeProject={this.onProjectChange}
                                            selectedSchedulingSet={this.state.schedulingUnit.scheduling_set_id}
                                            allSchedulingSets={this.state.schedulingSets}
                                            schedulingSetErrors={this.state.errors.scheduling_set_id}
                                            touchedSchedulingSet={true}
                                            setSchedUnitParams={this.setSchedulingSetParams}
                                            scheduleUnitDraft={scheduleunit_draft}
                                            setState={this.setState}
                                        />}
                                        <div className="p-field p-grid">
                                            <label htmlFor="observStrategy" className="col-lg-2 col-md-2 col-sm-12">
                                                Observation Strategy <span style={{color: 'red'}}>*</span>
                                            </label>
                                            {<ObservationStrategySelector
                                                selectedPurpose={this.state.observStrategyFilters.purpose}
                                                allPurposes={this.templatePurposes}
                                                selectedState={this.state.observStrategyFilters.states}
                                                allStates={this.templateState}
                                                selectedObservationStrategy={this.state.observStrategy}
                                                allObservationStrategies={this.state.observStrategies}
                                                observationIdSet={this.state.observationIdSet}
                                                changeStrategy={this.onStrategyChange}
                                                localStoreKey={"strategyFilterSUSetEditor"}
                                                setState={this.setState}
                                            />}


                                            <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                            <label htmlFor="noOfSU" className="col-lg-2 col-md-2 col-sm-12">No of
                                                Scheduling Unit <span style={{color: 'red'}}>*</span></label>
                                            <div className="col-lg-3 col-md-3 col-sm-12">
                                                <Dropdown data-testid="noOfSU" id="noOfSU"
                                                          editable
                                                          options={this.state.noOfSUOptions}
                                                          value={this.state.noOfSU}
                                                          onChange={(e) => this.setNoOfSUint(e.value)}
                                                          tooltip="Enter No. of Scheduling Units, Range - 1 to 500"
                                                          tooltipOptions={this.tooltipOptions}
                                                          placeholder='Enter No. of SU (1 to 500)'/>
                                                <label className={this.state.errors.noOfSU ? "error" : "info"}>
                                                    {this.state.errors.noOfSU ? this.state.errors.noOfSU : "Enter No. of Scheduling Units"}
                                                </label>
                                            </div>
                                        </div>
                                        {this.state.rowData && this.state.rowData.length > 0 &&
                                            <div className="p-field p-grid">
                                                <label htmlFor="observStrategy" className="col-lg-2 col-md-2 col-sm-12">Copy
                                                    Data With Header</label>
                                                <div className="col-lg-3 col-md-3 col-sm-12">
                                                    <Checkbox inputId="csvheader" role="csvheader"
                                                              tooltip="Include column headers while copying the data to clipboard"
                                                              tooltipOptions={this.tooltipOptions}
                                                              checked={this.state.copyHeader}
                                                              onChange={e => this.copyHeader(e.target.checked)}></Checkbox>

                                                    <Button label="Copy Only Header" icon="fas fa-copy"
                                                            onClick={this.copyOnlyHeader}
                                                            style={{marginLeft: '3em', width: '12em'}}
                                                            onAnimationEnd={() => this.setState({fade: false})}
                                                            data-testid="copyHeaderOnly" id="copyHeaderOnly"
                                                            className={this.state.fade ? 'p-button-primary fade' : 'p-button-primary'}
                                                            tooltip="Copy only header to clipboard"
                                                    />
                                                </div>
                                            </div>
                                        }

                                    </div>
                                    <>
                                        {this.state.isAGLoading ? <AppLoader/> :
                                            <>
                                                {this.state.rowData && this.state.rowData.length > 0 &&
                                                    <React.Fragment>
                                                        <Accordion onTabOpen={this.resetCommonData}
                                                                   style={{marginTop: '2em', marginBottom: '2em'}}>
                                                            <AccordionTab header={<React.Fragment><span
                                                                style={{paddingLeft: '0.5em', paddingRight: '0.5em'}}
                                                                data-testid='select-multiple-su-input'>
                                                                Input Values For Multiple Scheduling units</span>
                                                                <i className="fas fa-clone"></i></React.Fragment>}>
                                                                <div className="ag-theme-alpine" style={{
                                                                    overflowX: 'inherit !importent',
                                                                    height: '160px',
                                                                    marginBottom: '10px'
                                                                }} onKeyDown={this.topAGGridEvent}>
                                                                    <AgGridReact
                                                                        suppressClipboardPaste={false}
                                                                        columnDefs={this.state.globalColmunDef}
                                                                        columnTypes={this.state.columnTypes}
                                                                        defaultColDef={this.state.defaultColDef}
                                                                        rowSelection={this.state.rowSelection}
                                                                        onGridReady={this.onTopGridReady}
                                                                        rowData={this.state.commonRowData}
                                                                        frameworkComponents={this.state.frameworkComponents}
                                                                        context={this.state.context}
                                                                        components={this.state.components}
                                                                        modules={this.state.modules}
                                                                        enableRangeSelection={true}
                                                                        tooltipShowDelay={50}
                                                                    >
                                                                    </AgGridReact>

                                                                </div>
                                                                <div className="p-grid p-justify-start">
                                                                    <label htmlFor="observStrategy" className="p-col-1"
                                                                           style={{width: '14em'}}>Include empty
                                                                        value(s)</label>
                                                                    <Checkbox
                                                                        tooltip="Copy the input value ( empty values also ) as it is while apply the changes in table"
                                                                        tooltipOptions={this.tooltipOptions}
                                                                        checked={this.state.applyEmptyValue}
                                                                        onChange={e => this.setState({'applyEmptyValue': e.target.checked})}
                                                                        style={{marginTop: '10px'}}>
                                                                    </Checkbox>

                                                                    <div className="p-col-1"
                                                                         style={{width: 'auto', marginLeft: '2em'}}>
                                                                        <Button label="Apply to All Rows"
                                                                                tooltip="Apply changes to all rows in below table"
                                                                                className="p-button-primary"
                                                                                icon="fas fa-check-double"
                                                                                onClick={this.applyToAll}/>
                                                                    </div>
                                                                    <div className="p-col-1"
                                                                         style={{width: 'auto', marginLeft: '2em'}}>
                                                                        <Button label="Apply to Selected Rows"
                                                                                tooltip="Apply changes to selected row in below table"
                                                                                className="p-button-primary"
                                                                                icon="fas fa-check-square"
                                                                                onClick={this.applyToSelected}/>
                                                                    </div>
                                                                    <div className="p-col-1"
                                                                         style={{width: 'auto', marginLeft: '2em'}}>
                                                                        <Button label="Apply to Empty Rows"
                                                                                tooltip="Apply changes to empty row in below table"
                                                                                className="p-button-primary"
                                                                                icon="pi pi-check"
                                                                                onClick={this.applyToEmptyRows}/>
                                                                    </div>
                                                                    <div className="p-col-1"
                                                                         style={{width: 'auto', marginLeft: '2em'}}>
                                                                        <Button label="Reset"
                                                                                tooltip="Reset input values"
                                                                                className="p-button-primary"
                                                                                icon="pi pi-refresh"
                                                                                onClick={this.resetCommonData}/>
                                                                    </div>
                                                                </div>
                                                            </AccordionTab>
                                                        </Accordion>
                                                    </React.Fragment>
                                                }

                                                {this.state.observStrategy.id &&
                                                    <div className="ag-theme-alpine" style={{
                                                        overflowX: 'inherit !importent',
                                                        height: '500px',
                                                        marginBottom: '3em',
                                                        padding: '0.5em'
                                                    }} onKeyDown={this.clipboardEvent}>
                                                        <label>Scheduling Unit(s) </label>
                                                        <AgGridReact
                                                            suppressClipboardPaste={false}
                                                            columnDefs={this.state.columnDefs}
                                                            columnTypes={this.state.columnTypes}
                                                            defaultColDef={this.state.defaultColDef}
                                                            rowSelection={this.state.rowSelection}
                                                            onGridReady={this.onGridReady}
                                                            rowData={this.state.rowData}
                                                            frameworkComponents={this.state.frameworkComponents}
                                                            context={this.state.context}
                                                            components={this.state.components}
                                                            modules={this.state.modules}
                                                            enableRangeSelection={true}
                                                            enableCellChangeFlash={true}
                                                            onCellValueChanged={this.cellValueChageEvent}
                                                            suppressMultiRangeSelection={true}
                                                            tooltipShowDelay={50}
                                                            suppressScrollOnNewData={true}
                                                        >
                                                        </AgGridReact>
                                                    </div>
                                                }
                                            </>
                                        }
                                    </>
                                    {this.state.isObsoletStrategy &&
                                        <div>
                                            <span style={{color: 'red'}}>* Cannot create new Scheduling Units from obsolete strategy. To edit the existing one, please edit from the details page.</span>
                                        </div>
                                    }
                                    {!this.state.validConstraints && this.state.constraintsValidationMessage.length > 0 &&
                                        <div className="p-grid p-justify-start" style={{marginLeft: '10px'}}>
                                            <label className="error">{this.state.constraintsValidationMessage} </label>
                                            <i className="fa fa-exclamation-circle validationError" aria-hidden="true"
                                               onClick={this.showConstraintError}></i>
                                        </div>
                                    }
                                    {!this.state.validSpecification && this.state.specificationValidationMessage.length > 0 &&
                                        <div className="p-grid p-justify-start" style={{marginLeft: '10px'}}>
                                            <label className="error">{this.state.specificationValidationMessage}</label>
                                            <i className="fa fa-exclamation-circle validationError" aria-hidden="true"
                                               onClick={this.showSpecificationError}></i>
                                        </div>
                                    }
                                    <div className="p-grid p-justify-start act-btn-grp">
                                        <div className="p-col-1">
                                            <Button label="Save" className="p-button-primary" icon="pi pi-check"
                                                    disabled={!(!this.state.isObsoletStrategy && scheduleunit_draft.create && scheduleunit_draft.edit) || this.state.isFetchingData}
                                                    onClick={this.saveSchedulingUnit}
                                                    data-testid="save-btn"/>
                                        </div>
                                        <div className="p-col-1">
                                            <Button label="Cancel" className="act-btn-cancel" icon="pi pi-times"
                                                    onClick={this.checkIsDirty}/>
                                        </div>
                                    </div>
                                </div>
                            </>
                        }
                        {this.state.showAddSet &&
                            <SchedulingSet callbackFunction={this.setSUSet} project={this.state.selectedProject[0]}
                                           onCancel={this.refreshSchedulingSet}/>
                        }
                        <CustomDialog type={this.dialogType} visible={this.state.confirmDialogVisible}
                                      width={this.dialogWidth} height={this.dialogHeight}
                                      header={this.dialogHeader} message={this.dialogMsg}
                                      opacity={this.dialogHeader.startsWith("Paste") ? (this.isMacOS ? 1 : 0) : 1}
                                      content={this.dialogContent} onClose={this.onClose}
                                      onCancel={this.onCancel} onSubmit={this.callBackFunction}
                                      showIcon={this.showIcon} actions={this.actions}>
                        </CustomDialog>
                        <CustomPageSpinner visible={this.state.showSpinner} percentage={this.state.progressPercent}/>
                    </> : <AccessDenied/>}
            </React.Fragment>
        );
    }
}

export default SchedulingSetCreate;
