import { TabView,TabPanel  } from "primereact/tabview";
import PropTypes from 'prop-types';

import UtilService from "../../services/util.service";
import { ListBox } from 'primereact/listbox';
import  {useEffect, useState} from "react";
import  StationGeoView from './StationGeoView'


import PageHeader from '../../layout/components/PageHeader';
// see https://react-leaflet.js.org/docs/api-map/
export default  function StationView(props) { 
    const {
        location
    } = props

    const [stations, setStations] = useState()
    const [stationsGeo, setStationsGeo] = useState([])
    const [selectedStation, setSelectedStation] = useState()

    const [map, setMap] = useState()


    async function fetchStations() {
        let stationlist = await UtilService.getStations();
        let flatstationList = Object.entries(stationlist)
        setStations(flatstationList);
        let geostations = MakeGeoJson(stationlist);
        setStationsGeo(geostations);
    }
    


    function generateGeoJson(station, coordinates) {
        let name = station;
        let longitude = coordinates.longitude;
        let latitude = coordinates.latitude;

        return   {
            type: "Feature",
            properties: {
              name: name
            },
            "geometry": {
              "type": "Point",
              "coordinates": [
                longitude,
                latitude
              ]
            },
            id: name
          }
    }

     function MakeGeoJson(stations) {
        let StationgeojsonArray = [] 
        for (let station in  stations){
            const geostationJson = generateGeoJson(station,stations[station]);
            StationgeojsonArray.push(geostationJson);
        }
        return StationgeojsonArray;
     }

    
     
   function ZoomTo(station)  {
   let longitude = station[1].longitude;
    let latitude = station[1].latitude;
    map?.setView([latitude,longitude],14);
   }
    useEffect ( () => {
        fetchStations();
    }, []);
    

    return    <>

    <PageHeader location={location}  title={'Stations - List '} 
                            />
    
    <TabView>
    <TabPanel header="Station List" className="TabStation">
            <div className="StationListWrap" >
            <ListBox options={stations} optionLabel="0"  className="StationListbox" onChange={(e) => ZoomTo(e.value)}/>
            <StationGeoView stations={stationsGeo }  selected={selectedStation} mapRef={ setMap} ></StationGeoView >
            </div>
    </TabPanel>
    <TabPanel header="Geo Plot">
        <div className="GeoMap">
       <StationGeoView stations={stationsGeo} selected={selectedStation}></StationGeoView >
       </div>
    
    </TabPanel>
    </TabView>
    </>
}

StationView.propTypes = {
    title: PropTypes.string,
  
  };