import React, {Component} from 'react';
import _ from 'lodash';
import TaskService from '../../services/task.service';
import ViewTable from '../../components/ViewTable';
import AppLoader from '../../layout/components/AppLoader';
import { Button } from 'primereact/button';
import UtilService from '../../services/util.service';
import UIConstants from '../../utils/ui.constants';

/**
 * Component that list down the status change logs of subtasks
 */
export class TaskStatusLogs extends Component {

    constructor(props) {
        super(props);
        this.getStoreAttributes();
        this.state = {
            isLoading: true,
            logs: [],
            showLatest: this.taskListAttribute["showLatestLogs"] || false,
            defaultSortColumn: [this.taskListAttribute["statusLogSorting"] || {id: 'Updated At', desc: true}]
        }
        this.setShowLatest = this.setShowLatest.bind(this);
    }

    async componentDidMount() {
        let logs = await TaskService.getTaskStatusLogs(this.props.taskId);
        // logs = _.sortBy(logs, ['subtask_id', 'updated_at']);
        this.setState({logs: logs, isLoading: false});
    }

    /**
     * Function to show latest or all logs and store the option selected for rendering the same when opened again.
     */
    setShowLatest() {
        let showLatest = !this.state.showLatest
        this.taskListAttribute['showLatestLogs'] = showLatest;
        this.setState({showLatest: showLatest});
        this.setStoreAttributes();
    }

    /**
     * Callback function passed to ViewTable component. This function is called from the ViewTable component
     * when sorting on a column and the sorting option is stored in the local storage.
     * @param {Object} sortData 
     */
    toggleBySorting(sortData) {
        this.taskListAttribute["statusLogSorting"] = sortData;
        this.setStoreAttributes();
    }
    
    /**
     * Function to retrieve the stored attributes for this component.
     */
    getStoreAttributes() {
        this.taskListAttribute = UtilService.localStore({ type: 'get', key: "TASK_LIST_UI_ATTR" }) || {};
    }

    /**
     * Function to store the attributes of the component to use on re-initialization
     */
    setStoreAttributes() {
        UtilService.localStore({ type: 'set', key: 'TASK_LIST_UI_ATTR', value: this.taskListAttribute });
    }

    render() {
        let logs = _.sortBy(this.state.logs, ['subtask_id', 'updated_at']);
        if (this.state.showLatest) {
            logs = _(logs).orderBy(['subtask_id', 'updated_at'], ['asc', 'desc']).uniqBy('subtask_id').value();
        }
        return(
            <React.Fragment>
            { this.state.isLoading? <AppLoader /> : 
                <div className="p-grid">
                    <div className="col-lg-12" style={{zIndex:"1"}}>
                        <Button className="dialog-btn" label={this.state.showLatest?"Show All":"Show Latest Only"} 
                            style={{width:"150px"}} 
                            // icon={`pi ${this.state.showLatest?"pi-angle-left":"pi-angle-right"}`} 
                            // iconPos={`${this.state.showLatest?"left":"right"}`}
                            onClick={() => this.setShowLatest()} />
                    </div>
                    <div className="col-lg-12" style={{marginTop:"-30px", zIndex:"0"}}>
                        <ViewTable 
                            data={logs} 
                            defaultcolumns={[{subtask_id: "Subtask Id", subtask_type: "Type", 
                                                updated_at: {name:"Updated At", format:UIConstants.CALENDAR_DATETIME_FORMAT}, 
                                                old_state_value: "From State", new_state_value: "To State", user: 'User'}]} 
                            optionalcolumns={[{}]}
                            columnclassname={[{"Subtask Id": "filter-input-75", "Type": "filter-input-75",
                                                "Updated At": "filter-input-75", "From State": "filter-input-75",
                                                "To State": "filter-input-75", "User": "filter-input-75"}]}
                            defaultSortColumn={this.state.defaultSortColumn}
                            toggleBySorting={(sortData) => {this.toggleBySorting(sortData)}}
                            showAction="false"
                            keyaccessor="id"
                            paths={this.state.paths}
                            showTopTotal={false}
                            showGlobalFilter={false}
                            allowColumnSelection={false}
                        />
                    </div>
                </div>
            }
            </React.Fragment>
        );
    }
}

export default TaskStatusLogs;