import Flatpickr from "react-flatpickr";
import {Button} from "primereact/button";
import {useEffect, useState} from "react";
import JumpToSUId  from "./JumpToSuid";
import moment from "moment";
import UIConstants from "../../../../utils/ui.constants";

import axios from "axios";
import {getStore} from "../../../../services/store.helper";
import { Panel } from 'primereact/panel';

function DateSelector(props) {  //TODO: extract to separate class
    const {
        isRange,
        startTime,
        setStartAndEndTimeCallback
    } = props

    if (!startTime) {
        return null;
    }
    const title = isRange ? "Showing Date Range" : "Showing week from";
    const mode = isRange ? "range" : "single"
    const options = {
        "allowInput": true,
        "enableSeconds": isRange,
        "enableTime": isRange,
        "time_24hr": isRange,
        "mode": mode,
        "minuteIncrement": 1,
        "defaultHour": 0,
        "wrap": true,
        "inlineHideInput": true,
    }
    const resetButtonTitle = isRange ? "Reset to the default date range" : "Reset to the current week"

    return <div data-testid="nav-datetime-selector">
        <label>{title}</label>
        <Flatpickr data-enable-time={isRange}
                   data-input
                   value={startTime.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)}
                   options={options}
                   onClose={value => {
                       const chosenDate = moment(value[0]);
                       const newStartTime = moment(chosenDate.startOf('day'))
                       const newEndTime = moment(chosenDate.add(UIConstants.WEEKVIEW_SHOW_NUMBER_OF_DAYS, 'days').endOf('day'))
                       setStartAndEndTimeCallback(newStartTime, newEndTime)

                   }}>
            <input type="text" data-input className="p-inputtext p-component calendar-input"/>
            <Button className="p-button-icon-only calendar-button"
                    data-testid="nav-reset-button"
                    onClick={() => {
                        const now = moment(moment.now())
                        const newStartTime = moment(now.startOf('day'));
                        const newEndTime = moment(now.add(UIConstants.WEEKVIEW_SHOW_NUMBER_OF_DAYS, 'days').endOf('day'));
                        setStartAndEndTimeCallback(newStartTime, newEndTime)
                    }}
                    title={resetButtonTitle}>
                <i className="fas fa-undo"></i>
            </Button>
        </Flatpickr>
    </div>
}





function changeStartAndEndTimeCallback(setStartTime, setEndTime) {
    return (newStartTime, newEndTime) => {
        setStartTime(newStartTime);
        setEndTime(newEndTime);
    };
}

export default function DateTimeNavigator(props) {
    const {
        startTime,
        setStartTime,
        setEndTime
    } = props


    const [suIdInput, setSUIdInput] = useState()
    const [searchForSUIdTrigger, setSearchForSUIdTrigger] = useState(false)

    useEffect(() => {
        async function fetchStartTimeOfSUId(suId) {
            const shouldUseOnSkyTimes = getStore(UIConstants.STORE_KEY_TIMELINE).onSkyToggle ?? true
            const timeField = shouldUseOnSkyTimes ? "on_sky_start_time" : "process_start_time"
            const response = await axios.get('/api/scheduling_unit_blueprint/' + suId + "?fields=" + timeField);
            const newStartTime = moment(response.data[timeField]).startOf('day')
            const newEndTime = moment(moment(newStartTime).add(UIConstants.WEEKVIEW_SHOW_NUMBER_OF_DAYS, 'days').endOf('day'))
            if (newStartTime.isValid() && newEndTime.isValid()) {
                setStartTime(newStartTime);
                setEndTime(newEndTime);
            } else {
                console.error("Invalid start/end times found when jumping to the SU id's week.")
            }
        }

        if (suIdInput) {
            fetchStartTimeOfSUId(suIdInput).catch(e => console.error("Couldn't fetch the start time for SU id:", suIdInput, e))
        }
    }, [searchForSUIdTrigger])

    return <div className="p-grid timeline-datetime-navigator">
        <Panel header="Navigation">
            <div className="group">
                
                <div className="selector-container">
                    <DateSelector isRange={false} startTime={startTime}
                                  setStartAndEndTimeCallback={changeStartAndEndTimeCallback(setStartTime, setEndTime)}/>
                    <JumpToSUId suId={suIdInput} setSUId={setSUIdInput}
                                setSearchForSUIdTrigger={setSearchForSUIdTrigger}/>
                </div>
            </div>
        </Panel>
    </div>
}