import moment from "moment";
import  DateTimeNavigator from "./DateTimeNavigator";
import {  render } from "@testing-library/react";
import { clickItem, removeReact18ConsoleErrors } from "../../../../utils/test.helper";


removeReact18ConsoleErrors()



describe('DateTimeNavigator', () => {
    const mockSetStartTime = jest.fn();
    const mockSetEndTime = jest.fn();

    it('renders date selector, jump to and week changer correctly', () => {
        const pageContent = render(
            <DateTimeNavigator
                startTime={moment()}
                setStartTime={mockSetStartTime}
                setEndTime={mockSetEndTime}
            />
        );

        const dateSelector = pageContent.getByTestId('nav-datetime-selector');
        const resetButton = pageContent.getByTestId('nav-reset-button');
        const jumpToSUIdButton = pageContent.getByTestId('nav-jump-to-button');

        expect(dateSelector).toBeInTheDocument();
        expect(dateSelector.textContent).toBe("Showing week from");
        expect(resetButton).toBeInTheDocument();
        expect(resetButton.title).toBe("Reset to the current week");
        expect(jumpToSUIdButton).toBeInTheDocument();
        expect(jumpToSUIdButton.title).toBe("Set the week to the scheduling unit's on sky or process start time.")
    });

    it('Clicks buttons; reset, previous+next week', () => {
        const mockStartTime = moment('2023-08-15');
        const pageContent = render(
            <DateTimeNavigator
                startTime={mockStartTime}
                setStartTime={mockSetStartTime}
                setEndTime={mockSetEndTime}
            />
        );

        const resetButton = pageContent.getByTestId('nav-reset-button');
   
        clickItem(resetButton)
        expect(mockSetStartTime).toHaveBeenCalledTimes(1);
        expect(mockSetEndTime).toHaveBeenCalledTimes(1);
    });




    it('does not render week changer or date selector when startTime is not provided', () => {

        const pageContent = render(
            <DateTimeNavigator
                startTime={undefined}
                setStartTime={mockSetStartTime}
                setEndTime={mockSetEndTime}
            />
        );

        expect(pageContent.queryByTestId('nav-datetime-selector')).not.toBeInTheDocument()
        expect(pageContent.queryByTestId('nav-week-changer')).not.toBeInTheDocument()
    });
});