import { useState, useEffect } from 'react';
import { DataTable } from 'primereact/datatable';
import { Column } from 'primereact/column';
import Auth from '../../authenticate/auth.js';
import PermissionStackUtil from '../../authenticate/permission.stack.handler';
const UserOverView = () => {
  const [permissions, setPermissions] = useState([]);

  useEffect(() => {
    const fetchData =  async () => {
      try {
        const user = Auth.getUser();
        const userPermissions = await PermissionStackUtil.getPermissions(user);
        const objectpermissions = Object.entries(userPermissions);

        setPermissions(objectpermissions);
      } catch (error) {
        console.error('Error fetching user permissions:', error);
      }
    };

    fetchData();
  }, []); // The empty dependency array ensures that this effect runs once when the component mounts
  
  const checkMarkTemplate = (fieldvalue) => {
    const classes = fieldvalue===true ? 'pi true-icon pi-check-circle' : '';
    return <i className={classes}></i>;
};





  return (
    <div>
      <DataTable value={permissions} resizableColumns columnResizeMode="expand"  scrollable  showGridlines stripedRows >
        <Column field="0" header="Module" frozen  />
        <Column field="1.create" header="Create" dataType="boolean" body={(rowData ) => checkMarkTemplate(rowData[1].create)} />
        <Column field="1.edit" header="Edit" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].edit)} />
        <Column field="1.delete" header="Delete" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].delete)} />
        <Column field="1.list" header="List" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].list)}  />
        <Column field="1.add" header="Add" dataType="boolean" body={(rowData ) => checkMarkTemplate(rowData[1].add)}  />
        <Column field="1.setting" header="Setting" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].setting)}  />
        <Column field="1.canceltask" header="Cancel Task" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].canceltask)}  />
        <Column field="1.addreservation" header="addreservation" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].addreservation)}  />
        <Column field="1.addsystemevent" header="addsystemevent" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].addsystemevent)}  />
        <Column field="1.listreservation" header="listreservation" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].listreservation)}  />
        <Column field="1.listsystemevent" header="listsystemevent" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].listsystemevent)}  />
        <Column field="1.decide_acceptance" header="decide_acceptance" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].decide_acceptance)}  />
        <Column field="1.pi_verification" header="pi_verification" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].pi_verification)}  />
        <Column field="1.qa_reporting_sos" header="qa_reporting_sos" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].qa_reporting_sos)}  />
        <Column field="1.qa_reporting_to" header="qa_reporting_to" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].qa_reporting_to)}  />
        <Column field="1.unpin_data" header="unpin_data" dataType="boolean"  body={(rowData ) => checkMarkTemplate(rowData[1].unpin_data)}   />
      </DataTable>
    </div>
  );
};

export default UserOverView;
