import  { Component } from 'react';
import { Button } from 'primereact/button';
import { Dialog } from 'primereact/dialog';
import SunEditor from 'suneditor-react';
import { SystemEventListLite } from '../SystemEvent/system.event.listlite';
import { ExistingSystemEventList } from '../SystemEvent/existing.system.event.list';
import 'suneditor/dist/css/suneditor.min.css'; // Import Sun Editor's CSS File
import WorkflowService from '../../services/workflow.service';
import SystemEventService from '../../services/system.event.service';
import { Checkbox } from 'primereact/checkbox';
import QaReportingTemplate from './qa.reporting.template';
import { Link } from 'react-router-dom';
import TaskAssigner from './task.assigner';
import AppLoader from '../../layout/components/AppLoader';
import { SystemEventCreate } from '../SystemEvent';
import _ from 'lodash';
import moment from 'moment';
import { appGrowl } from '../../layout/components/AppGrowl';
import UIConstants from '../../utils/ui.constants';
import AuthStore from '../../authenticate/auth.store';
import AuthUtil from '../../utils/auth.util';
import { CustomDialog } from '../../layout/components/CustomDialog';
import UnitConverter from '../../utils/unit.converter';

//import katex from 'katex' // for mathematical operations on sun editor this component should be added
//import 'katex/dist/katex.min.css'

class QAreporting extends Component{
    SYSTEM_EVENT_LIST_TABLE_NAME = 'system_event_list';
    constructor(props) {
        super(props);
        this.state={            
            content: '',
            assignTo: '',
            currentWorkflowTask:{},
            operator_accept: true,
            templateLoading: true,
            showDialog: false,
            dialogTitle: 'System Event - Add',
            validForm: false,
            userrole: AuthStore.getState(),
            schedulingUnit: null,
            isDirty: false,
            currentTasks: [],
            affectedTasks: []
        };
        this.selectedRows = [];
        this.systemEvent = {};
        this.tmpSystemEvent = {};                                           // Tp pass as parameter to System Create
        this.seAffectedHardwareTemplate = {};
        this.paramsOutput = {};
        this.affectedTaskChips = [];     

        this.user =JSON.parse(localStorage.getItem("user"));
        this.Next = this.Next.bind(this);
        this.handleChange = this.handleChange.bind(this);
        this.assignTaskCB = this.assignTaskCB.bind(this);
        this.saveSystemEvent = this.saveSystemEvent.bind(this);
        this.showSystemEventDialog = this.showSystemEventDialog.bind(this);
        this.synchSystemEvent = this.synchSystemEvent.bind(this);
        this.cancelCreate = this.cancelCreate.bind(this);
        this.checkIsDirty = this.checkIsDirty.bind(this);
        this.close = this.close.bind(this);
        this.showSystemEventListDialog = this.showSystemEventListDialog.bind(this);
        this.onRowSelection = this.onRowSelection.bind(this);
        this.showAddTasksDialog = this.showAddTasksDialog.bind(this);
        this.addTaskToSystemEvent = this.addTaskToSystemEvent.bind(this);
    }

    async componentDidMount() {
        const promises = [  
            AuthUtil.getUserRolePermission()
        ];
        this.tmpSystemEvent = {};
        this.systemEvent = {};
        await Promise.all(promises).then(responses => {
            this.setState({userrole: responses[0]});
        });
        if (this.props.schedulingUnit) {
            let taskId = [];
            this.props.schedulingUnit.task_blueprints && this.props.schedulingUnit.task_blueprints.forEach(task => {
                if (task.name.toLowerCase().includes('observation') || task.name.toLowerCase().includes('pipeline')) {
                    taskId.push(task.id);
                }
            })
            let tempDuration = '';
            let timeDiff = Math.abs(moment(this.props.schedulingUnit.on_sky_start_time) - moment(this.props.schedulingUnit.on_sky_stop_time)) / 1000;
            tempDuration = UnitConverter.getSecsToDDHHmmss(timeDiff); 
            this.tmpSystemEvent = {
                'start': this.props.schedulingUnit.on_sky_start_time,
                'stop': this.props.schedulingUnit.on_sky_stop_time,
                'tasksId': taskId.sort().join(),
                'duration': tempDuration
            }
        }
        if (this.props.readOnly) {
            const QAreportingresponse = await WorkflowService.getQAReportingTo(this.props.process.qa_reporting_to);
            this.setState({
                schedulingUnit: this.props.schedulingUnit,
                content: QAreportingresponse.operator_report,
                operator_accept: QAreportingresponse.operator_accept,
                assignTo: this.props.workflowTask?this.props.workflowTask.owner:''     // Temporary value. This should be taken from ProcessTask owner.
            });
        }   else {
            const defaultTemplate = await QaReportingTemplate.getReportTemplate(this.props.process.su)
            const currentWorkflowTask = await this.props.getCurrentTaskDetails();
            const processPermission = await this.props.processPermission();
            this.setState({ schedulingUnit: this.props.schedulingUnit,
                            defaultTemplate: defaultTemplate, templateLoading: false,
                            assignTo: currentWorkflowTask.owner_id, processPermission: processPermission, 
                            currentWorkflowTask: currentWorkflowTask});
        }
    }

    /**
     * Method will trigger on click next buton
     * here onNext props coming from parent, where will handle redirection to other page
     */
     async Next() {
        let currentWorkflowTask = await this.props.getCurrentTaskDetails();
        const promise = [];
        promise.push(WorkflowService.updateQA_Perform(this.props.id, {"operator_report": this.state.content, "operator_accept": this.state.operator_accept}));
        Promise.all(promise).then(async(responses) => {
            if (responses.indexOf(null)<0) {
                // After completing the current task, get the next task triggered in the workflow and assign project role user for the next task.
                currentWorkflowTask = await this.props.getCurrentTaskDetails();
                if (currentWorkflowTask && !currentWorkflowTask.owner_id) {
                    await WorkflowService.updateAssignTo(currentWorkflowTask.id, "project_role=friend_of_project", { owner: this.state.assignTo });
                }
                this.props.onNext({ report: this.state.content });
            }   else {
                this.props.onError('error', 'Unable to proceed', 'Please clear your browser cookies and try again');
            }
        });
        this.props.callback();
    }

    /**
     * Method will trigger on change of operator report sun-editor
     */
    handleChange(e) {
        if (e === '<p><br></p>') {
            this.setState({ content: '' });
            return;
        }
        this.setState({ content: e });
    }

    /**
     * Callback function passed to the Task Assigner and is called after assigning or reassigning task owner.
     */
    async assignTaskCB() {
        let currentWorkflowTask = await this.props.getCurrentTaskDetails();
        this.setState({currentWorkflowTask: currentWorkflowTask, 
                        assignTo: currentWorkflowTask.owner_id, reassign: false});
    }

     /**
     * Show create system event dialog
     */
    showSystemEventDialog() {
        this.setState({
            showDialog: true,
        });
    }

    /**
     * Show system event list dialog
     */
    showSystemEventListDialog() {
        this.setState({
            showExistingSEDialog: true,
        });
    }

    /**
     * Close the create system event dialog
     */

    doCancel() {
        this.setState({showDialog: false});
    }

    /**
     * Synch system event details from child page
     * @param {*} systemevent 
     * @param {*} seAffectedHardwareTemplate 
     * @param {*} paramsOutput 
     * @param {*} validForm 
     * @param {*} isDirty 
     */
    synchSystemEvent(systemevent, seAffectedHardwareTemplate, paramsOutput, validForm, affectedTaskChips, isDirty) {
        this.systemEvent = systemevent;
        this.seAffectedHardwareTemplate = seAffectedHardwareTemplate;
        this.paramsOutput = paramsOutput;
        this.affectedTaskChips = affectedTaskChips;
        this.setState({validForm: validForm, isDirty: isDirty});
    }

    /**
     * Add new system event for current SUB
    */
    async saveSystemEvent() {
        let systemevent =this.systemEvent;
        systemevent['start'] = moment(systemevent['start']).format(UIConstants.CALENDAR_DATETIME_FORMAT);
        systemevent['stop'] = systemevent['stop'] ? moment(systemevent['stop_time']).format(UIConstants.CALENDAR_DATETIME_FORMAT) : null;
        systemevent['affected_hardware_template'] = this.seAffectedHardwareTemplate.url;
        systemevent['affected_hardware_doc'] = this.paramsOutput;
        let affected_tasks = [];
        if (this.affectedTaskChips) {
            for (const selectedTask of this.affectedTaskChips) {
                const leftIndex = selectedTask.props.children.indexOf("(");
                const rightIndex = selectedTask.props.children.indexOf(")");
                let id = selectedTask.props.children.substring(leftIndex+1, rightIndex);
                affected_tasks.push(`/api/task_blueprint/${id}`);
            }
        }
        systemevent['affected_tasks'] = affected_tasks;
        systemevent['jira_url'] = systemevent.jira_url? systemevent.jira_url.trim().length === 0? null:systemevent.jira_url:null;
        systemevent['notes'] = systemevent.notes? systemevent.notes.trim().length === 0? null:systemevent.notes:null;
        systemevent = await SystemEventService.saveSystemEvent(systemevent);
        if (systemevent && systemevent.id) {
            await this.setState({systemevent: {}, showDialog: false, isDirty: false });
            appGrowl.show({severity: 'success', summary: 'Success', detail: 'System Event is created successfully'});
        }   else {
            this.setState({ systemevent: {}, showDialog: false, isDirty: false })
            appGrowl.show({severity: 'error', summary: 'Error', detail: 'Unable to add new System Event'});
        }
        this.refreshSystemEventlist();
    }

    /**
     * Call function in child (system Event List) component
     */
    refreshSystemEventlist = () => {
        this.child.handleParentAction();
   } 
    /**
    * Cancel System Event creation and redirect
    */
     cancelCreate() {       
        this.setState({ showDialog: false, showCustomDialog: false, showExistingSEDialog: false, isDirty: false });
    }

    /**
     * Warn before cancel the page if any changes detected 
     */
    checkIsDirty() {
        if (this.state.isDirty) {
            this.setState({ showCustomDialog: true, showExistingSEDialog: false });
        } else {
            this.cancelCreate();
        }
    }
    
    close() {
        this.setState({showCustomDialog: false, showExistingSEDialog: false});
    }

    /**
     * When the row selected/unselected in Existing System Event list page it will update.
     * @param {Row ID-JSON Object array} selectedRows 
     */
    onRowSelection(selectedRows) {
        this.selectedRows = selectedRows;
    }

    /**
     * call child function to show picklist
     */
    showAddTasksDialog() {
        this.existingSEChild.showAddTasksDialog();
    }     

    /**
     * Update the System Event
     */
    async addTaskToSystemEvent() {
        let systemEvent = _.cloneDeep(this.state.systemevent);
        let affected_tasks = systemEvent['affected_tasks'];
        for (const affectedTask of this.state.affectedTasks) {
            affected_tasks.push(affectedTask.url);
        }
        systemEvent['stop'] = (systemEvent['stop'] && systemEvent['stop'] !== 'Invalid date' && systemEvent['stop'] !== 'Unknown') ? moment(systemEvent['stop']).format(UIConstants.CALENDAR_DATETIME_FORMAT) : null;
        systemEvent['affected_tasks'] = affected_tasks;
        systemEvent['jira_url'] = systemEvent['jira_url'].props['href'];
        systemEvent = await SystemEventService.updateSystemEvent(systemEvent);
        if (systemEvent && systemEvent.id) {
            appGrowl.show({ severity: 'success', summary: 'Success', detail: 'System Event updated successfully.' });
        } else {
            appGrowl.show({ severity: 'error', summary: 'Error Occured', detail: 'Unable to update System Event'});
        }
        this.setState({affectedTasks:[] });
    }

    render() {
        const permissions = this.state.userrole.userRolePermission.systemevent;
        return (
        <>
           <div>
                <div className={`p-fluid-grid workflow-wrap`}>
                    <div className="p-field p-grid workflow-wrap" style={{ paddingLeft: '-10px' }}>
                        <label htmlFor="assignTo" className="col-lg-2 col-md-2 col-sm-12">Assign To</label>
                        <div className="col-lg-3 col-md-3 col-sm-12" data-testid="assignTo" >
                        {/* Display the assigned owner of the task */}
                        {this.state.assignTo && 
                            <>{!this.props.readOnly ? this.state.currentWorkflowTask.owner_username: this.props.workflowTask.owner_username } 
                                {/* Allow to edit the owner if the task is not yet completed */}
                                {!this.props.readOnly && !this.state.reassign &&
                                    <>
                                        {this.state.currentWorkflowTask.editPermissions
                                        ? <Link onClick={()=> this.setState({reassign: true})} style={{marginLeft: '10px'}}>
                                            <i className="pi pi-pencil"></i>
                                        </Link>
                                        : <Link style={{ cursor: 'default', pointerEvents: "none", color: 'grey', marginLeft: '10px'}} onClick={() => this.setState({reassign: true})}>
                                            <i className="pi pi-pencil"></i>
                                        </Link>
                                        }
                                    </>       
                                }
                            </>
                        }
                        {/* Display the task assigner if owner is not assigned or to reassign to other user */}
                        {(!this.state.assignTo || this.state.reassign) && this.state.currentWorkflowTask &&
                            <TaskAssigner currentWorkflowTask={this.state.currentWorkflowTask} disableAssignToMe = {this.state.currentWorkflowTask.owner_username === this.user.name? true: false}
                                    projectRoles={this.props.projectRoles} callback={this.assignTaskCB} />
                        }
                        </div>
                    </div>
                    <div className="p-field p-grid workflow-wrap " style={{ padding: '12px', position:'relative' }}>
                        <label htmlFor="comments" >Comments {(!this.state.showEditor && !this.props.readOnly) && 
                            <>
                            <span style={{color:'red'}}>*</span>
                            </>}
                        </label>
                        <div className="col-lg-12 col-md-12 col-sm-12"></div>
                        {!this.props.readOnly && 
                        <>
                            {this.state.templateLoading?<AppLoader />:
                            <SunEditor setDefaultStyle="min-height: 250px; height: auto;" enableToolbar={true}  disabled={this.props.readOnly}
                                onChange={ this.handleChange }
                                setContents={this.state.defaultTemplate}
                                setOptions={{
                                    buttonList: [
                                        ['undo', 'redo', 'bold', 'underline', 'fontColor', 'table', 'link', 'image', 'video', 'italic', 'strike', 'subscript',
                                            'superscript', 'outdent', 'indent', 'fullScreen', 'showBlocks', 'codeView', 'preview', 'print', 'removeFormat']
                                    ]
                                }} />
                            }
                        </>
                        }

                        {!this.state.showEditor && <div onClick={() => !this.props.readOnly && this.setState({ showEditor: !this.state.showEditor })} className="operator-report"></div>}
                        {this.props.readOnly &&<div className="comment-editor-content" dangerouslySetInnerHTML={{ __html: this.state.content }}></div>}
                    </div>
                    <div style={{ marginLeft: '1em', position:'relative' }}>
                        <div  className="p-field p-grid workflow-wrap">
                            <label htmlFor="systemissues" >System Events</label>
                            <div style={{marginLeft: '1em'}}>
                                <Button
                                    style={{height: '2em'}}
                                    tooltip="Add System Event"                       
                                    className="p-button-primary" 
                                    icon="pi pi-plus" 
                                    onClick={this.showSystemEventDialog} 
                                    title = {permissions.create && this.props.showAddSystemEvent ?'Add System Event'
                                    : (permissions.create && !this.props.showAddSystemEvent)?" Workflow step (QA Reporting TO) is already completed, at this stage can't add System Event"
                                    : "Don't have permission to add System Event"}
                                    disabled = {!(permissions.create && this.props.showAddSystemEvent)} />
                            </div>
                            <div style={{marginLeft: '1em'}}>
                                <Button
                                    style={{height: '2em'}}
                                    tooltip="Search and add an exisitng System Event"                       
                                    className="p-button-primary" 
                                    icon="pi pi-search-plus"
                                    onClick={this.showSystemEventListDialog} 
                                    title = {permissions.create && this.props.showAddSystemEvent ?'Search and add an exisitng System Event'
                                    : (permissions.create && !this.props.showAddSystemEvent)?" Workflow step (QA Reporting TO) is already completed, at this stage can't add/update System Event"
                                    : "Don't have permission to add/update System Event"}
                                    disabled = {!(permissions.create && this.props.showAddSystemEvent)} />
                            </div>                   
                        </div>
                    </div> 
                    {this.state.schedulingUnit &&
                        <div className="p-col-12" style={{padding: '0.75em'}}>
                            <SystemEventListLite schedulingUnit={this.state.schedulingUnit} ref={(cd) => this.child = cd} SYSTEM_EVENT_LIST_TABLE_NAME = {this.SYSTEM_EVENT_LIST_TABLE_NAME} />
                        </div>
                    }
                    <div className="p-col-12" style={{width: "100%"}}>
                        <Checkbox disabled={this.props.readOnly} inputId="operator_accept" onChange={e => this.setState({operator_accept: e.checked})} checked={this.state.operator_accept}></Checkbox>
                        <label htmlFor="operator_accept " style={{paddingLeft:"5px"}}>The data quality adheres to policy (Operator evaluation)</label>
                    </div> 
                </div>
                {!this.props.readOnly && <div className="p-grid p-justify-start">
                    <div className="btn-bar">
                    <Button disabled={this.state.content && this.state.processPermission && this.state.currentWorkflowTask.editPermissions && this.state.currentWorkflowTask.owner_username === this.user.name? false: true } 
                    label="Next" className="p-button-primary" icon="pi pi-check" onClick={ this.Next } />
                    </div>
                    <div className="btn-bar">
                        <Button label="Cancel" className="act-btn-cancel" icon="pi pi-times"  style={{ width : '88px' }} 
                                    onClick={() => { this.props.onCancel()}} />
                    </div>
                </div>}
                {!this.props.readOnly && 
                    <>
                        {this.state.currentWorkflowTask.editPermissions && this.state.currentWorkflowTask.owner_username === this.user.name ? 
                        <span></span>: <span style={{color: 'red'}}>* You can only save and proceed next if you are assigned to this workflow step</span>}
                    </>
                }
            </div>

            <Dialog header={_.startCase(this.state.dialogTitle)} style={{width: '70vw'}} visible={this.state.showDialog} 
                maximizable maximized={false} onHide={this.checkIsDirty} inputId="confirm_dialog"
                footer={<div>
                        <Button  label="Save" icon="pi pi-check"  onClick={() => {this.saveSystemEvent()}} style={{width: '6em'}} disabled={!this.state.validForm} />
                        <Button className="act-btn-cancel" icon="pi pi-times" label="Cancel" onClick={this.checkIsDirty} />
                    </div>
                } 
                >
                <div className="ag-theme-balham" style={{ height: '75vh', width: '99%'}}>
                    <div className="p-fluid">
                        <div className="p-grid">
                            <div className="p-col-12" style={{width: "100%"}}>
                                 <SystemEventCreate systemevent={this.tmpSystemEvent} dataSynchCallBack={this.synchSystemEvent} schedulingUnit={this.state.schedulingUnit} />
                            </div>
                        </div>
                    </div>
                </div>
            </Dialog>

            <CustomDialog type="confirmation" visible={this.state.showCustomDialog} width="40vw"
                header={'Add System Event'} message={'Do you want to discard your changes? Your changes may not be saved..'}
                content={''} onClose={this.close} 
                actions={ [ {id:"yes", title: 'Discard', callback: this.cancelCreate, className:'act-btn-dispose' },
                                    {id:"no", title: 'Cancel', className:'act-btn-cancel', callback: this.close} ]} > 
            </CustomDialog>
            {this.state.showExistingSEDialog && 
                <Dialog header={"Existing System Events"} style={{width: '70vw'}} visible={this.state.showExistingSEDialog} 
                    maximizable maximized={false} onHide={this.close} inputId="confirm_dialog"
                    footer={<div>
                            <Button  label="Add Tasks" icon="pi pi-check"  onClick={() => {this.showAddTasksDialog()}} style={{width: '9em'}} />
                            <Button className="act-btn-cancel" icon="pi pi-times" label="Close" onClick={this.close} />
                        </div>
                    } 
                    >
                    <div className="ag-theme-balham" style={{ height: '75vh', width: '99%'}}>
                        <div className="p-fluid">
                            <div className="p-grid">
                                <div className="p-col-12" style={{width: "100%"}}>
                                    <ExistingSystemEventList ref={(cd) => this.existingSEChild = cd} onRowSelection={this.onRowSelection} schedulingUnit={this.props.schedulingUnit} refreshSystemList={this.refreshSystemEventlist}/>
                                </div>
                            </div>
                        </div>
                    </div>
                </Dialog> 
            }
        </>
    )
}                  
}
export default QAreporting;