import axios from "axios"

const SystemEventService = {
    getSEAffectedHardwareTemplates: async function () {
        try {
            const url = `/api/system_event_template`;
            const response = await axios.get(url);
            return response.data.results;
        } catch (error) {
            console.error('[system.event.services.getSEAffectedHardwareTemplates]',error);
        }
    },
    getSystemEventTemplate: async function(templateId) {
        try {
          const response = await axios.get('/api/system_event_template/' + templateId);
          return response.data;
        } catch (error) {
            console.error('[system.event.services.getSystemEventTemplate]',error);
        }
      },
    getSEIssueTypes: async function () {
        try {
            const url = `/api/system_event_type`;
            const response = await axios.get(url);
            return response.data.results;
        } catch (error) {
            console.error('[system.event.services.getSEIssueTypes]',error);
        }
    },
    getSEIssueSubTypes: async function () {
        try {
            const url = `/api/system_event_subtype`;
            const response = await axios.get(url);
            return response.data.results;
        } catch (error) {
            console.error('[system.event.services.getSEIssueSubTypes]',error);
        }
    },
    getSESeverity: async function () {
        try {
            const url = `/api/system_event_severity`;
            const response = await axios.get(url);
            return response.data.results;
        } catch (error) {
            console.error('[system.event.services.getSESeverity]',error);
        }
    },
    getSEStatus: async function () {
        try {
            const url = `/api/system_event_status`;
            const response = await axios.get(url);
            return response.data.results;
        } catch (error) {
            console.error('[system.event.services.getSEStatus]',error);
        }
    },
    saveSystemEvent: async function (systemevent) {
        try {
            const response = await axios.post(('/api/system_event/'), systemevent);
            return response.data;
          } catch (error) {
            console.error('[system.event.services.saveSystemEvent]',error);
            return null;
          }
    },
    updateSystemEvent: async function (systemevent) {
        try {
            const response = await axios.put((`/api/system_event/${systemevent.id}/`), systemevent);
            return response.data;
          } catch (error) {
            console.error('[system.event.services.updateSystemEvent]',error);
            return null;
          }
    },
    getSystemEventWithFilter: async function (filters, orderBy, limit, offset){
        let response = null;
        try {
            //Same function used for server side filter as well client side filter, so if client side filter then set " limit='' "
            if (limit === '') {
                response = await axios.get('/api/system_event/?limit=1');
                if (response.data.count > response.data.results.length) {
                    limit = response.data.count;
                }   else {
                    limit = response.data.results.length;
                }
            }
            response = null;
            let api = `/api/system_event/?`;
            api += (filters === '')? '' : filters+'&';
            api += (orderBy === '')? '' : orderBy+'&';
            api += 'limit='+limit+'&offset='
            api += (offset === '') ? 0 : offset;
            response = await axios.get(api);
        }   catch(error) {
            console.error('[system.event.services.getSystemEventWithFilter]',error);
        }
        return response;
    },
    getSystemEventFilterDefinition: async function () {
        let res = [];
        try {
            res = await axios.options(`/api/system_event/`);
        }   catch(error) {
            console.error('[system.event.services.getSystemEventFilterDefinition]',error);
        }
        return res;
    },
    getSystemEvent: async function (id) {
        try {
            const response =await axios.get(`/api/system_event/${id}`);
            return response.data;
          } catch (error) {
            console.error('[system.event.services.getSystemEvent]',error);
            return null;
          }
    }
}
export default SystemEventService;
